@@ -1,10 +1,10 @@
 --
---  $Id: sioc_ab.sql,v 1.33.2.12 2010/12/07 10:59:56 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2007 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -117,7 +117,10 @@ create procedure fill_ods_addressbook_sioc2 (
   declare graph_iri, addressbook_iri, socialnetwork_iri, contact_iri, creator_iri, role_iri, iri varchar;
 
   {
+    fill_ods_addressbook_services ();
+
     for (select WAI_ID,
+                WAI_IS_PUBLIC,
                 WAI_TYPE_NAME,
                 WAI_NAME,
                 WAI_ACL
@@ -127,7 +130,7 @@ create procedure fill_ods_addressbook_sioc2 (
     {
       graph_iri := SIOC..acl_graph (WAI_TYPE_NAME, WAI_NAME);
       exec (sprintf ('sparql clear graph <%s>', graph_iri));
-      SIOC..wa_instance_acl_insert (WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
+      SIOC..wa_instance_acl_insert (WAI_IS_PUBLIC, WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
       for (select P_DOMAIN_ID, P_ID, P_ACL
              from AB.WA.PERSONS
             where P_DOMAIN_ID = WAI_ID and P_ACL is not null) do
@@ -201,7 +204,8 @@ create procedure fill_ods_addressbook_sioc2 (
 								P_TAGS,
 	              P_FOAF,
                 P_IRI,
-                P_ACL
+                P_ACL,
+                P_CERTIFICATE
            from DB.DBA.WA_INSTANCE,
                 DB.DBA.WA_MEMBER,
                 AB.WA.PERSONS
@@ -213,7 +217,7 @@ create procedure fill_ods_addressbook_sioc2 (
           order by P_ID) do
   {
       contact_iri := SIOC..addressbook_contact_iri (P_DOMAIN_ID, P_ID);
-      graph_iri := SIOC..get_graph_new (null, coalesce (_access_mode, WAI_IS_PUBLIC), contact_iri);
+      graph_iri := SIOC..get_graph_new (coalesce (_access_mode, WAI_IS_PUBLIC), contact_iri);
       addressbook_iri := addressbook_iri (WAI_NAME);
       socialnetwork_iri := socialnetwork_iri (WAI_NAME);
     creator_iri := user_iri (WAM_USER);
@@ -271,7 +275,8 @@ create procedure fill_ods_addressbook_sioc2 (
                     P_UPDATED,
 											P_TAGS,
 		                  P_FOAF,
-				  P_IRI);
+                      P_IRI,
+                      P_CERTIFICATE);
 
       cnt := cnt + 1;
 		   if (mod (cnt, 500) = 0)
@@ -287,7 +292,34 @@ create procedure fill_ods_addressbook_sioc2 (
 
 -------------------------------------------------------------------------------
 --
-create procedure clean_ods_addressbook_sioc2 (
+create procedure fill_ods_addressbook_services ()
+{
+  declare graph_iri, services_iri, service_iri, service_url varchar;
+  declare svc_functions any;
+
+  graph_iri := get_graph ();
+
+  -- instance
+  svc_functions := vector ('addressbook.search', 'addressbook.new', 'addressbook.import', 'addressbook.export', 'addressbook.publication.new', 'addressbook.subscription.new', 'addressbook.options.set',  'addressbook.options.get');
+  ods_object_services (graph_iri, 'addressbook', 'ODS AddressBook instance services', svc_functions);
+
+  -- contact
+  svc_functions := vector ('addressbook.get', 'addressbook.edit', 'addressbook.delete', 'addressbook.comment.new', 'addressbook.relationship.new', 'addressbook.relationship.delete', 'addressbook.annotation.new');
+  ods_object_services (graph_iri, 'addressbook/contact', 'ODS AddressBook contact services', svc_functions);
+
+  -- contact comment
+  svc_functions := vector ('addressbook.comment.get', 'addressbook.comment.delete');
+  ods_object_services (graph_iri, 'addressbook/contact/comment', 'ODS AddressBook comment services', svc_functions);
+
+  -- contact annotation
+  svc_functions := vector ('addressbook.annotation.get', 'addressbook.annotation.claim', 'addressbook.annotation.delete');
+  ods_object_services (graph_iri, 'addressbook/contact/annotation', 'ODS AddressBook annotation services', svc_functions);
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create procedure clean_ods_addressbook_sioc (
   in _wai_name varchar := null,
   in _access_mode integer := null)
 {
@@ -324,7 +356,7 @@ create procedure clean_ods_addressbook_sioc2 (
           order by P_ID) do
     {
       contact_iri := SIOC..addressbook_contact_iri (P_DOMAIN_ID, P_ID);
-      graph_iri := SIOC..get_graph_new (null, coalesce (_access_mode, WAI_IS_PUBLIC), contact_iri);
+      graph_iri := SIOC..get_graph_new (coalesce (_access_mode, WAI_IS_PUBLIC), contact_iri);
 
       contact_delete (graph_iri,
                       P_DOMAIN_ID,
@@ -344,22 +376,6 @@ create procedure clean_ods_addressbook_sioc2 (
 }
 ;
 
--------------------------------------------------------------------------------
---
-create procedure ods_addressbook_services (
-  in graph_iri varchar, 
-  in forum_iri varchar,
-  in wai_id varchar := null,
-  in wai_name varchar := null)
-{
-  declare svc_iri varchar;
-  
-  -- dbg_obj_print (now (), 'ods_addressbook_services');
-  svc_iri := sprintf ('http://%s%s/services/addressbook', get_cname(), get_base_path ());
-  ods_sioc_service (graph_iri, svc_iri, forum_iri, null, 'text/xml', svc_iri||'/services.wsdl', svc_iri, 'SOAP');
-}
-;
-
 -------------------------------------------------------------------------------
 --
 create procedure contact_insert (
@@ -415,10 +431,12 @@ create procedure contact_insert (
   inout updated datetime,
 	inout tags varchar,
 	inout foaf varchar,
-	inout ext_iri varchar)
+  inout ext_iri varchar,
+  inout certificate varchar)
 {
   declare iri, iri2, temp_iri varchar;
 	declare person_iri varchar;
+  declare info, modulus, exponent, certificate_iri any;
 
 	declare exit handler for sqlstate '*'
 	{
@@ -439,7 +457,7 @@ create procedure contact_insert (
             and WAI_IS_PUBLIC > 0) do
   {
       iri := addressbook_contact_iri (domain_id, contact_id);
-      graph_iri := SIOC..get_graph_new (domain_id, WAI_IS_PUBLIC, iri);
+      graph_iri := SIOC..get_graph_new (WAI_IS_PUBLIC, iri);
       addressbook_iri := addressbook_iri (WAI_NAME);
       socialnetwork_iri := socialnetwork_iri (WAI_NAME);
     creator_iri := user_iri (WAM_USER);
@@ -649,6 +667,24 @@ create procedure contact_insert (
         DB.DBA.ODS_QUAD_URI_L (graph_iri, temp_iri, vcard_iri ('Country'), bCountry);
     }
   }
+
+  -- certificate
+  info := get_certificate_info (9, cast (certificate as varchar), 0);
+  if (info is not null and isarray (info) and cast (info[0] as varchar) = 'RSAPublicKey')
+  {
+    modulus := info[2];
+    exponent := info[1];
+    certificate_iri := iri || '#cert';
+    DB.DBA.ODS_QUAD_URI (graph_iri, certificate_iri, cert_iri ('identity'), iri);
+    DB.DBA.ODS_QUAD_URI (graph_iri, certificate_iri, rdf_iri ('type'), rsa_iri ('RSAPublicKey'));
+
+    DB.DBA.ODS_QUAD_URI_L_TYPED (graph_iri, certificate_iri, rsa_iri ('modulus'), bin2hex (modulus), cert_iri ('hex'), null);
+    DB.DBA.ODS_QUAD_URI_L_TYPED (graph_iri, certificate_iri, rsa_iri ('public_exponent'), cast (exponent as varchar), cert_iri ('int'), null);
+  }
+
+  -- contact services
+  SIOC..ods_object_services_attach (graph_iri, iri2, 'addressbook/contact');
+
   SIOC..contact_comments_insert (graph_iri, addressbook_iri, domain_id, contact_id);
   SIOC..contact_annotations_insert (graph_iri, domain_id, contact_id);
 }
@@ -671,7 +707,7 @@ create procedure contact_delete (
   iri := SIOC..addressbook_contact_iri (domain_id, contact_id);
   if (isnull (graph_iri))
   {
-    graph_iri := SIOC..get_graph_new (domain_id, null, iri);
+    graph_iri := SIOC..get_graph_new (AB.WA.domain_is_public (domain_id), iri);
     if (isnull (graph_iri))
       return;
   }
@@ -680,6 +716,8 @@ create procedure contact_delete (
 	scot_tags_delete (domain_id, iri, tags);
 	delete_quad_s_or_o (graph_iri, iri, iri);
 
+  SIOC..ods_object_services_dettach (graph_iri, iri, 'addressbook/contact');
+
   -- Social Data
   iri := SIOC..socialnetwork_contact_iri (domain_id, contact_id);
 	scot_tags_delete (domain_id, iri, tags);
@@ -746,7 +784,8 @@ create trigger PERSONS_SIOC_I after insert on AB.WA.PERSONS referencing new as N
                   N.P_UPDATED,
 									N.P_TAGS,
 									N.P_FOAF,
-									N.P_IRI);
+                  N.P_IRI,
+                  N.P_CERTIFICATE);
 }
 ;
 
@@ -810,7 +849,8 @@ create trigger PERSONS_SIOC_U after update on AB.WA.PERSONS referencing old as O
                   N.P_UPDATED,
 									N.P_TAGS,
 									N.P_FOAF,
-									N.P_IRI);
+                  N.P_IRI,
+                  N.P_CERTIFICATE);
 }
 ;
 
@@ -1004,23 +1044,24 @@ create procedure contact_comment_insert (
   master_id := cast (master_id as integer);
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
 	if (isnull (graph_iri))
-		{
-    graph_iri := get_graph_new (domain_id, null, master_iri);
+    graph_iri := get_graph_new (AB.WA.domain_is_public (domain_id), master_iri);
+
     if (isnull (graph_iri))
       return;
-		}
+
   if (isnull (forum_iri))
-	{
     forum_iri := AB.WA.forum_iri (domain_id);
+
     if (isnull (forum_iri))
       return;
-  }
+
 		comment_iri := addressbook_comment_iri (domain_id, master_id, comment_id);
   if (isnull (comment_iri))
     return;
 
       foaf_maker (graph_iri, u_url, u_name, u_mail);
-      ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_object_services_attach (graph_iri, comment_iri, 'addressbook/contact/comment');
   DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, sioc_iri ('has_reply'), comment_iri);
   DB.DBA.ODS_QUAD_URI (graph_iri, comment_iri, sioc_iri ('reply_of'), master_iri);
     }
@@ -1043,13 +1084,15 @@ create procedure contact_comment_delete (
   master_id := cast (master_id as integer);
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
   if (isnull (graph_iri))
-  {
-    graph_iri := SIOC..get_graph_new (domain_id, null, master_iri);
+    graph_iri := SIOC..get_graph_new (AB.WA.domain_is_public (domain_id), master_iri);
+
     if (isnull (graph_iri))
       return;
-  }
+
   comment_iri := SIOC..addressbook_comment_iri (domain_id, master_id, comment_id);
   delete_quad_s_or_o (graph_iri, comment_iri, comment_iri);
+  -- comment services
+  SIOC..ods_object_services_dettach (graph_iri, comment_iri, 'addressbook/contact/comment');
 }
 ;
 
@@ -1178,8 +1221,7 @@ create procedure contact_annotation_insert (
 	inout created datetime,
 	inout updated datetime)
 {
-	declare master_iri, annotattion_iri varchar;
-
+  declare master_iri, annotation_iri varchar;
 	declare exit handler for sqlstate '*'
 	{
 		sioc_log_message (__SQL_MESSAGE);
@@ -1190,19 +1232,20 @@ create procedure contact_annotation_insert (
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
 	if (isnull (graph_iri))
 		{
-    graph_iri := get_graph_new (domain_id, null, master_iri);
+    graph_iri := get_graph_new (AB.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
 		}
-  annotattion_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
-  DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, an_iri ('annotates'), master_iri);
-  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotattion_iri);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('author'), author);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('body'), body);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('created'), created);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('modified'), updated);
+  annotation_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
+  DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, an_iri ('annotates'), master_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotation_iri);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('author'), author);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('body'), body);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('created'), created);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('modified'), updated);
 
-	  addressbook_claims_insert (graph_iri, annotattion_iri, claims);
+  addressbook_claims_insert (graph_iri, annotation_iri, claims);
+  SIOC..ods_object_services_attach (graph_iri, annotation_iri, 'addressbook/contact/annotation');
 	}
 ;
 
@@ -1215,7 +1258,7 @@ create procedure contact_annotation_delete (
   inout annotation_id integer,
   inout claims any)
 {
-  declare master_iri, annotattion_iri varchar;
+  declare master_iri, annotation_iri varchar;
   declare exit handler for sqlstate '*'
   {
 		sioc_log_message (__SQL_MESSAGE);
@@ -1226,12 +1269,13 @@ create procedure contact_annotation_delete (
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
   if (isnull (graph_iri))
   {
-    graph_iri := SIOC..get_graph_new (domain_id, null, master_iri);
+    graph_iri := SIOC..get_graph_new (AB.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
   }
-  annotattion_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
-  SIOC..delete_quad_s_or_o (graph_iri, annotattion_iri, annotattion_iri);
+  annotation_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
+  SIOC..delete_quad_s_or_o (graph_iri, annotation_iri, annotation_iri);
+  SIOC..ods_object_services_dettach (graph_iri, annotation_iri, 'addressbook/contact/annotation');
 }
 ;
 
@@ -1239,7 +1283,7 @@ create procedure contact_annotation_delete (
 --
 create procedure addressbook_claims_insert (
   in graph_iri varchar,
-  in annotattion_iri varchar,
+  in annotation_iri varchar,
   in claims any)
 {
   declare N integer;
@@ -1256,7 +1300,7 @@ create procedure addressbook_claims_insert (
     } else {
       cPedicate := ODS.ODS_API."ontology.denormalize" (cPedicate);
   }
-    DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, cPedicate, cValue);
+    DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, cPedicate, cValue);
 }
 }
 ;
@@ -1329,9 +1373,24 @@ create procedure ods_addressbook_sioc_init ()
 }
 ;
 
---AB.WA.exec_no_error('ods_addressbook_sioc_init ()');
+-------------------------------------------------------------------------------
+--
+create procedure AB.WA.tmp_update ()
+{
+  if (registry_get ('ab_services_update') = '1')
+    return;
 
+  SIOC..fill_ods_addressbook_services();
+  registry_set ('ab_services_update', '1');
+}
+;
+
+AB.WA.tmp_update ();
+
+-------------------------------------------------------------------------------
+--
 use DB;
+
 -------------------------------------------------------------------------------
 --
 wa_exec_no_error ('drop view ODS_ADDRESSBOOK_CONTACTS');
