@@ -3,7 +3,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2007 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -37,6 +37,13 @@ function urlParam(fldName)
   return '';
 }
 
+function myA(obj) {
+  if (obj.href) {
+    document.location = obj.href + '?' + urlParam('sid') + urlParam('realm');
+    return false;
+  }
+}
+
 function myPost(frm_name, fld_name, fld_value) {
   createHidden(frm_name, fld_name, fld_value);
   document.forms[frm_name].submit();
@@ -61,6 +68,19 @@ function vspxPost(fButton, fName, fValue, f2Name, f2Value, f3Name, f3Value) {
   doPost ('F1', fButton);
 }
 
+function odsPost(obj, fields, button) {
+  var form = getParent (obj, 'form');
+  var formName = form.name;
+  for (var i = 0; i < fields.length; i += 2)
+    createHidden(formName, fields[i], fields[i+1]);
+
+  if (button) {
+    doPost(formName, button);
+  } else {
+    form.submit();
+  }
+}
+
 function toolbarPost(fValue) {
   vspxPost('command', 'select', fValue);
 }
@@ -178,8 +198,10 @@ function confirmAction(confirmMsq, form, txt, selectionMsq) {
   return false;
 }
 
-function selectCheck(obj, prefix) {
+function selectCheck(obj, prefix, noToolbars) {
   coloriseRow(getParent(obj, 'tr'), obj.checked);
+  if (noToolbars)
+    return;
   enableToolbars(obj.form, prefix);
 }
 
@@ -207,7 +229,7 @@ function enableElement(id, id_gray, showFlag) {
 }
 }
 
-function selectAllCheckboxes (obj, prefix) {
+function selectAllCheckboxes(obj, prefix, noToolbars) {
   var objForm = obj.form;
   for (var i = 0; i < objForm.elements.length; i++) {
     var o = objForm.elements[i];
@@ -219,12 +241,15 @@ function selectAllCheckboxes (obj, prefix) {
       coloriseRow(getParent(o, 'tr'), o.checked);
     }
   }
+  obj.focus();
   if (obj.value == 'Select All')
     obj.value = 'Unselect All';
   else
     obj.value = 'Select All';
-  selectCheck (obj, prefix);
-  obj.focus();
+
+  if (noToolbars)
+    return;
+  enableToolbars(objForm, prefix);
 }
 
 function anySelected (form, txt, selectionMsq) {
@@ -288,13 +313,18 @@ function showTab(tabs, tabsCount, tabNo) {
   }
 }
 
-function windowShow(sPage, width, height) {
-	if (!width)
-		width = 520;
-	if (!height)
-    height = 420;
-	sPage += urlParam('sid') + urlParam('realm');
-	win = window.open(sPage, null, "width="+width+",height="+height+", top=100, left=100, scrollbars=yes, resize=yes, menubar=no");
+function windowShow(sPage, sPageName, width, height) {
+	if (width == null)
+		width = 700;
+	if (height == null)
+		height = 500;
+  if (sPage.indexOf('form=') == -1)
+    sPage += '&form=F1';
+  if (sPage.indexOf('sid=') == -1)
+    sPage += urlParam('sid');
+  if (sPage.indexOf('realm=') == -1)
+    sPage += urlParam('realm');
+  win = window.open(sPage, sPageName, "width="+width+",height="+height+",top=100,left=100,status=yes,toolbar=no,menubar=no,scrollbars=yes,resizable=yes");
   win.window.focus();
 }
 
@@ -516,8 +546,17 @@ function updateGeodata(mode) {
 	OAT.AJAX.GET(S, '', function(arg) {cb(arg, mode);});
 }
 
-function davBrowse(fld) {
-	var options = {mode: 'browser', onConfirmClick: function(path, fname) {$(fld).value = path + fname;}};
+function davBrowse(fld, folders) {
+	/* load stylesheets */
+	OAT.Style.include("grid.css");
+	OAT.Style.include("webdav.css");
+
+  var options = {
+    mode: 'browser',
+    onConfirmClick: function(path, fname) {$(fld).value = '/DAV' + path + fname;}
+  };
+  if (!folders) {folders = false;}
+  OAT.WebDav.options.foldersOnly = folders;
   OAT.WebDav.open(options);
 }
 
@@ -577,6 +616,36 @@ function destinationChange(obj, actions) {
 			}
     }
   }
+	if (actions.exec) {
+		var a = actions.exec;
+		for ( var i = 0; i < a.length; i++) {
+			a[i](obj);
+		}
+	}
+}
+
+function excLabel(obj) {
+  var srcLabel = $('excLabel');
+  if (!srcLabel)
+    return;
+  if ($('exc_options_type_1').checked)
+    srcLabel.innerHTML = 'WebDAV File URL (.ics)';
+  if ($('exc_options_type_3').checked)
+    srcLabel.innerHTML = 'CalDAV URL';
+  if ($('exc_options_type_2').checked)
+    srcLabel.innerHTML = 'File URL (.ics)';
+}
+
+function iType(obj) {
+  var i = -1;
+  if ($('i_type_0').checked)
+    i = 0;
+  if ($('i_type_1').checked)
+    i = 1;
+  if ($('i_type_2').checked)
+    i = 2;
+  if (i < 0)
+    $('i_type_0').checked = true;
 }
 
 function urlParam(fldName)
@@ -666,7 +735,9 @@ function checkState()
     } else {
       progressTimer = null;
       progressPollTimer = null;
-      $('btn_Stop').click();
+      $('btn_Stop').value = 'Close';
+      OAT.Dom.hide('btn_Background');
+      doPost ('F1', 'btn_Background');
     }
   }
   OAT.AJAX.POST('ajax.vsp', "a=load&sa=state&id="+progressID+urlParam("sid")+urlParam("realm"), x);
@@ -843,20 +914,12 @@ AB.setFOAFValue = function(fValue, fName) {
 
 AB.aboutDialog = function() {
   var aboutDiv = $('aboutDiv');
-	if (aboutDiv) {
+  if (aboutDiv)
 		OAT.Dom.unlink(aboutDiv);
-	}
-	aboutDiv = OAT.Dom.create('div', {
-		width : '430px',
-		height : '150px'
-	});
+
+  aboutDiv = OAT.Dom.create('div', {height: '160px', overflow: 'hidden'});
   aboutDiv.id = 'aboutDiv';
-	aboutDialog = new OAT.Dialog('About ODS AddressBook', aboutDiv, {
-		width : 445,
-		buttons : 0,
-		resize : 0,
-		modal : 1
-	});
+  aboutDialog = new OAT.Dialog('About ODS AddressBook', aboutDiv, {width:445, buttons: 0, resize:0, modal:1});
 	aboutDialog.cancel = aboutDialog.hide;
 
   var x = function (txt) {
