@@ -1,10 +1,10 @@
 --
---  $Id: sioc_bmk.sql,v 1.19.2.9 2010/10/13 12:15:19 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -105,8 +105,13 @@ create procedure fill_ods_bookmark_sioc2 (
   declare id, deadl, cnt integer;
   declare domain_id, bookmark_id integer;
   declare graph_iri, forum_iri, creator_iri, bookmark_iri, iri varchar;
+
  {
+    -- init services
+    SIOC..fill_ods_bookmark_services ();
+
     for (select WAI_ID,
+                WAI_IS_PUBLIC,
                 WAI_TYPE_NAME,
                 WAI_NAME,
                 WAI_ACL
@@ -116,7 +121,7 @@ create procedure fill_ods_bookmark_sioc2 (
     {
       graph_iri := SIOC..acl_graph (WAI_TYPE_NAME, WAI_NAME);
       exec (sprintf ('sparql clear graph <%s>', graph_iri));
-      SIOC..wa_instance_acl_insert (WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
+      SIOC..wa_instance_acl_insert (WAI_IS_PUBLIC, WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
       for (select BD_DOMAIN_ID, BD_ID, BD_ACL
              from BMK..BOOKMARK_DOMAIN
             where BD_DOMAIN_ID = WAI_ID and BD_ACL is not null) do
@@ -161,7 +166,7 @@ create procedure fill_ods_bookmark_sioc2 (
           order by BD_ID) do
       {
       bookmark_iri := SIOC..bmk_post_iri (BD_DOMAIN_ID, BD_ID);
-      graph_iri := SIOC..get_graph_new (null, coalesce (_access_mode, WAI_IS_PUBLIC), bookmark_iri);
+      graph_iri := SIOC..get_graph_new (coalesce (_access_mode, WAI_IS_PUBLIC), bookmark_iri);
       forum_iri := SIOC..bmk_iri (coalesce (_wai_name, WAI_NAME));
       creator_iri := SIOC..user_iri (WAM_USER);
 
@@ -191,7 +196,34 @@ create procedure fill_ods_bookmark_sioc2 (
 
 -------------------------------------------------------------------------------
 --
-create procedure clean_ods_bookmark_sioc2 (
+create procedure fill_ods_bookmark_services ()
+{
+  declare graph_iri, services_iri, service_iri, service_url varchar;
+  declare svc_functions any;
+
+  graph_iri := get_graph ();
+
+  -- instance
+  svc_functions := vector ('bookmark.new', 'bookmark.import', 'bookmark.export', 'bookmark.publication.new', 'bookmark.subscription.new', 'bookmark.options.set',  'bookmark.options.get');
+  ods_object_services (graph_iri, 'bookmark', 'ODS Bookmark instance services', svc_functions);
+
+  -- item
+  svc_functions := vector ('bookmark.get', 'bookmark.edit', 'bookmark.delete', 'bookmark.comment.new', 'bookmark.annotation.new');
+  ods_object_services (graph_iri, 'bookmark/item', 'ODS Bookmark item services', svc_functions);
+
+  -- item comment
+  svc_functions := vector ('bookmark.comment.get', 'bookmark.comment.delete');
+  ods_object_services (graph_iri, 'bookmark/item/comment', 'ODS Bookmark comment services', svc_functions);
+
+  -- item annotation
+  svc_functions := vector ('bookmark.annotation.get', 'bookmark.annotation.claim', 'bookmark.annotation.delete');
+  ods_object_services (graph_iri, 'bookmark/item/annotation', 'ODS Bookmark annotation services', svc_functions);
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create procedure clean_ods_bookmark_sioc (
   in _wai_name varchar := null,
   in _access_mode integer := null)
 {
@@ -229,9 +261,9 @@ create procedure clean_ods_bookmark_sioc2 (
           order by BD_ID) do
     {
       bookmark_iri := SIOC..bmk_post_iri (BD_DOMAIN_ID, BD_ID);
-      graph_iri := SIOC..get_graph_new (WAI_ID, coalesce (_access_mode, WAI_IS_PUBLIC), bookmark_iri);
+      graph_iri := SIOC..get_graph_new (coalesce (_access_mode, WAI_IS_PUBLIC), bookmark_iri);
 
-      bookmark_domain_delete (graph_iri, BD_DOMAIN_ID, BD_ID);
+      SIOC..bookmark_domain_delete (graph_iri, BD_DOMAIN_ID, BD_ID);
 
       cnt := cnt + 1;
       if (mod (cnt, 500) = 0)
@@ -245,22 +277,6 @@ create procedure clean_ods_bookmark_sioc2 (
 }
 ;
 
--------------------------------------------------------------------------------
---
-create procedure ods_bookmark_services (
-  in graph_iri varchar, 
-  in forum_iri varchar,
-  in wai_id varchar := null,
-  in wai_name varchar := null)
-{
-  declare svc_iri varchar;
-  
-  -- dbg_obj_print (now (), 'ods_bookmark_services');
-  svc_iri := sprintf ('http://%s%s/services/bookmark', get_cname(), get_base_path ());
-  ods_sioc_service (graph_iri, svc_iri, forum_iri, null, 'text/xml', svc_iri||'/services.wsdl', svc_iri, 'SOAP');
-}
-;
-
 -------------------------------------------------------------------------------
 --
 create procedure bookmark_domain_insert (
@@ -300,7 +316,7 @@ create procedure bookmark_domain_insert (
           and WAM_INST = WAI_NAME
             and U_ID = WAM_USER) do
   {
-      graph_iri := SIOC..get_graph_new (WAI_ID, WAI_IS_PUBLIC, bookmark_iri);
+      graph_iri := SIOC..get_graph_new (WAI_IS_PUBLIC, bookmark_iri);
       forum_iri := SIOC..bmk_iri (WAI_NAME);
       creator_iri := SIOC..user_iri (WAM_USER);
 
@@ -316,6 +332,9 @@ create procedure bookmark_domain_insert (
   SIOC..ods_sioc_post (graph_iri, bookmark_iri, forum_iri, creator_iri, name, created, updated, uri, description, null, linksTo);
   SIOC..scot_tags_insert (domain_id, bookmark_iri, tags);
 
+  -- bookmark services
+  SIOC..ods_object_services_dettach (graph_iri, bookmark_iri, 'bookmark/item');
+
   SIOC..bookmark_comments_insert (graph_iri, forum_iri, domain_id, bookmark_id);
   SIOC..bookmark_annotations_insert (graph_iri, domain_id, bookmark_id);
 }
@@ -337,11 +356,12 @@ create procedure bookmark_domain_delete (
   bookmark_iri := SIOC..bmk_post_iri (domain_id, bookmark_id);
   if (isnull (graph_iri))
   {
-    graph_iri := SIOC..get_graph_new (domain_id, null, bookmark_iri);
+    graph_iri := SIOC..get_graph_new (BMK.WA.domain_is_public (domain_id), bookmark_iri);
     if (isnull (graph_iri))
       return;
   }
   SIOC..delete_quad_s_or_o (graph_iri, bookmark_iri, bookmark_iri);
+  SIOC..ods_object_services_attach (graph_iri, bookmark_iri, 'bookmark/item');
   SIOC..bookmark_comments_delete (graph_iri, domain_id, bookmark_id);
   SIOC..bookmark_annotations_delete (graph_iri, domain_id, bookmark_id);
 }
@@ -569,7 +589,7 @@ create procedure bmk_comment_insert (
   master_iri := SIOC..bmk_post_iri (domain_id, master_id);
 	if (isnull (graph_iri))
 		{
-    graph_iri := get_graph_new (domain_id, null, master_iri);
+    graph_iri := get_graph_new (BMK.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
 		}
@@ -584,7 +604,8 @@ create procedure bmk_comment_insert (
     return;
 
       foaf_maker (graph_iri, u_url, u_name, u_mail);
-      ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_object_services_attach (graph_iri, comment_iri, 'bookmark/item/comment');
   DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, sioc_iri ('has_reply'), comment_iri);
   DB.DBA.ODS_QUAD_URI (graph_iri, comment_iri, sioc_iri ('reply_of'), master_iri);
     }
@@ -609,12 +630,13 @@ create procedure bmk_comment_delete (
   master_iri := SIOC..bmk_post_iri (domain_id, master_id);
   if (isnull (graph_iri))
   {
-    graph_iri := SIOC..get_graph_new (domain_id, null, master_iri);
+    graph_iri := SIOC..get_graph_new (BMK.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
   }
   comment_iri := bmk_comment_iri (domain_id, master_id, comment_id);
   delete_quad_s_or_o (graph_iri, comment_iri, comment_iri);
+  SIOC..ods_object_services_dettach (graph_iri, comment_iri, 'bookmark/item/comment');
 }
 ;
 
@@ -742,8 +764,7 @@ create procedure bmk_annotation_insert (
 	inout created datetime,
 	inout updated datetime)
 {
-	declare master_iri, annotattion_iri varchar;
-
+  declare master_iri, annotation_iri varchar;
 	declare exit handler for sqlstate '*'
 	{
 		sioc_log_message (__SQL_MESSAGE);
@@ -754,19 +775,20 @@ create procedure bmk_annotation_insert (
   master_iri := SIOC..bmk_post_iri (domain_id, master_id);
 	if (isnull (graph_iri))
 		{
-    graph_iri := get_graph_new (domain_id, null, master_iri);
+    graph_iri := get_graph_new (BMK.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
 		}
-		annotattion_iri := bmk_annotation_iri (domain_id, master_id, annotation_id);
-  DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, an_iri ('annotates'), master_iri);
-  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotattion_iri);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('author'), author);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('body'), body);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('created'), created);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('modified'), updated);
+  annotation_iri := bmk_annotation_iri (domain_id, master_id, annotation_id);
+  DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, an_iri ('annotates'), master_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotation_iri);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('author'), author);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('body'), body);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('created'), created);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('modified'), updated);
 
-	  bmk_claims_insert (graph_iri, annotattion_iri, claims);
+  bmk_claims_insert (graph_iri, annotation_iri, claims);
+  SIOC..ods_object_services_attach (graph_iri, annotation_iri, 'bookmark/item/annotation');
 	}
 ;
 
@@ -779,7 +801,7 @@ create procedure bmk_annotation_delete (
   inout annotation_id integer,
   inout claims any)
 {
-  declare master_iri, annotattion_iri varchar;
+  declare master_iri, annotation_iri varchar;
 	declare exit handler for sqlstate '*'
 	{
 		sioc_log_message (__SQL_MESSAGE);
@@ -790,12 +812,13 @@ create procedure bmk_annotation_delete (
   if (isnull (graph_iri))
   {
     master_iri := SIOC..bmk_post_iri (domain_id, master_id);
-    graph_iri := SIOC..get_graph_new (domain_id, null, master_iri);
+    graph_iri := SIOC..get_graph_new (BMK.WA.domain_is_public (domain_id), master_iri);
     if (isnull (graph_iri))
       return;
   }
-	annotattion_iri := bmk_annotation_iri (domain_id, master_id, annotation_id);
-  SIOC..delete_quad_s_or_o (graph_iri, annotattion_iri, annotattion_iri);
+  annotation_iri := bmk_annotation_iri (domain_id, master_id, annotation_id);
+  SIOC..delete_quad_s_or_o (graph_iri, annotation_iri, annotation_iri);
+  SIOC..ods_object_services_dettach (graph_iri, annotation_iri, 'bookmark/item/annotation');
 }
 ;
 
@@ -803,7 +826,7 @@ create procedure bmk_annotation_delete (
 --
 create procedure bmk_claims_insert (
   in graph_iri varchar,
-  in annotattion_iri varchar,
+  in annotation_iri varchar,
   in claims any)
 {
   declare N integer;
@@ -820,7 +843,7 @@ create procedure bmk_claims_insert (
     } else {
       cPedicate := ODS.ODS_API."ontology.denormalize" (cPedicate);
   }
-    DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, cPedicate, cValue);
+    DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, cPedicate, cValue);
 }
 }
 ;
@@ -895,7 +918,21 @@ create procedure ods_bookmark_sioc_init ()
 
 -------------------------------------------------------------------------------
 --
--- Bookmrks RDF Views
+create procedure BMK.WA.tmp_update ()
+{
+  if (registry_get ('bmk_services_update') = '1')
+    return;
+
+  SIOC..fill_ods_bookmark_services();
+  registry_set ('bmk_services_update', '1');
+}
+;
+
+BMK.WA.tmp_update ();
+
+-------------------------------------------------------------------------------
+--
+-- Bookmarks RDF Views
 --
 -------------------------------------------------------------------------------
 use DB;
