@@ -1,10 +1,10 @@
 /*
- *  $Id: bookmark.js,v 1.27.2.11 2010/11/19 16:49:52 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -58,6 +58,19 @@ function toolbarPost(value)
   doPost ('F1', 'toolbar');
 }
 
+function odsPost(obj, fields, button) {
+  var form = getParent (obj, 'form');
+  var formName = form.name;
+  for (var i = 0; i < fields.length; i += 2)
+    createHidden(formName, fields[i], fields[i+1]);
+
+  if (button) {
+    doPost(formName, button);
+  } else {
+    form.submit();
+  }
+}
+
 function dateFormat(date, format) {
 	function long(d) {
 		return ((d < 10) ? "0" : "") + d;
@@ -363,14 +376,18 @@ function showTab(tabs, tabsCount, tabNo)
   }
 }
 
-function windowShow(sPage, width, height)
-{
-  if (!width)
+function windowShow(sPage, sPageName, width, height) {
+	if (width == null)
     width = 700;
-  if (!height)
-    height = 420;
-  sPage += urlParam("sid")+urlParam("realm");
-  win = window.open(sPage, null, "width="+width+",height="+height+", top=100, left=100, scrollbars=yes, resize=yes, menubar=no");
+	if (height == null)
+		height = 500;
+  if (sPage.indexOf('form=') == -1)
+    sPage += '&form=F1';
+  if (sPage.indexOf('sid=') == -1)
+    sPage += urlParam('sid');
+  if (sPage.indexOf('realm=') == -1)
+    sPage += urlParam('realm');
+  win = window.open(sPage, sPageName, "width="+width+",height="+height+",top=100,left=100,status=yes,toolbar=no,menubar=no,scrollbars=yes,resizable=yes");
   win.window.focus();
 }
 
@@ -609,6 +626,13 @@ function urlParam (fldName)
   return '';
 }
 
+function myA(obj) {
+  if (obj.href) {
+    document.location = obj.href + '?' + urlParam('sid') + urlParam('realm');
+    return false;
+  }
+}
+
 var progressTimer = null;
 var progressID = null;
 var progressMax = null;
@@ -618,7 +642,7 @@ var progressInc = 100 / progressSize;
 function stopState()
 {
   progressTimer = null;
-  OAT.AJAX.POST('ajax.vsp', "a=load&sa=stop&id="+progressID+urlParam("sid")+urlParam("realm"), x, {async: false});
+  OAT.AJAX.POST('ajax.vsp', "a=load&sa=stop&id="+progressID+urlParam("sid")+urlParam("realm"), null, {async: false});
 }
 
 function initState ()
@@ -653,7 +677,8 @@ function checkState()
         setTimeout("checkState()", 500);
       } else {
       progressTimer = null;
-        $('btn_Stop').click();
+      $('btn_Stop').value = 'Close';
+      OAT.Dom.hide('btn_Background');
       }
     }
   OAT.AJAX.POST('ajax.vsp', "a=load&sa=state&id="+progressID+urlParam("sid")+urlParam("realm"), x);
@@ -724,12 +749,20 @@ function readBookmark (id)
   var realm = '';
   if (document.forms[0].elements['realm'])
     realm = document.forms[0].elements['realm'].value;
-  OAT.AJAX.POST ("ajax.vsp", "sid="+sid+"&realm="+realm+"&id="+id+"&a=visited", function(){}, {onstart:function(){}, onerror:function(){}});
+  OAT.AJAX.POST ('ajax.vsp', "sid="+sid+"&realm="+realm+"&id="+id+"&a=visited", function(){}, {onstart:function(){}, onerror:function(){}});
 }
 
-function davBrowse (fld)
-{
-  var options = {mode: 'browser', onConfirmClick: function(path, fname) {$(fld).value = path + fname;}};
+function davBrowse(fld, folders) {
+	/* load stylesheets */
+	OAT.Style.include("grid.css");
+	OAT.Style.include("webdav.css");
+
+  var options = {
+    mode: 'browser',
+    onConfirmClick: function(path, fname) {$(fld).value = '/DAV' + path + fname;}
+  };
+  if (!folders) {folders = false;}
+  OAT.WebDav.options.foldersOnly = folders;
   OAT.WebDav.open(options);
 }
 
@@ -1266,7 +1299,7 @@ BMK.loadItems = function(nodeID, nodePath)
     nodePath = nodeID;
   var pane = $('pane_right_top');
   pane.innerHTML = '';
-  var URL = 'forms.vspx?sa=browse&node='+encodeURIComponent(nodeID)+'&path='+encodeURIComponent(nodePath)+BMK.sessionParams();
+  var URL = 'forms.vspx?sa=browse&node='+encodeURIComponent(nodeID)+'&path='+encodeURIComponent('/'+BMK.trim(nodePath, '/'))+BMK.sessionParams();
   var v = $('nodeItem');
   if (v && (v.value != '')) {
     URL += '&item=' + v.value;
@@ -1399,8 +1432,10 @@ BMK.formPostAfter = function (action)
 BMK.aboutDialog = function ()
 {
   var aboutDiv = $('aboutDiv');
-  if (aboutDiv) {OAT.Dom.unlink(aboutDiv);}
-  aboutDiv = OAT.Dom.create('div', {width:'430px', height:'150px'});
+  if (aboutDiv)
+    OAT.Dom.unlink(aboutDiv);
+
+  aboutDiv = OAT.Dom.create('div', {height: '160px', overflow: 'hidden'});
   aboutDiv.id = 'aboutDiv';
   aboutDialog = new OAT.Dialog('About ODS Booomarks', aboutDiv, {width:445, buttons: 0, resize:0, modal:1});
 	aboutDialog.cancel = aboutDialog.hide;
