@@ -1,10 +1,10 @@
 --
---  $Id: calendar-a-table.sql,v 1.26.2.5 2010/11/19 16:50:15 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2007 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -111,7 +111,7 @@ CAL.WA.exec_no_error ('
 --
 create procedure CAL.WA.my_calendars (
   in domain_id any,
-  in privacy varchar)
+  in privacy integer)
 {
   declare calendar_id, calendar_privacy integer;
 
@@ -475,6 +475,7 @@ CAL.WA.exec_no_error ('
     CAL.WA.upstream_event_update (N.E_DOMAIN_ID, N.E_ID, _uid, N.E_TAGS, \'U\');
     CAL.WA.exchange_event_update (N.E_DOMAIN_ID);
     CAL.WA.syncml_entry_update (N.E_DOMAIN_ID, N.E_ID, _uid, N.E_KIND, \'U\');
+    CAL.WA.domain_ping (N.E_DOMAIN_ID);
   }
 ');
 
@@ -485,14 +486,16 @@ CAL.WA.exec_no_error ('
     delete from CAL.WA.ALARMS where A_EVENT_ID = O.E_ID;
 
     CAL.WA.upstream_event_update (O.E_DOMAIN_ID, O.E_ID, O.E_UID, O.E_TAGS, \'D\');
+    CAL.WA.exchange_event_update (O.E_DOMAIN_ID);
     CAL.WA.syncml_entry_update (O.E_DOMAIN_ID, O.E_ID, O.E_UID, O.E_KIND, \'D\');
+    CAL.WA.domain_ping (O.E_DOMAIN_ID);
   }
 ');
 
 -------------------------------------------------------------------------------
 --
 create procedure CAL.WA.tags_update (
-  inout domain_id integer,
+  in domain_id integer,
   in oTags any,
   in nTags any)
 {
