@@ -1,10 +1,10 @@
 --
---  $Id: sioc_feeds.sql,v 1.20.2.9 2010/10/13 12:14:21 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -232,12 +232,16 @@ create procedure fill_ods_subscriptions_sioc (in graph_iri varchar, in site_iri
 --
 create procedure fill_ods_feeds_sioc (in graph_iri varchar, in site_iri varchar, in _wai_name varchar := null)
 {
-  declare iri, m_iri, f_iri, t_iri, u_iri, c_iri varchar;
+  declare acl_graph_iri, iri, m_iri, f_iri, t_iri, u_iri, c_iri varchar;
   declare tags, linksTo any;
   declare id, deadl, cnt any;
 
  {
+    -- init service containers
+    fill_ods_feeds_services ();
+
     for (select WAI_ID,
+                WAI_IS_PUBLIC,
                 WAI_TYPE_NAME,
                 WAI_NAME,
                 WAI_ACL
@@ -245,9 +249,9 @@ create procedure fill_ods_feeds_sioc (in graph_iri varchar, in site_iri varchar,
           where ((_wai_name is null) or (WAI_NAME = _wai_name))
             and WAI_TYPE_NAME = 'eNews2') do
     {
-      graph_iri := SIOC..acl_graph (WAI_TYPE_NAME, WAI_NAME);
-      exec (sprintf ('sparql clear graph <%s>', graph_iri));
-      SIOC..wa_instance_acl_insert (WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
+      acl_graph_iri := SIOC..acl_graph (WAI_TYPE_NAME, WAI_NAME);
+      exec (sprintf ('sparql clear graph <%s>', acl_graph_iri));
+      SIOC..wa_instance_acl_insert (WAI_IS_PUBLIC, WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
       for (select EFD_DOMAIN_ID, EFD_FEED_ID, EFD_ACL
              from ENEWS.WA.FEED_DOMAIN
             where EFD_DOMAIN_ID = WAI_ID and EFD_ACL is not null) do
@@ -379,6 +383,37 @@ create procedure fill_ods_feeds_sioc (in graph_iri varchar, in site_iri varchar,
 }
 ;
 
+-------------------------------------------------------------------------------
+--
+create procedure fill_ods_feeds_services ()
+{
+  declare graph_iri, services_iri, service_iri, service_url varchar;
+  declare svc_functions any;
+
+  graph_iri := get_graph ();
+
+  -- instance
+  svc_functions := vector ('feeds.subscribe', 'feeds.blog.subscribe', 'feeds.options.set',  'feeds.options.get');
+  ods_object_services (graph_iri, 'feeds', 'ODS feeds instance services', svc_functions);
+
+  -- feed
+  svc_functions := vector ('feeds.get', 'feeds.unsubscribe', 'feeds.refresh');
+  ods_object_services (graph_iri, 'feeds/feed', 'ODS Feeds feed services', svc_functions);
+
+  -- blog
+  svc_functions := vector ('feeds.blog.unsubscribe', 'feeds.blog.refresh');
+  ods_object_services (graph_iri, 'feeds/blog', 'ODS Feeds blog services', svc_functions);
+
+  -- feed item comment
+  svc_functions := vector ('feeds.comment.get', 'feeds.comment.delete');
+  ods_object_services (graph_iri, 'feeds/item/comment', 'ODS Feeds comment services', svc_functions);
+
+  -- feed item annotation
+  svc_functions := vector ('feeds.annotation.get', 'feeds.annotation.claim', 'feeds.annotation.delete');
+  ods_object_services (graph_iri, 'feeds/item/annotation', 'ODS Feeds annotation services', svc_functions);
+}
+;
+
 -------------------------------------------------------------------------------
 --
 -- ENEWS..FEED
@@ -683,7 +718,8 @@ create procedure feeds_comment_insert (
   declare feed_id integer;
   declare graph_iri, iri, feed_iri, item_iri varchar;
 
-  declare exit handler for sqlstate '*' {
+  declare exit handler for sqlstate '*'
+  {
     sioc_log_message (__SQL_MESSAGE);
     return;
   };
@@ -693,9 +729,12 @@ create procedure feeds_comment_insert (
   feed_iri := feed_iri (feed_id);
   item_iri := feed_item_iri (feed_id, item_id);
   iri := feed_comment_iri (domain_id, item_id, id);
-  if (not isnull (iri)) {
+  if (not isnull (iri))
+  {
   foaf_maker (graph_iri, u_url, u_name, u_mail);
     ods_sioc_post (graph_iri, iri, feed_iri, null, title, last_update, last_update, feed_item_url (domain_id, item_id), comment, null, null, u_url);
+    -- services
+    SIOC..ods_object_services_attach (graph_iri, iri, 'feeds/item/comment');
     DB.DBA.ODS_QUAD_URI (graph_iri, item_iri, sioc_iri ('has_reply'), iri);
     DB.DBA.ODS_QUAD_URI (graph_iri, iri, sioc_iri ('reply_of'), item_iri);
 }
@@ -719,7 +758,8 @@ create procedure feeds_comment_delete (
   graph_iri := get_graph ();
   iri := feed_comment_iri (domain_id, item_id, id);
   delete_quad_s_or_o (graph_iri, iri, iri);
-  return;
+  -- services
+  SIOC..ods_object_services_dettach (graph_iri, iri, 'feeds/item/comment');
 }
 ;
 
@@ -766,7 +806,7 @@ create procedure feeds_annotation_insert (
   inout created datetime,
   inout updated datetime)
 {
-  declare master_iri, annotattion_iri varchar;
+  declare master_iri, annotation_iri varchar;
 
   declare exit handler for sqlstate '*' {
     sioc_log_message (__SQL_MESSAGE);
@@ -785,26 +825,27 @@ create procedure feeds_annotation_insert (
       forum_iri := feeds_iri (WAI_NAME);
     }
 
-  if (not isnull (graph_iri)) {
+  if (isnull (graph_iri))
+    return;
+
     declare feed_id integer;
 
     feed_id := (select EFI_FEED_ID from ENEWS.WA.FEED_ITEM where EFI_ID = master_id);
     master_iri := feed_item_iri (feed_id, master_id);
-    annotattion_iri := feed_annotation_iri (domain_id, cast (master_id as integer), annotation_id);
+  annotation_iri := feed_annotation_iri (domain_id, cast (master_id as integer), annotation_id);
 
-	  DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, sioc_iri ('has_container'), forum_iri);
-	  DB.DBA.ODS_QUAD_URI (graph_iri, forum_iri, sioc_iri ('container_of'), annotattion_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, sioc_iri ('has_container'), forum_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, forum_iri, sioc_iri ('container_of'), annotation_iri);
 
-	  DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, an_iri ('annotates'), master_iri);
-	  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotattion_iri);
-	  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('author'), author);
-	  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('body'), body);
-	  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('created'), created);
-	  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('modified'), updated);
+  DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, an_iri ('annotates'), master_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotation_iri);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('author'), author);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('body'), body);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('created'), created);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('modified'), updated);
 
-	  feeds_claims_insert (graph_iri, annotattion_iri, claims);
-  }
-  return;
+  feeds_claims_insert (graph_iri, annotation_iri, claims);
+  SIOC..ods_object_services_attach (graph_iri, annotation_iri, 'feeds/item/annotation');
 }
 ;
 
@@ -816,7 +857,7 @@ create procedure feeds_annotation_delete (
   inout master_id integer,
   inout claims any)
 {
-  declare graph_iri, annotattion_iri varchar;
+  declare graph_iri, annotation_iri varchar;
 
   declare exit handler for sqlstate '*' {
     sioc_log_message (__SQL_MESSAGE);
@@ -824,8 +865,9 @@ create procedure feeds_annotation_delete (
   };
 
   graph_iri := get_graph ();
-  annotattion_iri := feed_annotation_iri (domain_id, master_id, annotation_id);
-  delete_quad_s_or_o (graph_iri, annotattion_iri, annotattion_iri);
+  annotation_iri := feed_annotation_iri (domain_id, master_id, annotation_id);
+  delete_quad_s_or_o (graph_iri, annotation_iri, annotation_iri);
+  SIOC..ods_object_services_dettach (graph_iri, annotation_iri, 'feeds/item/annotation');
 }
 ;
 
@@ -922,6 +964,22 @@ create procedure ods_feeds_sioc_init ()
 ;
 --ENEWS.WA.exec_no_error('ods_feeds_sioc_init ()');
 
+-------------------------------------------------------------------------------
+--
+create procedure ENEWS.WA.tmp_update ()
+{
+  if (registry_get ('news_services_update') = '1')
+    return;
+
+  SIOC..fill_ods_feeds_services();
+  registry_set ('news_services_update', '1');
+}
+;
+
+ENEWS.WA.tmp_update ();
+
+-------------------------------------------------------------------------------
+--
 use DB;
 -- FEEDS
 
