@@ -1,10 +1,10 @@
 /*
- *  $Id: common.js,v 1.13.2.40 2011/03/26 14:22:25 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -26,35 +26,42 @@ function showError(msg) {
   return false;
 }
 
-function toggleControl (ctr1, val, ctr2)
-{
-  if (ctr2 == null)
-    return;
+function findParent(obj, tag) {
+  var obj = obj.parentNode;
+  if (obj.tagName.toLowerCase() == tag)
+    return obj;
+  return findParent(obj, tag);
+    }
 
-  if (ctr1 && ctr1.value == val)
+function odsPost(obj, fields, button) {
+  var form = findParent (obj, 'form');
+  for (var i = 0; i < fields.length; i += 2)
+    hiddenCreate(fields[i], form, fields[i+1]);
+
+  if (button) {
+    doPost(form.name, button);
+  } else {
+    form.submit();
+  }
+}
+
+function toggleControl (ctr1, val, ctr2)
     {
-      ctr2.disabled = true;
-    }
-  else
-    {
-      ctr2.disabled = false;
-    }
+  if (ctr2 != null)
+    ctr2.disabled = (ctr1 && ctr1.value == val);
 }
 
 function setSelectLists (val, form, pref)
 {
-  var i;
   if (val == 0 || form == null || pref == null)
     return;
-  for (i = 0; i < form.elements.length; i++)
+  for (var i = 0; i < form.elements.length; i++)
     {
       var contr = form.elements[i];
       if (contr != null && contr.type == 'select-one' && contr.name.indexOf (pref) != -1)
-        {
           contr.value = val;
         }
     }
-}
 
 function dateFormat(date, format)
 {
@@ -308,6 +315,18 @@ function divs_switch ( showhide, divname_a, divname_b)
   }
 }
 
+function readCookie(name) {
+	var cookies = document.cookie.split(';');
+
+	for ( var i = 0; i < cookies.length; i++) {
+		cookies[i] = cookies[i].trim();
+
+		if (cookies[i].indexOf(name + '=') == 0)
+			return cookies[i].substring (name.length + 1, cookies[i].length);
+	}
+	return false;
+}
+
 function windowShow(sPage, width, height)
 {
   if (width == null)
@@ -488,6 +507,7 @@ function updateGeodata(mode)
 
 function initLoadProfile()
 {
+  def_btn = null;
   if (top.location == self.location)
     return true;
   parent.nav.showUserProfile();
@@ -550,8 +570,107 @@ function pageFocus(tab) {
   }
 }
 
+function accountDisable(userName)
+{
+  var S = '/ods/api/user.disable?name='+encodeURIComponent($v(userName))
+        + '&sid=' + document.forms[0].elements['sid'].value
+        + '&realm=' + document.forms[0].elements['realm'].value;
+	var x = function(data) {
+    var xml = OAT.Xml.createXmlDoc(data);
+    if (!hasError(xml, false)) {
+      alert('User\'s account is disabled!');
+      if (parent) {
+			  parent.document.location = document.location.protocol + '//' + document.location.host + '/ods';
+			} else {
+			  document.location = document.location.protocol + '//' + document.location.host + '/ods';
+			}
+    }
+	}
+  OAT.AJAX.GET(S, '', x);
+}
+
+function destinationChange(obj, changes) {
+  function destinationChangeInternal(actions) {
+    if (!obj)
+      return;
+
+    if (actions.hide) {
+      var a = actions.hide;
+      for ( var i = 0; i < a.length; i++) {
+        var o = $(a[i])
+        if (o) {
+          OAT.Dom.hide(o);
+        }
+      }
+    }
+    if (actions.show) {
+      var a = actions.show;
+      for ( var i = 0; i < a.length; i++) {
+        var o = $(a[i])
+        if (o) {
+          OAT.Dom.show(o);
+        }
+      }
+    }
+    if (actions.clear) {
+      var a = actions.clear;
+      for ( var i = 0; i < a.length; i++) {
+        var o = $(a[i])
+        if (o && o.value) {
+          o.value = '';
+        }
+      }
+    }
+  }
+  if (!changes)
+    return;
+
+  if (obj.checked && changes.checked)
+    destinationChangeInternal(changes.checked);
+
+  if (!obj.checked && changes.unchecked)
+    destinationChangeInternal(changes.unchecked);
+}
+
+function swapRows(newIndex)
+{
+  var row_0 = $('pf_tabs_0_row_' + ((newIndex < 6)? '1': '0'));
+  var row_1 = $('pf_tabs_0_row_' + ((newIndex < 6)? '0': '1'));
+  if (!row_0.nextElementSibling)
+    swapNodes(row_0, row_1);
+}
+
+function swapNodes(node1, node2)
+{
+  if (!node1) return;
+  if (!node2) return;
+  var tmp = node1.cloneNode(1);
+  var parent = node1.parentNode;
+  node2 = parent.replaceChild(tmp, node2);
+  parent.replaceChild(node2, node1);
+  parent.replaceChild(node1, tmp);
+  tmp = null;
+}
+
+function loginChange(login, register)
+{
+  var l = $(login);
+  var r = $(register);
+  if (!l.checked)
+    r.checked = false;
+}
+
+function registerChange(login, register)
+{
+  var l = $(login);
+  var r = $(register);
+  if (r.checked)
+    l.checked = true;
+}
+
 // RDF Relations
 // ---------------------------------------------------------------------------
+var rdfDialog
 var RDF = new Object();
 RDF.tablePrefix = 'r';
 RDF.itemTypes = new Object();
@@ -633,7 +752,6 @@ RDF.ontologies['xsd'] = {"name": 'http://www.w3.org/2001/XMLSchema#', "hidden":
 RDF.loadOntology = function (ontologyName, cb, options)
 {
   var ontology = this.getOntologyByName(ontologyName);
-  var dependent = ontology.dependent;
   var prefix = this.ontologyPrefix(ontologyName);
   if (!prefix) {
     var N = 0;
@@ -646,9 +764,12 @@ RDF.loadOntology = function (ontologyName, cb, options)
   }
 
   // load ontology classes
-  var S = '/ods/api/ontology.classes?ontology='+encodeURIComponent(ontologyName);
-  if (dependent)
+  var S = '/ods/api/ontology.classes?ontology='+encodeURIComponent(ontologyName)+ '&prefix='+encodeURIComponent(prefix);
+  var dependent;
+  if (ontology && ontology.dependent) {
+    dependent = ontology.dependent;
     S += '&dependentOntology='+encodeURIComponent(this.getOntologyByPrefix(dependent).name);
+  }
   var x = function(data) {
     var o = null;
     try {
@@ -698,11 +819,9 @@ RDF.getOntologyByClass = function(className)
 RDF.getOntologyClass = function(className)
 {
   var ontology = this.getOntologyByClass(className);
-  if (ontology)
-  {
+  if (ontology) {
     var classes = ontology.classes;
-    for (var i = 0; i < classes.length; i++)
-    {
+    for (var i = 0; i < classes.length; i++) {
       if (classes[i].name == className)
         return classes[i];
     }
@@ -726,8 +845,13 @@ RDF.getOntologyClassProperty = function(className, propertyName)
       if (properties[i].name == propertyName)
         return properties[i];
     }
-    if (ontologyClass.subClassOf)
-      return this.getOntologyClassProperty(ontologyClass.subClassOf, propertyName)
+
+      if (ontologyClass.subClassOf instanceof Array)
+        for (var i=0; i<ontologyClass.subClassOf.length; i++) {
+          var property = this.getOntologyClassProperty(ontologyClass.subClassOf[i], propertyName);
+          if (property)
+            return property;
+        }
   }
   }
   return null;
@@ -757,8 +881,11 @@ RDF.isKindOfClass = function(objectClassName, propertyClassName)
     return true;
 
   var ontologyClass = this.getOntologyClass(objectClassName);
-  if (ontologyClass && ontologyClass.subClassOf)
-    return this.isKindOfClass (ontologyClass.subClassOf, propertyClassName);
+  if (ontologyClass && (ontologyClass.subClassOf instanceof Array))
+    for (var i = 0; i < ontologyClass.subClassOf; i++) {
+      if (this.hasClassProperties(ontologyClass.subClassOf[i]))
+        return true;
+    }
 
   return false;
 }
@@ -771,18 +898,23 @@ RDF.loadClassProperties = function(ontologyClass, cbFunction)
   }
   var ontologyName = RDF.getOntologyByClass(ontologyClass.name).name;
   var prefix = RDF.ontologyPrefix(ontologyName);
-  var S = '/ods/api/ontology.classProperties?ontologyClass='+encodeURIComponent(ontologyClass.name)+'&ontology='+encodeURIComponent(ontologyName)+'&prefix='+prefix;
+  var S = '/ods/api/ontology.classProperties?ontology='+encodeURIComponent(ontologyName)+'&prefix='+prefix+'&ontologyClass='+encodeURIComponent(ontologyClass.name);
   var x = function(data) {
     var o = null;
     try {
       o = OAT.JSON.parse(data);
     } catch (e) { o = null; }
     ontologyClass.properties = o;
-    var ontologySubClass = RDF.getOntologyClass(ontologyClass.subClassOf);
+    if (ontologyClass.subClassOf instanceof Array) {
+      for (var i = 0; i < ontologyClass.subClassOf.length; i++) {
+        var ontologySubClass = RDF.getOntologyClass(ontologyClass.subClassOf[i]);
     if (ontologySubClass) {
       RDF.loadClassProperties(ontologySubClass, cbFunction)
-      return;
+          cbFunction = null;
+        }
     }
+    }
+    if (cbFunction)
     cbFunction();
   }
   OAT.AJAX.GET(S, '', x, {});
@@ -804,8 +936,11 @@ RDF.hasClassProperties = function(className)
     if (ontologyClass.properties.length)
       return true;
 
-    if (ontologyClass.subClassOf)
-      return this.hasClassProperties(ontologyClass.subClassOf);
+    if (ontologyClass.subClassOf instanceof Array)
+      for (var i = 0; i < ontologyClass.subClassOf.length; i++) {
+        if (this.hasClassProperties(ontologyClass.subClassOf[i]))
+          return true;
+      }
   }
   return false;
 }
@@ -833,6 +968,87 @@ RDF.clearTable = function()
   OAT.Dom.show(noTR);
 }
 
+// Item Types functions
+RDF.showRDF = function(prefix, format)
+{
+  function preparePropertiesWork(prefix, ontologyNo, itemNo) {
+    var form = document.forms['page_form'];
+    var itemProperties = [];
+    for (var L = 0; L < form.elements.length; L++) {
+      if (!form.elements[L])
+        continue;
+
+      var ctrl = form.elements[L];
+      if (typeof(ctrl.type) == 'undefined')
+        continue;
+
+      if (ctrl.name.indexOf(prefix+"_item_"+ontologyNo+"_prop_"+itemNo+"_fld_1_") != 0)
+        continue;
+
+      var propertyNo = ctrl.name.replace(prefix+"_item_"+ontologyNo+"_prop_"+itemNo+"_fld_1_", "");
+      var propertyName = ctrl.value;
+      var propertyType = $v(prefix+"_item_"+ontologyNo+"_prop_"+itemNo+"_fld_2_"+propertyNo);
+      var propertyValue = $v(prefix+"_item_"+ontologyNo+"_prop_"+itemNo+"_fld_3_"+propertyNo);
+      var propertyLanguage = $v(prefix+"_item_"+ontologyNo+"_prop_"+itemNo+"_fld_4_"+propertyNo);
+      if (propertyType == 'object') {
+        var item = RDF.getItemByName(propertyValue);
+        if (item)
+          propertyValue = item.id;
+      }
+      itemProperties.push({"name": propertyName, "value": propertyValue, "type": propertyType, "language": propertyLanguage});
+    }
+    return itemProperties;
+  }
+
+  var L = 0;
+  var form = document.forms['page_form'];
+  var ontologies = [];
+  for (var N = 0; N < form.elements.length; N++)
+  {
+    if (!form.elements[N])
+      continue;
+
+    var ctrl = form.elements[N];
+    if (typeof(ctrl.type) == 'undefined')
+      continue;
+
+    if (ctrl.name.indexOf(prefix+"_fld_2_") != 0)
+      continue;
+
+    var ontologyNo = ctrl.name.replace(prefix+"_fld_2_", "");
+    var ontologyName = ctrl.value;
+    var ontologyItems = [];
+    for (var M = 0; M < form.elements.length; M++)
+    {
+      if (!form.elements[M])
+        continue;
+
+      var ctrl = form.elements[M];
+      if (typeof(ctrl.type) == 'undefined')
+        continue;
+
+      if (ctrl.name.indexOf(prefix+"_item_"+ontologyNo+"_fld_2_") != 0)
+        continue;
+
+      var itemNo = ctrl.name.replace(prefix+"_item_"+ontologyNo+"_fld_2_", "");
+      var itemName = ctrl.value;
+      var itemProperties = preparePropertiesWork(prefix, ontologyNo, itemNo);
+      ontologyItems.push({"id": itemNo, "className": itemName, "properties": itemProperties});
+    }
+    ontologies.push({"id": ''+L++, "ontology": ontologyName, "items": ontologyItems});
+  }
+  var items = OAT.JSON.stringify(ontologies);
+	var x = function(data) {
+    if (!rdfDialog) {
+      rdfDialog = new OAT.Dialog("Show Data", "rdfDiv", {width: 800, height: 500, resize: 0, modal: 1, buttons: 1});
+      OAT.Dom.show('rdfDiv');
+    }
+    $('rdfData').innerHTML = data;
+		rdfDialog.show();
+	}
+  OAT.AJAX.GET('/ods/api/objects.rdf', 'items='+encodeURIComponent(items)+'&format='+format, x);
+}
+
 // Item Types functions
 RDF.showItemTypes = function()
 {
@@ -1020,13 +1236,14 @@ RDF.showItemsTable = function(itemType)
     var prefixItem = prefix + '_item_' + No;
 
     var fld = OAT.Dom.create('span');
+    fld.title = 'Add Element';
     fld.onclick = function(){var id = RDF.newItemId(); TBL.createRow(prefixItem, null, {No: id, fld_1: {mode: 45, cssText: 'display: none;'}, fld_2: {mode: 44, itemType: itemType, labelValue: 'New Item: '}, btn_1: {mode: 42}, btn_2: {mode: 43}});};
     OAT.Dom.addClass(fld, 'button pointer');
 
     var img = OAT.Dom.create('img');
     img.src = '/ods/images/icons/add_16.png';
-    img.alt = 'Add row';
-    img.title = fld.alt;
+    img.alt = 'Add Element';
+    img.title = img.alt;
     OAT.Dom.addClass(img, 'button');
 
     fld.appendChild(img);
@@ -1168,11 +1385,10 @@ RDF.addItemToSelects = function(item)
   var tbl = $(this.tablePrefix+'_tbl');
   if (!tbl) {return;}
 
-  var selects = tbl.getElementsByTagName('select');
-  if (!selects) {return;}
-  for (var i = 0; i < selects.length; i++)
-  {
-    var obj = selects[i];
+  var combolists = tbl.getElementsByTagName('input');
+  if (!combolists) {return;}
+  for (var i = 0; i < combolists.length; i++) {
+    var obj = combolists[i];
     if ((obj.id.indexOf('_prop_') != -1) && (obj.id.indexOf('_fld_1_') != -1) && (obj.value != ''))
     {
       var ontologyClassProperty = this.getOntologyClassProperty(obj.item.className, obj.value);
@@ -1182,7 +1398,7 @@ RDF.addItemToSelects = function(item)
         {
           if (item.className == ontologyClassProperty.objectProperties[j])
           {
-      	    var fld = $(obj.id.replace(/_fld_1_/, '_fld_2_'));
+      	    var fld = $(obj.id.replace(/_fld_1_/, '_fld_3_'));
             if (fld && fld.combolist)
               fld.combolist.addOption(RDF.getItemName(item));
     	    }
@@ -1207,16 +1423,16 @@ RDF.itemInSelects = function(item, mode)
   var tbl = $(this.tablePrefix+'_tbl');
   if (!tbl) {return;}
 
-  var selects = tbl.getElementsByTagName('select');
-  if (!selects) {return;}
-  for (var i = 0; i < selects.length; i++) {
-    var obj = selects[i];
+  var combolists = tbl.getElementsByTagName('input');
+  if (!combolists) {return;}
+  for (var i = 0; i < combolists.length; i++) {
+    var obj = combolists[i];
     if ((obj.id.indexOf('_prop_') != -1) && (obj.id.indexOf('_fld_1_') != -1) && (obj.value != '')) {
       var ontologyClassProperty = this.getOntologyClassProperty(obj.item.className, obj.value);
       if (ontologyClassProperty.objectProperties) {
         for (var j = 0; j < ontologyClassProperty.objectProperties.length; j++) {
           if (item.className == ontologyClassProperty.objectProperties[j]) {
-      	    var fld = $(obj.id.replace(/_fld_1_/, '_fld_2_'));
+      	    var fld = $(obj.id.replace(/_fld_1_/, '_fld_3_'));
             if (fld && fld.combolist) {
               if ((fld.value == RDF.getItemName(item)) && (mode == 'check'))
                     return confirm ('The selected object is used. Delete?');
@@ -1310,13 +1526,14 @@ RDF.showPropertiesTable = function(item)
       var prefixProp = prefix + '_prop_' + No;
 
       var fld = OAT.Dom.create('span');
+      fld.title = 'Add Property';
       fld.onclick = function(){TBL.createRow(prefixProp, null, {fld_1: {mode: 46, item: item}, fld_2: {mode: 48, item: item}, fld_3: {mode: 47, item: item}, fld_4: {mode: 49, item: item}});};
       OAT.Dom.addClass(fld, 'button pointer');
 
       var img = OAT.Dom.create('img');
       img.src = '/ods/images/icons/add_16.png';
-      img.alt = 'Add row';
-      img.title = fld.alt;
+      img.alt = 'Add Property';
+      img.title = img.alt;
       OAT.Dom.addClass(img, 'button');
 
       fld.appendChild(img);
@@ -1343,17 +1560,19 @@ RDF.showPropertiesTable = function(item)
 
 RDF.changePropertyValue = function(obj)
 {
-  var S = obj.parentNode.id;
+  var fld = obj.input;
+  var fldTd = findParent(fld, 'td');
+  var S = fldTd.id;
 
-  var fldName = (obj.id).replace(/fld_1/, 'fld_2');
+  var fldName = (fld.id).replace(/fld_1/, 'fld_2');
   var td = $(S.substr(0,S.lastIndexOf('_')+1)+'2');
-  TBL.createCell48(td, '', fldName, 0, {item: obj.item, value: {name: obj.value}});
+  TBL.createCell48(td, '', fldName, 0, {item: fld.item, value: {name: fld.value}});
 
-  var fldName = (obj.id).replace(/fld_1/, 'fld_3');
+  var fldName = (fld.id).replace(/fld_1/, 'fld_3');
   var td = $(S.substr(0,S.lastIndexOf('_')+1)+'3');
-  TBL.createCell47(td, '', fldName, 0, {item: obj.item, value: {name: obj.value}});
+  TBL.createCell47(td, '', fldName, 0, {item: fld.item, value: {name: fld.value}});
 
-  var fldName = (obj.id).replace(/fld_1/, 'fld_4');
+  var fldName = (fld.id).replace(/fld_1/, 'fld_4');
   var td = $(S.substr(0,S.lastIndexOf('_')+1)+'4');
-  TBL.createCell49(td, '', fldName, 0, {item: obj.item, value: {name: obj.value}});
+  TBL.createCell49(td, '', fldName, 0, {item: fld.item, value: {name: fld.value}});
 }
