@@ -1,10 +1,10 @@
 --
---  $Id: facebook.sql,v 1.14.2.7 2010/12/10 16:04:14 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -20,6 +20,8 @@
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 --
 
+use DB;
+
 wa_exec_no_error('drop type DB.DBA.FacebookRestClient');
 wa_exec_no_error('drop type Facebook');
 
@@ -186,8 +188,9 @@ create method post_request(
     post_string:=string_output_string(aResult);
 
      declare ret_header, rq_header any;
-  rq_header := 'Content-type: application/x-www-form-urlencoded \r\nUser-Agent: Facebook API VSP Client 1.1';
 
+  commit work;
+  rq_header := 'Content-type: application/x-www-form-urlencoded \r\nUser-Agent: Facebook API VSP Client 1.1';
     _result:=http_get ('http://api.facebook.com/restserver.php', ret_header, 'POST', rq_header, post_string); --'127.0.0.1:8888'
      return _result;
 }
@@ -1151,27 +1154,14 @@ create procedure _get_cookie_vec (in lines any)
 ;
 
 create procedure _get_ods_fb_settings (
-  out fb_settings any,
-  in fb_user_id integer := 2)
+  out fb_settings any)
 {
-   declare dba_options,fb_dba_options any;
-   declare exit handler for sqlstate '*' {return 0;};
-
    fb_settings := null;
-  fb_dba_options := null;
-   dba_options := (select US_KEY from WA_USER_SVC where US_U_ID = fb_user_id and US_SVC = 'FBKey');
-   if(length(dba_options) >0)
+  for (select a_key, a_secret from OAUTH.DBA.APP_REG where a_owner = 0 and a_name = 'Facebook API') do
    {
-     fb_dba_options:=replace(dba_options,'\r\n','&');
-     fb_dba_options:=replace(fb_dba_options,'\n','&');
-     fb_dba_options:=split_and_decode(fb_dba_options);
-   }
-  if (fb_dba_options is not null and length(trim(get_keyword('key',fb_dba_options)))> 4 and length(trim(get_keyword('secret',fb_dba_options)))> 4)
-   {
-      fb_settings:=vector(trim(get_keyword('key',fb_dba_options)),trim(get_keyword('secret',fb_dba_options)));
+    fb_settings := vector (a_key, a_secret);
       return 1;
    }
-
    return 0;
 }
 ;
