@@ -1,12 +1,12 @@
 --
---  $Id: openid.sql,v 1.17.2.2 2010/07/07 18:53:57 source Exp $
+--  $Id$
 --
 --  OpenID protocol support.
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -58,10 +58,14 @@ OPENID_INIT ();
 
 create procedure yadis (in uname varchar, in tp varchar := null)
 {
-  declare url, srv varchar;
+  declare url, ssl, srv varchar;
   if (tp not in ('person/', 'organization/'))
     tp := '';
   url := db.dba.wa_link (1, '/dataspace/'||tp||uname);
+  ssl := ODS.ODS_API.getDefaultHttps ();
+  if (exists (select 1 from DB..WA_SETTINGS where WS_HTTPS = 1) and ssl is not null)
+    srv := 'https://' || ssl || '/openid';
+  else
   srv := db.dba.wa_link (1, '/openid');
   for select WAUI_OPENID_URL, WAUI_OPENID_SERVER, WAUI_NICK
     from DB.DBA.WA_USER_INFO, DB.DBA.SYS_USERS where WAUI_U_ID = U_ID and U_NAME = uname
@@ -112,6 +116,22 @@ create procedure sreg_ns_v1 ()
   return 'http://openid.net/extensions/sreg/1.1';
 }
 ;
+
+create procedure oauth_ns ()
+{
+  return 'http://specs.openid.net/extensions/oauth/1.0';
+}
+;
+
+create procedure check_session (in _identity varchar, in sid varchar)
+{
+  declare uname varchar;
+  uname := oid_get_user_id (_identity);
+  delete from DB.DBA.VSPX_SESSION where VS_SID = sid and VS_REALM = 'wa' and VS_UID <> uname;
+  return row_count ();
+}
+;
+
 create procedure server
 	(
 	 in "openid.mode" varchar := 'unknown'
@@ -119,8 +139,8 @@ create procedure server
 	__SOAP_HTTP 'text/html'
 {
   declare ret, lines, params, oid_sid, cookies_vec any;
-  declare ns varchar;
-  declare ver int;
+  declare ns, oauth_ns, op varchar;
+  declare ver, pos int;
 
   params := http_param ();
   lines := http_request_header ();
@@ -131,11 +151,25 @@ create procedure server
   cookies_vec := DB.DBA.vsp_ua_get_cookie_vec (lines);
   oid_sid := get_keyword ('openid.sid', cookies_vec);
   ns := get_keyword ('openid.ns', params, 'http://openid.net/signon/1.1');
+  oauth_ns := 'openid.oauth.';
+  pos := position (oauth_ns (), params);
+  if (pos > 1)
+    {
+      declare tmp any;
+      tmp := params [pos - 2];
+      tmp := sprintf_inverse (tmp, '%s.%s.%s', 0);
+      if (length (tmp) = 3 and tmp[0] = 'openid' and tmp[1] = 'ns')
+	oauth_ns := 'openid.' || tmp[2] || '.';
+    }
 
   if (ns = ns_v2 ())
     ver := 2;
   else
     ver := 1;
+  if (is_https_ctx ())
+    op := sprintf ('https://%{WSHost}s/openid');
+  else
+    op := sprintf ('http://%{WSHost}s/openid');
 
   if ("openid.mode" = 'associate')
     ret := associate (ver,
@@ -155,7 +189,10 @@ create procedure server
 	0,
 	get_keyword ('openid.sreg.required', params),
 	get_keyword ('openid.sreg.optional', params),
-	get_keyword ('openid.sreg.policy_url', params)
+	get_keyword ('openid.sreg.policy_url', params),
+	get_keyword (oauth_ns || 'consumer', params),
+	get_keyword (oauth_ns || 'scope', params),
+	op
 	);
   else if ("openid.mode" = 'checkid_setup')
     ret := checkid_setup (
@@ -167,7 +204,10 @@ create procedure server
 	oid_sid,
 	get_keyword ('openid.sreg.required', params),
 	get_keyword ('openid.sreg.optional', params),
-	get_keyword ('openid.sreg.policy_url', params)
+	get_keyword ('openid.sreg.policy_url', params),
+	get_keyword (oauth_ns || 'consumer', params),
+	get_keyword (oauth_ns || 'scope', params),
+	op
 	);
   else if ("openid.mode" = 'check_authentication')
     ret := check_authentication (
@@ -391,7 +431,7 @@ create procedure get_user_details (in gr varchar, in _identity varchar)
 	  if (regexp_match ('http://[^/]+/dataspace/(person|organization)/.+', _identity) is not null)
 	    {
 	      arr1 := sprintf_inverse (_identity, '%s://%s/dataspace/%s/%s', 1);
-	      if (length (arr1) = 4 and length (arr2) = 3 and arr1[0] = arr2[0] and arr1[1] = arr2[1] and 
+	      if (length (arr1) = 4 and length (arr2) = 3 and arr1[0] = arr2[0] and arr1[1] = arr2[1] and
 		  exists (select 1 from DB.DBA.SYS_USERS, DB.DBA.WA_USER_INFO where WAUI_U_ID = U_ID and U_NAME = arr1[3]))
 		goto verified;
 	    }
@@ -450,11 +490,14 @@ create procedure checkid_immediate
 	 in flag int := 0, -- called via checkid_setup
 	 in sreg_required varchar := null,
 	 in sreg_optional varchar := null,
-	 in policy_url varchar := null
+	 in policy_url varchar := null,
+	 in oauth_consumer varchar := null,
+	 in oauth_scope varchar := null,
+	 in op varchar := null
     	)
 {
   declare signature, rhf, delim any;
-  declare login, hdr, usr, ns, ns_sign varchar;
+  declare login, hdr, usr, ns, ns_sign, tok, oauthf, oauth_resp varchar;
 
   if (trust_root is null)
     trust_root := _identity;
@@ -463,10 +506,12 @@ create procedure checkid_immediate
     return 'error:no_identity';
   if (length (return_to) = 0)
     return 'error:no_return_to';
-
---  dbg_obj_print ('checkid_immediate', sid, ' ver ', ver);
+  check_session (_identity, sid);
+  --dbg_obj_print_vars ('checkid_immediate', sid, ver);
   ns := '';
   ns_sign := '';
+  oauthf := '';
+  oauth_resp := '';
   if (isstring (ver))
     ver := atoi (ver);
   if (ver = 2)
@@ -484,9 +529,13 @@ create procedure checkid_immediate
       else
         delim := '?';
       login :=
-      sprintf ('%s?return_to=%U&identity=%U&assoc_handle=%U&trust_root=%U&sreg_required=%U&sreg_optional=%U&policy_url=%U&ver=%d',
-	    get_login_url (_identity), return_to, _identity, coalesce (assoc_handle, ''), trust_root,
-	    coalesce (sreg_required, ''), coalesce (sreg_optional, ''), coalesce (policy_url, ''), ver);
+      sprintf ('%s?return_to=%U&identity=%U&assoc_handle=%U&trust_root=%U&sreg_required=%U&sreg_optional=%U&policy_url=%U&ver=%d&op_endpoint=%U',
+	    get_login_url (_identity), return_to, _identity, coalesce (assoc_handle, ''), coalesce (trust_root, ''),
+	    coalesce (sreg_required, ''), coalesce (sreg_optional, ''), coalesce (policy_url, ''), ver, op);
+      if (length (oauth_consumer))
+        {
+          login := login || sprintf ('&oauth_consumer=%U&oauth_scope=%U', oauth_consumer, oauth_scope);
+        }
       --dbg_obj_print (sprintf ('Location: %s?openid.mode=id_res&openid.user_setup_url=%U\r\n', return_to, login));
       http_header (http_header_get () || sprintf ('Location: %s%sopenid.mode=id_res%s&openid.user_setup_url=%U\r\n',
 	    return_to, delim, ns, login));
@@ -582,6 +631,11 @@ create procedure checkid_immediate
 		  values (assoc_handle, ss_key, ss_key_data, '3DES', dateadd ('hour', 1, now()));
 	    }
 	}
+      -- make OAuth request_token call
+      if (length (oauth_consumer) > 0)
+	tok := OAUTH..hybrid_request_token (sid, oauth_consumer);
+      else
+        tok := null;
 
       rhf := WS.WS.PARSE_URI (return_to);
       if (rhf[4] <> '')
@@ -600,10 +654,15 @@ create procedure checkid_immediate
 
       if (ver = 2)
 	{
-	  declare op, nonce varchar;
-	  op := db.dba.wa_link (1, '/openid');
-	  if (0 and is_https_ctx ())
-	    op := 'https://' || DB.DBA.WA_HTTPS() || '/openid';
+	  declare nonce varchar;
+	  --op := db.dba.wa_link (1, '/openid');
+	  if (op is null)
+	    {
+	  if (is_https_ctx ())
+	    op := sprintf ('https://%{WSHost}s/openid');
+	      else
+	    op := sprintf ('http://%{WSHost}s/openid');
+	    }
 	  nonce := DB.DBA.date_iso8601 (dt_set_tz (curdatetime (0), 0)) || cast (msec_time () as varchar);
 	  ns := sprintf ('&openid.ns=%U&openid.ns.sreg=%U&openid.op_endpoint=%U&openid.response_nonce=%U&openid.claimed_id=%U',
 	  ns_v2 (), sreg_ns_v1 (), op, nonce, _identity);
@@ -634,7 +693,20 @@ create procedure checkid_immediate
 		}
 	    }
 	}
-
+      -- add ns.oauth, oauth.request_token and oauth.scope to the signature string
+      if (tok is not null)
+        {
+	  oauthf := ',ns.oauth,oauth.request_token';
+	  http (sprintf ('ns.oauth:%s\x0A', oauth_ns ()), ses);
+	  http (sprintf ('oauth.request_token:%s\x0A', tok), ses);
+	  oauth_resp := sprintf ('&openid.ns.oauth=%U&openid.oauth.request_token=%U', oauth_ns (), tok);
+	  if (length (oauth_scope) > 0)
+	    {
+	      oauthf := oauthf || ',oauth.scope';
+	      http (sprintf ('oauth.scope:%s\x0A', oauth_scope), ses);
+	      oauth_resp := oauth_resp || sprintf ('&openid.oauth.scope=%U', oauth_scope);
+	    }
+        }
 
       if (user <> 'OpenID')
         set_user_id ('OpenID');
@@ -648,9 +720,9 @@ create procedure checkid_immediate
       if (length (assoc_handle) = 0)
 	assoc_handle := '';
       hdr :=  sprintf ('Location: %s%sopenid.mode=id_res%s&openid.identity=%U&openid.return_to=%U'||
-      			'&openid.assoc_handle=%U&openid.signed=%U&openid.sig=%U%s%s\r\n',
+      			'&openid.assoc_handle=%U&openid.signed=%U&openid.sig=%U%s%s%s\r\n',
 	    		return_to, delim, ns, _identity, return_to, coalesce (assoc_handle, ''),
-			ns_sign||'mode,identity,return_to,assoc_handle'||sregf, signature, inv, sreg);
+			ns_sign||'mode,identity,return_to,assoc_handle'||sregf||oauthf, signature, inv, sreg, oauth_resp);
 --      dbg_obj_print ('hdr:', hdr);
       http_header (http_header_get () || hdr);
     }
@@ -668,10 +740,14 @@ create procedure checkid_setup
 	 in sid varchar,
 	 in sreg_required varchar := null,
 	 in sreg_optional varchar := null,
-	 in policy_url varchar := null
+	 in policy_url varchar := null,
+	 in oauth_consumer varchar := null,
+	 in oauth_scope varchar := null,
+	 in op varchar := null
 	 )
 {
   declare rhf, delim, login, ss_key any;
+  check_session (_identity, sid);
   if (not exists (select 1 from DB.DBA.VSPX_SESSION where VS_SID = sid and VS_REALM = 'wa'))
     {
       rhf := WS.WS.PARSE_URI (return_to);
@@ -682,14 +758,19 @@ create procedure checkid_setup
       http_request_status ('HTTP/1.1 302 Found');
 
       login :=
-      sprintf ('%s?return_to=%U&identity=%U&assoc_handle=%U&trust_root=%U&sreg_required=%U&sreg_optional=%U&policy_url=%U&ver=%d',
-	    get_login_url (_identity), return_to, _identity, coalesce (assoc_handle, ''), trust_root,
-	    coalesce (sreg_required, ''), coalesce (sreg_optional, ''), coalesce (policy_url, ''), ver);
+      sprintf ('%s?return_to=%U&identity=%U&assoc_handle=%U&trust_root=%U&sreg_required=%U&sreg_optional=%U&policy_url=%U&ver=%d&op_endpoint=%U',
+	    get_login_url (_identity), return_to, _identity, coalesce (assoc_handle, ''), coalesce (trust_root, ''),
+	    coalesce (sreg_required, ''), coalesce (sreg_optional, ''), coalesce (policy_url, ''), ver, op);
+      if (length (oauth_consumer))
+        {
+          login := login || sprintf ('&oauth_consumer=%U&oauth_scope=%U', oauth_consumer, oauth_scope);
+        }
       http_header (http_header_get () || sprintf ('Location: %s\r\n', login));
       --http_header (http_header_get () || sprintf ('Location: %s%sopenid.mode=cancel\r\n', return_to, delim));
       return '';
     }
-  return checkid_immediate (ver, _identity, assoc_handle, return_to, trust_root, sid, 1, sreg_required, sreg_optional, policy_url);
+  return checkid_immediate (ver, _identity, assoc_handle, return_to, trust_root, sid, 1, sreg_required, sreg_optional, policy_url,
+      oauth_consumer, oauth_scope, op);
 };
 
 create procedure cancel (in ver int := 1, in return_to varchar)
@@ -731,8 +812,20 @@ create procedure oid_set_sid (in sid varchar, in pars any)
 
 create procedure oid_get_user_id (in _identity any)
 {
-  declare iarr, uname any;
+  declare iarr, uname, webid any;
+  declare webid, gr varchar;
   uname := null;
+  --dbg_obj_print_vars (_identity);
+  if (strchr (_identity, '@') is not null)
+    {
+      gr := sioc..get_graph ();
+      webid := (SPARQL
+	       PREFIX owl: <http://www.w3.org/2002/07/owl#>
+	       PREFIX foaf: <http://xmlns.com/foaf/0.1/>
+	       SELECT ?openid WHERE { graph `iri(?:gr)` { ?webid owl:sameAs `iri(?:_identity)` ; foaf:openid ?openid . }});
+      if (webid is not null)
+        _identity := webid;
+    }
   if (regexp_match ('http://[^/]+/dataspace/(person|organization)/.+', _identity) is not null)
     {
       iarr := sprintf_inverse (_identity, 'http://%s/dataspace/%s/%s', 1);
@@ -796,11 +889,13 @@ create procedure get_login_url (in _identity any)
   uname := oid_get_user_id (_identity);
   whenever not found goto nf;
   select UC_CERT, UC_LOGIN into cert, enabled from DB.DBA.WA_USER_CERTS, DB.DBA.SYS_USERS where UC_U_ID = U_ID and U_NAME = uname;
-  if (length (cert) and enabled)
+  if (length (cert) and enabled) -- was and exists (select 1 from DB..WA_SETTINGS where WS_HTTPS = 1))
     {
       return wa_ssl_link (1, 'openid_login.vspx');
     }
   nf:
+  if (is_https_ctx ())
+    return sprintf ('https://%{WSHost}s/ods/openid_login.vspx');
   return DB.DBA.wa_link(1, 'openid_login.vspx');
 }
 ;
