@@ -1,11 +1,11 @@
 #!/bin/sh
 #
-#  $Id: make_vad.sh,v 1.22.2.9 2011/02/03 10:26:16 source Exp $
+#  $Id$
 #
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2006 OpenLink Software
+#  Copyright (C) 1998-2013 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -28,9 +28,8 @@ LANG=C
 LC_ALL=POSIX
 export LANG LC_ALL
 
-
 LOGDIR=`pwd`
-VERSION="0.3.58"
+VERSION="1.0.0"
 LOGFILE="${LOGDIR}/make_vad.log"
 STICKER="make_vad.xml"
 PACKDATE=`date +"%Y-%m-%d %H:%M"`
@@ -59,21 +58,6 @@ else
   LN="ln -fs"
   RM="rm -f"
 fi
-VOS=0
-if [ -f ../../autogen.sh ]
-then
-    VOS=1
-fi
-
-if [ "z$SERVER" = "z" ]  
-then
-    if [ "x$HOST_OS" != "x" ]
-    then
-	SERVER=virtuoso-odbc-t.exe
-    else
-	SERVER=virtuoso
-    fi
-fi
 
 rm -rf vad
 
@@ -86,14 +70,55 @@ else
   myrm=$RM
 fi
 
-VERSION_INIT() 
+
+VOS=0
+if [ -f ../../autogen.sh ]
+then
+    VOS=1
+fi
+
+version_init() 
 {
+  if [ $VOS -eq 1 ]
+  then
+      if [ -f vad_version ]
+      then
+	  VERSION=`cat vad_version`
+      else
+        LOG "The vad_version does not exist, please verify your checkout"
+	exit 1
+      fi
+  else
   rm -f version.tmp
-  for i in `find . -name 'Entries' | grep -v "vad/"`; do
-        cat $i | grep "^[^D].*" | cut -f 3 -d "/" | sed -e "s/1\.//g" >> version.tmp
+      for i in `find . -name 'Entries' | grep -v "vad/" | grep -v "/tests/"`; do
+	  cat "$i" | grep -v "version\."| grep "^[^D].*" | cut -f 3 -d "/" | sed -e "s/1\.//g" >> version.tmp
   done
-  VERSION=`cat version.tmp | awk ' BEGIN { cnt=78 } { cnt = cnt + $1} END { printf "1.%02.02f", cnt/100 + 1}'`
+      LANG=POSIX
+      export LANG
+
+      BASE="0"
+#      echo $BASE
+      if [ -f version.base ] ; then
+	  BASE=`cat version.base`
+      fi
+
+      VERSION=`cat version.tmp | awk ' BEGIN { cnt=10 } { cnt = cnt + $1 } END { print cnt }'`
+
+      VERSION=`expr $BASE + $VERSION`
+      CURR_VERSION=$VERSION
+      if [ -f version.curr ] ; then
+	  CURR_VERSION=`cat version.curr`
+      fi
+      if [ $CURR_VERSION -gt $VERSION ] ; then
+	  BASE=`expr $CURR_VERSION - $VERSION + 1`
+	  echo $BASE > version.base
+	  VERSION=$CURR_VERSION
+      fi
+      echo $VERSION > version.curr
+      VERSION=`echo $VERSION | awk ' { printf "1.%02.02f", $1/100 }'`
   rm -f version.tmp
+      echo "$VERSION" > vad_version
+  fi
 }
 
 virtuoso_start() {
@@ -226,7 +251,7 @@ sticker_init() {
   echo "  <name package=\"Gallery\">" >> $STICKER
   echo "    <prop name=\"Title\" value=\"ODS Gallery\"/>" >> $STICKER
   echo "    <prop name=\"Developer\" value=\"OpenLink Software\"/>" >> $STICKER
-  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2011 OpenLink Software\"/>" >> $STICKER
+  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2013 OpenLink Software\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.com/virtuoso\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.co.uk/virtuoso\"/>" >> $STICKER
   echo "  </name>" >> $STICKER
@@ -239,8 +264,8 @@ sticker_init() {
   echo "  <require>" >> $STICKER
   echo "   <name package=\"Framework\">" >> $STICKER
   echo "   </name>" >> $STICKER
-  echo "    <versions_later package=\"1.71.75\">" >> $STICKER
-  echo "      <prop name=\"Date\" value=\"2010-06-02 12:00\" />" >> $STICKER
+  echo "    <versions_later package=\"1.81.91\">" >> $STICKER
+  echo "      <prop name=\"Date\" value=\"2011-05-16 12:00\" />" >> $STICKER
   echo "      <prop name=\"Comment\" value=\"An incompatible version of the ODS Framework\" />" >> $STICKER
   echo "   </versions_later>" >> $STICKER
   echo "  </require>" >> $STICKER
@@ -401,7 +426,7 @@ echo '----------------------'
 
 STOP_SERVER
 directory_clean
-VERSION_INIT
+version_init
 directory_init
 virtuoso_init
 sticker_init
@@ -409,7 +434,6 @@ vad_create
 virtuoso_shutdown
 STOP_SERVER
 chmod 644 ods_gallery_dav.vad
-directory_clean
 
 CHECK_LOG
 RUN egrep  '"\*\*.*FAILED:|\*\*.*ABORTED:"' "$LOGFILE"
@@ -419,5 +443,7 @@ then
 	exit 1
 fi
 
+directory_clean
+
 BANNER "COMPLETED VAD PACKAGING"
 exit 0
