@@ -1,11 +1,11 @@
 #!/bin/sh
 #
-#  $Id: make_vad.sh,v 1.34.2.14 2010/07/09 14:37:01 source Exp $
+#  $Id$
 #
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2006 OpenLink Software
+#  Copyright (C) 1998-2013 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -30,7 +30,7 @@ export LANG LC_ALL
 
 MODE=$1
 LOGDIR=`pwd`
-VERSION="1.4.111"
+VERSION="1.5.122"
 LOGFILE="${LOGDIR}/vad_make.log"
 STICKER_DAV="vad_dav.xml"
 STICKER_FS="vad_filesystem.xml"
@@ -45,7 +45,7 @@ VAD_DAV="$VAD_NAME"_dav.vad
 VAD_FS="$VAD_NAME"_filesystem.vad
 DSN="$HOST:$PORT"
 HOST_OS=`uname -s | grep WIN`
-NEED_VERSION=04.50.2905
+NEED_VERSION=06.01.3127	 	# VOS
 
 if [ "x$HOST_OS" != "x" ]
 then
@@ -63,11 +63,6 @@ else
   LN="ln -fs"
   RM="rm -f"
 fi
-VOS=0
-if [ -f ../../autogen.sh ]
-then
-    VOS=1
-fi
 
 if [ "z$SERVER" = "z" ]  
 then
@@ -90,6 +85,56 @@ else
   myrm=$RM
 fi
 
+VOS=0
+if [ -f ../../autogen.sh ]
+then
+    VOS=1
+fi
+
+version_init()
+{
+  if [ $VOS -eq 1 ]
+  then
+      if [ -f vad_version ]
+      then
+	  VERSION=`cat vad_version`
+      else
+        LOG "The vad_version does not exist, please verify your checkout"
+	exit 1
+      fi
+  else
+      rm -f version.tmp
+      for i in `find . -name 'Entries' | grep -v "vad/" | grep -v "/tests/"`; do
+	  cat "$i" | grep -v "version\."| grep "^[^D].*" | cut -f 3 -d "/" | sed -e "s/1\.//g" >> version.tmp
+      done
+      LANG=POSIX
+      export LANG
+
+      BASE="0"
+#      echo $BASE
+      if [ -f version.base ] ; then
+	  BASE=`cat version.base`
+      fi
+
+      VERSION=`cat version.tmp | awk ' BEGIN { cnt=10 } { cnt = cnt + $1 } END { print cnt }'`
+
+      VERSION=`expr $BASE + $VERSION`
+      CURR_VERSION=$VERSION
+      if [ -f version.curr ] ; then
+	  CURR_VERSION=`cat version.curr`
+      fi
+      if [ $CURR_VERSION -gt $VERSION ] ; then
+	  BASE=`expr $CURR_VERSION - $VERSION + 1`
+	  echo $BASE > version.base
+	  VERSION=$CURR_VERSION
+      fi
+      echo $VERSION > version.curr
+      VERSION=`echo $VERSION | awk ' { printf "1.%02.02f", $1/100 }'`
+      rm -f version.tmp
+      echo "$VERSION" > vad_version
+  fi
+}
+
 virtuoso_start() {
   echo "Starting $SERVER"
   echo $BUILD
@@ -225,7 +270,7 @@ sticker_init() {
   echo "  <name package=\"Mail\">" >> $STICKER
   echo "    <prop name=\"Title\" value=\"ODS Mail\"/>" >> $STICKER
   echo "    <prop name=\"Developer\" value=\"OpenLink Software\"/>" >> $STICKER
-  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2010 OpenLink Software\"/>" >> $STICKER
+  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2013 OpenLink Software\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.com/virtuoso\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.co.uk/virtuoso\"/>" >> $STICKER
   echo "  </name>" >> $STICKER
@@ -237,8 +282,8 @@ sticker_init() {
   echo "<dependencies>" >> $STICKER
   echo "  <require>" >> $STICKER
   echo "    <name package=\"Framework\"/>" >> $STICKER
-  echo "    <versions_later package=\"1.73.21\">" >> $STICKER
-  echo "      <prop name=\"Date\" value=\"2010-07-08 12:00\" />" >> $STICKER
+  echo "    <versions_later package=\"1.82.09\">" >> $STICKER
+  echo "      <prop name=\"Date\" value=\"2011-05-20 12:00\" />" >> $STICKER
   echo "      <prop name=\"Comment\" value=\"An incompatible version of the ODS Framework\" />" >> $STICKER
   echo "    </versions_later>" >> $STICKER
   echo "  </require>" >> $STICKER
@@ -385,6 +430,7 @@ echo '----------------------'
 
 STOP_SERVER
 directory_clean
+version_init
 directory_init
 virtuoso_init
 if [ "$MODE" = "" ] || [ "$MODE" = "1" ]
@@ -401,7 +447,6 @@ virtuoso_shutdown
 STOP_SERVER
 chmod 644 ods_webmail_dav.vad
 chmod 644 ods_webmail_filesystem.vad
-directory_clean
 
 CHECK_LOG
 RUN egrep  '"\*\*.*FAILED:|\*\*.*ABORTED:"' "$LOGFILE"
@@ -411,5 +456,7 @@ then
 	exit 1
 fi
 
+directory_clean
+
 BANNER "COMPLETED VAD PACKAGING"
 exit 0
