@@ -2,7 +2,7 @@
 //  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 //  project.
 //  
-//  Copyright (C) 1998-2006 OpenLink Software
+//  Copyright (C) 1998-2013 OpenLink Software
 //  
 //  This project is free software; you can redistribute it and/or modify it
 //  under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 //  
 //  
 //
-// $Id: VirtuosoCommand.cs,v 1.7.2.1 2010/04/16 12:36:08 source Exp $
+// $Id$
 //
 
 using System;
@@ -32,6 +32,7 @@ using System.Text;
 using System.ComponentModel;
 using System.Drawing;
 #endif
+using System.Text.RegularExpressions;
 
 #if ODBC_CLIENT
 namespace OpenLink.Data.VirtuosoOdbcClient
@@ -458,15 +459,24 @@ namespace OpenLink.Data.Virtuoso
 
 			bool schemaOnly = SchemaOnlyDataReader (behavior);
 			string text = GetCommandText ();
+		        bool isSparql = text.TrimStart(null).StartsWith("sparql", StringComparison.OrdinalIgnoreCase);
 			if (schemaOnly)
 			{
 				if (!isPrepared)
 				{
+                    			if (!isSparql)
+						text = replaceNamedParams (text);
 					innerCommand.Prepare (text);
 				}
 			}
 			else
 			{
+				if (!isSparql && parameters != null && parameters.Count > 0)
+				{
+					VirtuosoParameterCollection _parameters = handleNamedParams (text, parameters);
+					Debug.Assert (_parameters.Count == parameters.Count, "Count mismatch in reordered parameter array");
+					text = replaceNamedParams(text);
+				}
 				innerCommand.SetParameters (parameters);
 				if (!isPrepared)
 				{
@@ -552,6 +562,8 @@ namespace OpenLink.Data.Virtuoso
 
 		protected override void Dispose (bool disposing)
 		{
+			try
+			{
 			if (disposing)
 			{
 				DisposeStatement ();
@@ -563,6 +575,12 @@ namespace OpenLink.Data.Virtuoso
 			connection = null;
 			transaction = null;
 		}
+			catch (Exception e)
+			{
+				Debug.WriteLineIf(CLI.FnTrace.Enabled,
+					"VirtuosoCommand.Dispose caught exception: " + e.Message);
+			}
+		}
 
 		internal void OnConnectionClose ()
 		{
@@ -702,5 +720,52 @@ innerCommand != null)
                     }
                 }
 #endif
+
+		// Replaces any '@<name>' named parameters in the SQL command 
+		// with '?' placeholders.
+		internal string replaceNamedParams (string sql) 
+		{
+			string paramPattern = @"@\w+";
+			Regex paramRegEx = new Regex(paramPattern);
+			return paramRegEx.Replace(sql, "?");
+		}
+
+		// Reorders the parameter collection so that the order of named
+		// parameters in the collection matches their order in the SQL 
+		// command.
+		internal VirtuosoParameterCollection handleNamedParams (string sql, VirtuosoParameterCollection _parameters)
+		{
+			string paramPattern = @"@\w+";
+			Regex paramRegEx =  new Regex(paramPattern);
+			Match match = paramRegEx.Match(sql);
+
+			// If statement doesn't contain any named parameters
+			if (!match.Success)
+				return _parameters;
+
+			VirtuosoParameterCollection newParameters = new VirtuosoParameterCollection (this);
+
+			while(match.Success)
+			{ 
+				string paramName = String.Copy(match.Value);
+
+				if (_parameters.IndexOf (paramName) >= 0)
+					newParameters.Add (_parameters[_parameters.IndexOf (paramName)]);
+				else
+				{
+					// Seems param name saved in parameter 
+					// collection may or may not have a 
+					// leading @
+					string paramNameWithoutPrefix = String.Copy(paramName.Substring(1));
+					if (_parameters.IndexOf (paramNameWithoutPrefix) >= 0)
+					newParameters.Add (_parameters[_parameters.IndexOf (paramNameWithoutPrefix)]);
+				}
+
+		 		match = match.NextMatch();
+			}
+   
+			return newParameters;
+		}
+
 	}
 }
