@@ -24,5 +24,3 @@ use DB;
 load DET_S3.sql;
 use DB;
 load Versioning/DET_Versioning.sql;
-#use DB;
-#load DET_WebMail.sql;
