@@ -1,9 +1,9 @@
 /*
- *  $Id: statusui.js,v 1.1.2.2 2010/03/12 09:01:02 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2009 OpenLink Software
+ *  Copyright (C) 2009-2013 OpenLink Software
  *
  *  See LICENSE file for details.
  *
@@ -24,8 +24,25 @@ iSPARQL.StatusUI = {
 	else
 	    $("statMsgElem").innerHTML = msg;
     },
+    init: function () {
+	OAT.Dom.show ("splashThrobber");
+    },
     errMsg: function (msg) {
 	$("statMsgErr").innerHTML = msg;
+    },
+    addCustomTemplate: function (div) {
+	var c = this.newTplCtr ();
+	OAT.Dom.append ([c, div]);
+    },
+    newTplCtr: function () {
+	var custCtr = OAT.Dom.create ("div",{className:"statusUICustCtr"});
+	OAT.Dom.append(["statMsgMain",custCtr]);
+	return custCtr;
+    },
+    absorb: function (ctr) {
+	var c = OAT.Dom.create ("div", {className: "statusUIAbsorbedCtr"})
+	OAT.Dom.unlink (ctr);
+	OAT.Dom.append ([c, ctr]);
     }
 };
 
@@ -33,5 +50,5 @@ iSPARQL.ErrorUI = {
     hide: function () {},
     clear: function () {},
     fromGenericXhr: function () {},
-    fromSparqlQuery: function () {},
+    fromSparqlQuery: function () {}
 };
