@@ -1,10 +1,10 @@
 /*
- *  $Id: VirtBulkUpdateHandler.java,v 1.6.2.3 2010/04/01 10:54:36 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2010 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -60,18 +60,8 @@ public class VirtBulkUpdateHandler extends SimpleBulkUpdateHandler {
 	VirtGraph _graph=(VirtGraph)this.graph;
 	List list = notify ? new ArrayList() : null;
 
-	try {
-		boolean autoCommit = _graph.getConnection().getAutoCommit();
-	        if (autoCommit)
-		    _graph.getConnection().setAutoCommit(false);
-		_graph.add(it, list);
-	        if (autoCommit)  {
-		    _graph.getConnection().commit();
-		    _graph.getConnection().setAutoCommit(true);
-		}
-	} catch (Exception e) {
-		throw new JenaException("Couldn't create transaction:"+e);
-	}
+	_graph.add(null, it, list);
+
 	if (notify)
 		manager.notifyAddIterator( graph, list);
     }
@@ -94,18 +84,8 @@ public class VirtBulkUpdateHandler extends SimpleBulkUpdateHandler {
 	VirtGraph _graph=(VirtGraph)this.graph;
 	List list = notify ? new ArrayList() : null;
 
-	try {
-		boolean autoCommit = _graph.getConnection().getAutoCommit();
-		if (autoCommit) 
-		    _graph.getConnection().setAutoCommit(false);
-		_graph.delete(it, list);
-		if (autoCommit) {
-		    _graph.getConnection().commit();
-		    _graph.getConnection().setAutoCommit(true);
-		}
-	} catch (Exception e) {
-		throw new JenaException("Couldn't create transaction:"+e);
-	}
+	_graph.delete(it, list);
+
         if (notify) 
         	manager.notifyDeleteIterator( graph, list);
     }
