@@ -1,10 +1,10 @@
 /*
- *  $Id: VirtTransactionHandler.java,v 1.3.2.1 2010/04/01 10:54:36 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2010 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -95,14 +95,7 @@ public class VirtTransactionHandler extends TransactionHandlerBase {
 			try {
 				Connection c = graph.getConnection();
 				c.commit();
-				try {
 					c.setAutoCommit(true);
-				} catch (Exception e) {
-					e.printStackTrace();
-				}
-				// not sure why read_uncommitted is set, below. commented
-				// out by kw.
-				// c.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
 			} catch (SQLException e) {
 				throw new JenaException("Transaction commit failed: ", e);
 			}
