@@ -1,9 +1,9 @@
 /*
- *  $Id: rdftabs.js,v 1.63.2.10 2010/06/28 09:35:57 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2005-2010 OpenLink Software
+ *  Copyright (C) 2005-2013 OpenLink Software
  *
  *  See LICENSE file for details.
  */
@@ -64,7 +64,8 @@ OAT.RDFTabs.browser = function(parent,optObj) {
 	pageSize:20,
 	removeNS:true,
 	description:"This view shows all RDF data grouped by subject resource.",
-	desc:"RDF data by subject resource"
+	desc:"RDF data by subject resource",
+	raw_iris: false
     }
 
     for (var p in optObj) { self.options[p] = optObj[p]; }
@@ -129,8 +130,8 @@ OAT.RDFTabs.browser = function(parent,optObj) {
 	var s = OAT.Dom.create("a");
 	var uri = OAT.IRIDB.getIRI(item.uri);
 
-	s.href = OAT.IRIDB.getIRI(uri);
-	s.title = s.href;
+	s.href = uri;
+	s.title = uri;
 	s.innerHTML = self.parent.getTitle(item);
 
 	OAT.Dom.append([s_ctr,h],[h,s]);
@@ -159,15 +160,7 @@ OAT.RDFTabs.browser = function(parent,optObj) {
 	    var p_ctr = OAT.Dom.create("div",{className:"rdf_p"});
 	    var p_a = OAT.Dom.create("a");
 	    var p_iri = OAT.IRIDB.getIRI(p);
-	    var p_ciri = OAT.IRIDB.getCIRIByID(p);
-
-	    if (!p_ciri) {
-		var spiri = OAT.IRIDB.splitIRI(p_iri);
-		if (!spiri) 
-		    p_ciri = p_iri;
-		else
-		    p_ciri = spiri[0];
-	    }
+	    var p_ciri = OAT.IRIDB.resolveCIRI(p);
 
 	    p_a.href = p_iri;
 
@@ -352,7 +345,10 @@ OAT.RDFTabs.browser = function(parent,optObj) {
 		elm.href = OAT.IRIDB.getIRI(value);
 		attach(elm,value);
 	    }
-	    elm.innerHTML = self.parent.store.getCIRIorSplit(value);
+
+	    if (self.options.raw_iris) elm.innerHTML = elm.href; 
+            else elm.innerHTML = self.parent.store.getCIRIorSplit(value);
+
 	    if (i) { self.sortDiv.appendChild(OAT.Dom.text(", ")); }
 	    self.sortDiv.appendChild(elm);
 	}
@@ -381,9 +377,11 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 
     this.options = {
 	limit:5,
-	description:"This module is used to navigate through locally cached data, one resource at a time. Note that filters are not applied here.",
-	desc:"Navigate through locally cached data (Filters not applied)"
+	description:"This module is used to navigate through locally cached data, one resource at a time. Filters are not applied.",
+	desc:"Navigate through locally cached data (Filters not applied)",
+        raw_iris: false
     }
+
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
     this.initialized = false;
@@ -394,9 +392,10 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
     this.waiting = false;
     this.mlCache = [];
     this.topDiv = OAT.Dom.create("div",{className:"rdf_nav"});
-    this.mainDiv = OAT.Dom.create("div");
+    this.mainDiv = OAT.Dom.create("div",{className:"rdf_nav_content"});
     this.description = self.options.description;
     this.desc = self.options.desc;
+
     OAT.Dom.append([self.elm,self.topDiv,self.mainDiv]);
 
     this.gd = new OAT.GhostDrag();
@@ -472,6 +471,8 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
     this.getTypeObject = function() { /* object of resource types */
 	var obj = [];
 	var noTypeArr = [];
+
+	OAT.IRIDB.insertIRI('http://openlinksw.com/schemas/oat/rdftabs#','oatrdftabs');
 	var ntIID = OAT.IRIDB.insertIRI('http://openlinksw.com/schemas/oat/rdftabs#isReferencedBy');
 
 	var data = self.parent.data.all;
@@ -599,7 +600,10 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	    } else if (type == 1) { /* dereferencable link */
 		content = OAT.Dom.create("a");
 		    content.href = value.getIRI();
-		    content.innerHTML = self.parent.store.getCIRIorSplit(value.getIID());
+
+		    if (self.options.raw_iris) content.innerHTML = value.getIRI();
+                    else content.innerHTML = self.parent.store.getCIRIorSplit(value.getIID());
+
 		    self.dattach(content,value.getIRI());
 		} 
 	    } else { /* text */
@@ -624,10 +628,10 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	    }
 	}
 	else if (typeof(value) == "object") { /* resource */
-	    var items = self.parent.store.items;
+	    var graphs = self.parent.store.graphs;
 	    var dereferenced = false;
-	    for (var j=0;j<items.length;j++) {
-		var item = items[j];
+	    for (var j=0;j<graphs.length;j++) {
+		var item = graphs[j];
 		/* handle anchors to local file */
 		var baseuri = OAT.IRIDB.getIRI(value.iid).match(/^[^#]+/);
 		baseuri = baseuri ? baseuri[0] : "";
@@ -661,8 +665,12 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	    obj.push ([p, item.preds[p]]);
 	}
 
-	obj.push ([refByIID, OAT.IRIDB.getIRI(item.back)]);
-	self.drawSpotlight(self.parent.getTitle(item),obj);
+	for (var i=0;i<item.back.length;i++)
+          obj.push ([refByIID, item.back[i]]); // XXX item.back is a list of backreferences
+
+	self.drawSpotlight(self.parent.getTitle(item),
+			   OAT.IRIDB.getIRI(item.uri),
+			   obj);
     }
 
     this.breadCrumbClickFun = function (index) {
@@ -674,9 +682,9 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
     this.drawBreadCrumbs = function () {
 	var bc = self.nav.breadCrumbs;
 	OAT.Dom.clear(bc);
-	var a = OAT.Dom.create("a", {className: "bc_title"});
+	var a = OAT.Dom.create("span", {className: "bc_title"});
 	a.innerHTML = "Result";
-	a.href = "#";
+//	a.href = "#";
 	a.title = "Initial query result";
 
 	OAT.Event.attach(a,"click",function() {
@@ -696,10 +704,12 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	    if (iid in self.parent.store.labels)
 		a.innerHTML = self.parent.store.labels[iid].label;
 	    else
+		if (self.options.raw_iris) 
+                    a.innerHTML = OAT.IRIDB.getIRI(iid);
+                else 
 		a.innerHTML = self.parent.store.getCIRIorSplit(iid);
 
-	    a.title = OAT.IRIDB.getIRI(iid);
-	    a.href = "#";
+	    a.title = a.href = OAT.IRIDB.getIRI(iid);
 
 	    OAT.Event.attach (a,"click", self.breadCrumbClickFun (i));
 
@@ -767,15 +777,25 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	td.colSpan = 3;
 
 	var simple;
+
+	if (self.options.raw_iris) 
+            simple = OAT.IRIDB.getIRI(label); 
+        else 
 	simple = self.parent.store.getCIRIorSplit(label);
 
 	if (cnt > 1 && simple.charAt(0) != "[" && simple in self.plurals) {
 	    simple = self.plurals[simple];
 	}//]
 
-	td.innerHTML = simple;
-	td.title = OAT.IRIDB.getIRI(label);
+	var a = OAT.Dom.create("a", {className: "nav_rdf_subject"});
+
+	a.innerHTML = simple;
+	a.title = OAT.IRIDB.getIRI(label);
+	a.href = simple;
+
+	td.appendChild (a);
 	tr.appendChild(td);
+
 	OAT.Event.attach(arrow,"click",function() {
 	    state = (state+1) % 2;
 	    arrow.innerHTML = states[state];
@@ -857,13 +877,22 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	}
     }
 
-    this.drawSpotlight = function(title,obj) { /* list of resources */
+    this.drawSpotlight = function(title, uri, obj) { /* list of resources */
 	OAT.Dom.clear(self.mainDiv);
-	var h3 = OAT.Dom.create("h3",{className:"rdf_nav_title"});
-	h3.innerHTML = title;
+	var t_elm = OAT.Dom.create("h3",{className:"rdf_nav_title"});
+
+	if (uri) {
+            var a = OAT.Dom.create ("a");
+            a.href = uri;
+            a.innerHTML = title;
+            OAT.Dom.append ([t_elm, a]);
+	}
+	else 
+            t_elm.innerHTML = title;
+
 	var table = OAT.Dom.create("table",{className:"rdf_nav_spotlight"});
 	var tbody = OAT.Dom.create("tbody");
-	OAT.Dom.append([self.mainDiv,h3,table],[table,tbody]);
+	OAT.Dom.append([self.mainDiv,t_elm,table],[table,tbody]);
 	var remain = false;
 
 	for (i=0;i<obj.length;i++)
@@ -876,9 +905,9 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 	    self.navigate(self.historyIndex);
 	    return;
 	}
-	/* give a list of items for navigation */
+	/* give a list of graphs for navigation */
 	var obj = self.getTypeObject();
-	self.drawSpotlight("Click on a Data Entity to explore its Linked Data Web.",obj);
+	self.drawSpotlight("Click on a Data Entity to explore its Linked Data Web.",false,obj);
 	self.redrawTop();
     }
 
@@ -937,7 +966,8 @@ OAT.RDFTabs.triples = function(parent,optObj) {
 	pageSize:100,
 	removeNS:true,
 	description:"This module displays all filtered triples.",
-	desc:"All filtered triples"
+	desc:"All filtered triples",
+        raw_iris: false
     }
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
@@ -1027,13 +1057,24 @@ OAT.RDFTabs.triples = function(parent,optObj) {
 		iid = atom; // this is a plain IID value
 	}
 
-	var a = OAT.Dom.create("a");
+
+	try { 	// Dirty data does exist, you see...
 	var iri = decodeURIComponent(OAT.IRIDB.getIRI(iid));
-	a.innerHTML = self.parent.store.getCIRIorSplit(iid);
-	a.href = iri;
+            var col_v_elm = OAT.Dom.create("a");
+
+            if (self.options.raw_iris)
+              col_v_elm.innerHTML = iri;
+            else 
+              col_v_elm = self.parent.store.getCIRIorSplit(iid);
+
+	    col_v_elm.href = iri;
+	    self.parent.processLink(col_v_elm, iri);
+        } catch (e) {
+	    col_v_elm = OAT.Dom.create("span",{className:"error_col"});
+            col_v_elm.innerHTML = OAT.IRIDB.getIRI(iid) + " (invalid URI)";
+	}
 	OAT.Dom.clear(v);
-	v.appendChild(a);
-	self.parent.processLink(a,iri);
+	v.appendChild(col_v_elm);
     }
 
     this.redraw = function() {
@@ -1067,7 +1108,8 @@ OAT.RDFTabs.svg = function(parent,optObj) {
     this.options = {
 	limit:100,
 	description:"",
-	desc:"Filtered data as SVG Graph"
+	desc:"Filtered data as SVG Graph",
+        raw_iris: false
     }
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
@@ -1087,23 +1129,44 @@ OAT.RDFTabs.svg = function(parent,optObj) {
 	var cnt = self.parent.data.triples.length;
 
 	if (cnt > self.options.limit) {
-	    var note = new OAT.Notify();
+	    var note = new OAT.Notify(false, {notifyType: 2});
 	    var msg = "Note: Display limited to " + self.options.limit + " triples."
-	    note.send (msg,{delayIn:10,width:350,height:50,timeout:5000});
+	    note.send (msg,{delayIn:10,timeout:5000});
 	    cnt = self.options.limit;
 	}
 
 	for (var i=0;i<cnt;i++) {
 	    var t = self.parent.data.triples[i];
-	    var triple = [OAT.IRIDB.getIRI(t[0]), OAT.IRIDB.getIRI(t[1])]
 
-	    if (t[2].isIRI()) {
-		triple.push (t[2].getIRI());
-		triple.push (1);
-	    } else {
-		triple.push (t[2].getValue());
-		triple.push(0);
+	    //
+	    // XXX: FIXME here and in RDFStore
+	    //
+ 
+	    var new_o;
+
+	    if (t[2].constructor == OAT.RDFAtom) {
+		if (t[2].isIRI())
+		    new_o = t[2].getIRI();
+		else 
+		    new_o = t[2].value;
 	    }
+	    else {
+		if (typeof t[2] == 'object') {
+		    new_o = OAT.IRIDB.getIRI(t[2].iid)
+		}
+		else new_o = t[2];
+	    }
+	    var triple = [OAT.IRIDB.getIRI(t[0]), 
+			  OAT.IRIDB.getIRI(t[1]),
+			  new_o];
+
+//	    if (t[2].isIRI()) {
+//		triple.push (t[2].getIRI());
+//		triple.push (1);
+//	    } else {
+//		triple.push (t[2].getValue());
+//		triple.push(0);
+//	    }
 
 	    triples.push(triple);
 	}
@@ -1113,7 +1176,7 @@ OAT.RDFTabs.svg = function(parent,optObj) {
 
 	for (var i=0;i<self.graphsvg.data.length;i++) {
 	    var node = self.graphsvg.data[i];
-	    if (node.name.match(/^http/i)) {
+	    if (node.name && node.name.match(/^http/i)) {
 		self.parent.processLink(node.svg,node.name);
 	    }
 	}
@@ -1123,7 +1186,7 @@ OAT.RDFTabs.svg = function(parent,optObj) {
 
 //
 // Ordered data structure for points
-// Not a R-Tree but should improve speed for some cases.
+// Not a R-Tree but may improve speed for some cases.
 //
 
 OAT.RDFTabs.PointList = function (opts) {
@@ -1274,6 +1337,10 @@ OAT.RDFTabs.PointList = function (opts) {
     }
 }
 
+//
+// Sends MAP_NOTHING_TO_SHOW if there's nothing to show on map
+//
+
 OAT.RDFTabs.map = function(parent,optObj) {
     var self = this;
     OAT.RDFTabs.parent(self);
@@ -1285,7 +1352,11 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	description:"This module plots all geodata found in filtered resources onto a map.",
 	desc:"Plots all geodata onto a map",
 	clickPopup:true,
-	hoverPopup:true
+	hoverPopup:true,
+	height: "600px",
+	useMobileOpts:false,
+	supportedMobileDetected:false,
+        raw_iris: false
     }
 
     for (var p in optObj) { self.options[p] = optObj[p]; }
@@ -1296,24 +1367,36 @@ OAT.RDFTabs.map = function(parent,optObj) {
     this.description = self.options.description;
     this.desc = self.options.desc;
     this.elm.style.position = "relative";
-    this.elm.style.height = "600px";
 
-    // Various properties used to obtain ICBM address
+    var useragent = navigator.userAgent;
+    
+    if (useragent.indexOf('iPhone') != -1 || useragent.indexOf('Android') != -1 ) {
+	self.elm.style.width = '100%';
+	self.elm.style.height = '100%';
+	self.options.supportedMobileDetected = true;
+    } else {
+    this.elm.style.height = self.options.height;
+    }
+
+    // Various properties used to obtain @ICBM.mil address
     //
 
     this.keyProperties    = OAT.IRIDB.insertIRIArr (["http://xmlns.com/foaf/0.1/based_near",
 						    "http://www.w3.org/2003/01/geo/wgs84_pos",
-						    "http://www.w3.org/2003/01/geo/geometry"]); /* containing coords */
+						     "http://www.w3.org/2003/01/geo/geometry",
+						     "http://www.w3.org/2003/01/geo/wgs84_pos#geometry"]); /* containing coords */
 
     this.locProperties    = OAT.IRIDB.insertIRIArr (["http://xmlns.com/foaf/0.1/location", 
-						     "http://www.w3.org/2006/vcard/ns#Locality",
+						     "http://www.w3.org/2006/vcard/ns#locality",
 						     "http://www.w3.org/2001/vcard-rdf/3.0#Locality"]); /* containing location */
 
     this.latProperties    = OAT.IRIDB.insertIRIArr (["http://www.w3.org/2003/01/geo/lat",
 						     "http://www.w3.org/2003/01/geo/wgs84_pos#lat",
 						     "http://www.w3.org/2003/01/geo/latitude", 
 						     "http://www.w3.org/2006/vcard/ns#latitude",
-						     "http://www.w3.org/2001/vcard-rdf/3.0#latitude"]);
+						     "http://www.w3.org/2001/vcard-rdf/3.0#latitude",
+						     "http://dbpedia.org/property/lat",
+						     "http://www.openlinksw.com/schemas/zillow#latitude"]);
 
     this.lonProperties    = OAT.IRIDB.insertIRIArr (["http://www.w3.org/2003/01/geo/lng",
 						     "http://www.w3.org/2003/01/geo/wgs84_pos#long",
@@ -1321,14 +1404,41 @@ OAT.RDFTabs.map = function(parent,optObj) {
 						     "http://www.w3.org/2003/01/geo/long",
 						     "http://www.w3.org/2003/01/geo/longitude",
 						     "http://www.w3.org/2006/vcard/ns#longitude",
-						     "http://www.w3.org/2001/vcard-rdf/3.0#longitude"]);
+						     "http://www.w3.org/2001/vcard-rdf/3.0#longitude",
+						     "http://dbpedia.org/property/long",
+						     "http://www.openlinksw.com/schemas/zillow#longitude"]);
 
     this.lookupProperties = OAT.IRIDB.insertIRIArr (["http://xmlns.com/foaf/0.1/name",
 						     "http://xmlns.com/foaf/0.1/location"]); /* interesting to be put into lookup pin */
 
     this.pointTypes = OAT.IRIDB.insertIRIArr (["http://www.w3.org/2003/01/geo/Point", 
 					       "http://www.w3.org/2003/01/geo/wgs84_pos#Point",
-					       "http://www.georss.org/georss/point"]);
+					       "http://www.georss.org/georss/point",
+					       "http://www.openlinksw.com/schemas/virtrdf#Geometry"]);
+
+    this.markerPredBlacklist = OAT.IRIDB.insertIRIArr (["http://www.openlinksw.com/schemas/oat/rdftabs#useMarker",
+							"http://xmlns.com/foaf/0.1/based_near",
+							"http://www.w3.org/2003/01/geo/wgs84_pos",
+							"http://www.w3.org/2003/01/geo/geometry",
+							"http://www.w3.org/2003/01/geo/wgs84_pos#geometry",
+							"http://www.w3.org/2003/01/geo/lat",
+							"http://www.w3.org/2003/01/geo/wgs84_pos#lat",
+							"http://www.w3.org/2003/01/geo/latitude", 
+							"http://www.w3.org/2006/vcard/ns#latitude",
+							"http://www.w3.org/2001/vcard-rdf/3.0#latitude",
+							"http://dbpedia.org/property/lat",
+							"http://www.w3.org/2003/01/geo/lng",
+							"http://www.w3.org/2003/01/geo/wgs84_pos#long",
+							"http://www.w3.org/2003/01/geo/lon",
+							"http://www.w3.org/2003/01/geo/long",
+							"http://www.w3.org/2003/01/geo/longitude",
+							"http://www.w3.org/2006/vcard/ns#longitude",
+							"http://www.w3.org/2001/vcard-rdf/3.0#longitude",
+							"http://dbpedia.org/property/long",
+							"http://www.w3.org/2003/01/geo/Point", 
+							"http://www.w3.org/2003/01/geo/wgs84_pos#Point",
+							"http://www.georss.org/georss/point",
+							"http://www.openlinksw.com/schemas/virtrdf#Geometry"]);
 
     this.usedBlanknodes = [];
     this.pointList = new OAT.RDFTabs.PointList({uniqueInsert:true});
@@ -1363,18 +1473,19 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	var coords = [0,0];
 
 	if (item.type) {
-	    for (var i=0;i<item.type.length;i++)
+	    for (var i=0;i<item.type.length;i++) {
 		if (self.pointTypes.indexOf(item.type[i].getValue()) != -1)
 	    coords = self.extractCoords (item.preds);
+	}
 	}
 
 	    for (var p in preds) {
 		var pred = preds[p];
 	    if (!!(p = parseInt(p))) {	    
 		if (self.keyProperties.indexOf(p) != -1) { 
+		    if (pred[0] instanceof OAT.RDFAtom && pred[0].isLit())
 		    pointResource = pred[0].getValue(); 
 		} /* resource containing geo coordinates */
-		
 		if (self.locProperties.indexOf(p) != -1) { 
 		    locValue = pred[0].getValue(); 
 		} /* resource containing geo coordinates */
@@ -1388,7 +1499,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	    }
 
 	if (coords[0] != 0 && coords[1] != 0) {
-	    if (!!window.console) window.console.log ('found coords :' + coords[0] + ' ' + coords[1]);
+	    if (!!window.console && window.oat_debug) window.console.log ('found coords :' + coords[0] + ' ' + coords[1]);
 	    self.attachMarker(coords, item);
 	    return;
 	}
@@ -1398,8 +1509,8 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	}
 	
 	if (!pointResource && locValue) { /* geocode location */
-	    if (!!window.console) window.console.log ('geocoding: '+locValue);
-		self.geoCode(locValue,item);
+	    if (!!window.console && window.oat_debug) window.console.log ('geocoding: '+locValue);
+	    self.geoCode(locValue,item);
 		return;
 	    }
 	
@@ -1409,7 +1520,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 		    coords[0] = cmatches[2];
 		    coords[1] = cmatches[1];
 		    if (coords[0] == 0 || coords[1] == 0) { return; }
-		    self.attachMarker(coords,item);
+		self.attachMarker(coords,item);
 		    return;
 		}
 	    
@@ -1423,8 +1534,6 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	    }
 	    }
 	
-	if (pointResource.isLit()) { return; } /* not a reference */
-
 	    self.usedBlanknodes.push(pointResource);
 
 	    /* normal marker add */
@@ -1432,7 +1541,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 
 	if (coords[0] == 0 || coords[1] == 0) { return; }
 
-	self.attachMarker(coords,item);
+	self.attachMarker(coords, item);
     } /* tryItem */
 
     this.trySimple = function(item) {
@@ -1466,13 +1575,34 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	    return markerPath + '01.png';
 	    break;
 	case OAT.RDFTabsData.MARKER_MODE_EXPLICIT:
-	    mpred = item.preds[m_p_iid];
-	    if (typeof (mpred) != 'undefined')
-		markerFile = markerPath + mpred + '.png';
+	    if (typeof (item.preds[m_p_iid]) != 'undefined') {
+	    mpred = item.preds[m_p_iid][0];
+		if (mpred.constructor == OAT.RDFAtom) {
+		    if (mpred.isIRI())
+			markerFile = mpred.getIRI();
+		    else if (mpred.isLit()) {
+			markerFile = mpred.getValue();
+			if (!markerFile.toUpperCase().match(/HTTP(S?):\/\//)) {
+			    markerFile = markerPath + markerFile+'.png';
+			}
+		    }
+		}
+	    }
 	    break;
 	case OAT.RDFTabsData.MARKER_MODE_AUTO:
-	    mpred = item.preds[m_p_iid];
-	    if (typeof (mpred) != 'undefined') { // explicit marker def takes precedence
+	    if (typeof (item.preds[m_p_iid]) != 'undefined') { // explicit marker def takes precedence
+	    mpred = item.preds[m_p_iid][0];
+		if (mpred.constructor == OAT.RDFAtom) {
+		    if (mpred.isIRI())
+			markerFile = mpred.getIRI();
+		    else if (mpred.isLit()) {
+			markerFile = mpred.getValue();
+			if (!markerFile.toUpperCase().match(/HTTP(S?):\/\//)) {
+			    markerFile = markerPath + markerFile+'.png';
+			}
+		    }
+		}
+		else
 		markerFile = markerPath + mpred + '.png';
 		break;
 	    }
@@ -1499,47 +1629,149 @@ OAT.RDFTabs.map = function(parent,optObj) {
 		return item.preds[p][0].getValue();
 	    }
 	}
+	return false;
 	}
 
+    // an item isCoordinateContainer if it's referred by another item and
+    // its type property values are all geo point types and/or it only has geo coordinate properties
+
+    this.isCoordinateContainer = function (item) {
+	type_iid = OAT.IRIDB.getIRIID("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+	if (!item.back.length) 
+	    return false;
+	var preds = item.preds;
+	var nc_cnt = 0;
+	for (var p in preds) {
+	    var pVal = parseInt(p);
+	    if (pVal == type_iid) { // go through types - pointTypes don't count
+		for (var i=0;i<preds[p].length;i++) {
+		    if (self.pointTypes.find(preds[p][i].getIID()) == -1)
+			return false;
+		}
+	    } else {
+		if (self.lonProperties.find(pVal) != -1 ||
+		    self.latProperties.find(pVal) != -1 || 
+		    self.keyProperties.find(pVal) != -1) {
+		    continue;
+		}
+		else return false; // got a non-geo-property
+	    }
+	}
+	return true;
+    }
+
+    this.drawReferences = function (item, container) {
+        var p_table = OAT.Dom.create ("table");
+	for (i=0;i<item.back.length;i++) { // include backreferences;
+	    var predC = OAT.Dom.create("tr",{className:"predicate"});
+	    var predT = OAT.Dom.create("td",{className:"pred_title"});
+	    predT.innerHTML = "Referenced by";
+	    var predV = OAT.Dom.create("td",{className:"pred_value"});
+	    var content = self.parent.getContent(item.back[i], "replace");
+	    OAT.Dom.append([predV,content],[predC,predT,predV], [p_table,predC]);
+	}
+	OAT.Dom.append ([container,p_table]);
+    }
+
 //
 // Return marker content for item
 //    
 
-    this.drawAllProps = function (item) {
-	var div = OAT.Dom.create ("div",{className:"all_props_ctr"});
+    this.drawAllProps = function (item, container) {
 	var preds = item.preds;
+        var p_table = OAT.Dom.create ("table");
 	for (var p in preds) {
+	    if (self.markerPredBlacklist.find(parseInt(p)) != -1) continue; // Not all predicates are created equal
 	    var pred = preds[p];
-	    var simple = self.parent.store.getCIRIorSplit(p);
+            var simple;
+
+	    if (self.options.raw_iris)
+              simple = OAT.IRIDB.getIRI (p); 
+            else 
+              simple = self.parent.store.getCIRIorSplit(p);
+
 	    if (pred.length == 1 || self.lookupProperties.find(simple) != -1) {
 		var predC = OAT.Dom.create("tr",{className:"predicate"});
 		var predT = OAT.Dom.create("td",{className:"pred_title"});
 		predT.innerHTML = simple;
 		var predV = OAT.Dom.create("td",{className:"pred_value"});
 		var content = self.parent.getContent(pred[0],"replace");
-		OAT.Dom.append([predV,content],[predC,predT,predV],[div,predC]);
+		OAT.Dom.append([predV,content],[predC,predT,predV],[p_table,predC]);
 	    } /* only interesting data */
 	} /* for all predicates */
-	return div;
+        OAT.Dom.append([container,p_table])
+	return container;
     }
 
+    this.showMarkerPopup = function () {
+
+    }
+
+    this.hideMarkerPopup = function () {
+
+    }
+
+    this.makeAnchor = function (ctr, content, href) {
+	var a = OAT.Dom.create ("a");
+	a.href=href;
+        
+    }
     this.drawMarker = function (item) {
-	var titleH = OAT.Dom.create("h2",{className:""});
-	var title = self.parent.getTitle(item);
-	if (title.match(/^http/i)) {
-	    self.parent.processLink(titleH,title);
-	    titleH.style.cursor = "pointer";
+	if (self.isCoordinateContainer(item))
+	    var s_item = item.back[0];
+	else 
+	    s_item = false;
+
+	var titleH = OAT.Dom.create("h2",{className:"markerTitle"});
+
+	var title = self.parent.getTitle(s_item ? s_item : item);
+	var titleHref='';
+	var popup_ctr;
+
+	if (self.options.supportedMobileDetected && self.options.useMobileOpts) {
+	    var titleA = OAT.Dom.create ("a");
+	    OAT.Dom.attach (titleA,"click",self.showMarkerPopup);
 	}
-	titleH.innerHTML = title;
-	var ctr = OAT.Dom.create("div",{overflow:"auto",className:'marker_ctr'});
+	else {
+	if (title.match(/^http/i)) {
+	        var titleA = OAT.Dom.create("a",{href:titleHref,target:"_blank"});
+		self.parent.processLink(titleA, title);
+		titleA.innerHTML = title;
+		OAT.Dom.append ([titleH, titleA]);		
+	} else {
+		var titleHref = self.parent.getURI(s_item ? s_item : item);
+	    if (titleHref) {
+		var titleA = OAT.Dom.create("a",{href:titleHref,target:"_blank"});
+		    self.parent.processLink(titleA, titleHref);
+		titleA.innerHTML = title;
+		OAT.Dom.append ([titleH, titleA]);
+	    } else {
+		titleH.innerHTML = title;
+	    }
+	}
+
+	    var ctr = OAT.Dom.create("div",{className:'marker_ctr'});
+	    var abstr = self.getAbstract(s_item ? s_item : item);
+
+	if (abstr) {
 	var abstrC = OAT.Dom.create("div",{className:'abstract'});
-	abstrC.innerHTML = self.getAbstract(item);
+	    abstrC.innerHTML = abstr;
+	}	
+	}
 	
 	//	if (self.parent.store.itemHasType (item, "")) { }
 	//	if (self.parent.store.itemHasType (item, "")) { }
 
-	var ap = self.drawAllProps (item);
-	OAT.Dom.append([ctr,titleH,abstrC,ap]);
+	var props_ctr = OAT.Dom.create("div", {className: "props_ctr"});
+
+	if (abstr)
+	    OAT.Dom.append([ctr,titleH,abstrC,props_ctr]);
+	else 
+	    OAT.Dom.append([ctr,titleH,props_ctr]);
+
+	self.drawAllProps (s_item ? s_item : item, props_ctr);
+	self.drawReferences (s_item ? s_item : item, props_ctr);
+
 	return ctr;
     }
 
@@ -1608,13 +1840,17 @@ OAT.RDFTabs.map = function(parent,optObj) {
 		var item = self.parent.data.structured[i];
 		self.trySimple(item);
 	    }
+
+	    
 	    self.addMarkers();
+	    
 	}
 
 	if (!self.map_loaded) {
 	    self.map = new OAT.Map(self.elm,
 				   self.options.provider,
-				   {fix:self.options.fix,markerIcon:"toolkit/images/markers/01.png",markerIconSize:[18,41]},
+				   {fix:self.options.fix,markerIcon:"toolkit/images/markers/01.png",
+				    markerIconSize:[18,41]},
 				   self.options.specificOpts);
 	    OAT.Map.loadApi(self.options.provider,{callback: cb});
 	}
@@ -1624,13 +1860,15 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	function tryList() {
 	    if (!self.pointListLock && self.map_loaded) {
 		if (!self.pointList.length()) {
-		    var note = new OAT.Notify();
+		    var note = new OAT.Notify (false, {notifyType: 2});
 		    var msg = "Current data set contains nothing that could be displayed on the map.";
-		    note.send(msg);
+		    note.send(msg, {timeout: 4000});
+		    OAT.MSG.send (self, "MAP_NOTHING_TO_SHOW", false);
 		}
+                clearTimeout (window.tryListTo);
 		self.map.optimalPosition(self.pointList.makePointsArray(false));
  	    } else {
-		setTimeout(tryList,500);
+		window.tryListTo = setTimeout(tryList,500);
 	    }
 	}
 	tryList();
@@ -1644,7 +1882,8 @@ OAT.RDFTabs.timeline = function(parent,optObj) {
     this.options = {
 	imagePath:OAT.Preferences.imagePath,
 	description:"This module displays all date/time containing resources on an interactive timeline.",
-	desc:"Date/time on timeline"
+	desc:"Date/time on timeline",
+        raw_iris: false
     }
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
@@ -1751,11 +1990,15 @@ OAT.RDFTabs.timeline = function(parent,optObj) {
 	if (!uris.length) {
 	    var note = new OAT.Notify();
 	    var msg = "Current data set contains nothing that could be displayed on the timeline.";
-	    note.send(msg);
+	    note.send(msg, {timeout: 4000});
 	}
     }
 }
 
+//
+// XXX unfinished
+//
+
 OAT.RDFTabs.people = function(parent,optObj) {
     var self = this;
     OAT.RDFTabs.parent(self);
@@ -1763,7 +2006,8 @@ OAT.RDFTabs.people = function(parent,optObj) {
     this.options = {
 	pictSize:150,
 	columns: 2,
-	width: 800
+	width: 800,
+        raw_iris: false
     };
 
     this.personTypes = ["http://xmlns.com/foaf/0.1/Person"];
@@ -1789,8 +2033,10 @@ OAT.RDFTabs.images = function(parent,optObj) {
 	thumbSize:150,
 	size:600,
 	description:"This module displays all images found in filtered data set.",
-	desc:"Images from filtered data set"
+	desc:"Images from filtered data set",
+        raw_iris: false
     }
+
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
     this.elm.style.textAlign = "center";
@@ -1882,6 +2128,7 @@ OAT.RDFTabs.images = function(parent,optObj) {
 	    OAT.Dom.center(self.dimmer,1,1);
 	});
 	img.src = self.images[index][0];
+	img.title = self.parent.getTitle(item);
 	self.fixNav();
     }
 
@@ -1907,6 +2154,7 @@ OAT.RDFTabs.images = function(parent,optObj) {
 	});
 	img.src = uri;
 	img.title = self.parent.getTitle(item);
+	img.alt = self.parent.getTitle(item);
 	OAT.Event.attach(img,"click",function() { self.showBig(index); });
     }
 
@@ -1940,7 +2188,8 @@ OAT.RDFTabs.images = function(parent,optObj) {
 		} else {
 		    /* look for predicates that are/contain image links */
 		    for (var j=0;j<pred.length;j++) {
-			if (pred[j].getTag() != OAT.RDFTag.IRI) { continue; }
+			if (pred[j].constructor != OAT.RDFAtom ||
+			    pred[j].getTag() != OAT.RDFTag.IRI) { continue; }
 			var value = pred[j].getIRI();
 			if (self.parent.getContentType(value) == 3) {
 			    self.addUriItem(value,item);
@@ -1953,7 +2202,7 @@ OAT.RDFTabs.images = function(parent,optObj) {
 		    } /* for all values */
 		}
 	    } /* for all predicates */
-	} /* for all items */
+	} /* for all graphs */
     }
 
     this.redraw = function() {
@@ -1984,7 +2233,8 @@ OAT.RDFTabs.tagcloud = function(parent,optObj) {
 
     this.options = {
 	description:"This module displays all links found in filtered data set.",
-	desc:"Links from filtered data set"
+	desc:"Links from filtered data set",
+        raw_iris: false
     }
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
@@ -2050,7 +2300,7 @@ OAT.RDFTabs.tagcloud = function(parent,optObj) {
 	    if (item.type == tcP) {
 		self.addCloud(item);
 	    }
-	} /* for all items */
+	} /* for all graphs */
 
 	for (var i=0; i<this.clouds.length;i++) {
 	    var cloud = this.clouds[i];
@@ -2075,7 +2325,8 @@ OAT.RDFTabs.fresnel = function(parent, optObj) {
 	defaultURL:"",
 	autoload:false,
 	description:"This module applies Fresnel RDF Vocabularies to all dereferenced data.",
-	desc:"Fresnel RDF Vocabularies to dereferenced data"
+	desc:"Fresnel RDF Vocabularies to dereferenced data",
+        raw_iris: false
     }
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
