@@ -1,9 +1,9 @@
 /*
- *  $Id: tree.js,v 1.19.2.3 2010/03/18 10:45:54 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2005-2009 OpenLink Software
+ *  Copyright (C) 2005-2013 OpenLink Software
  *
  *  See LICENSE file for details.
  */
@@ -177,7 +177,7 @@ OAT.TreeNode = function(li,ul,parent,root,value) {
 
 	this.deselect = function() {
 		self.selected = 0;
-		var index = self.root.selectedNodes.find(self);
+		var index = self.root.selectedNodes.indexOf(self);
 		self.root.selectedNodes.splice(index,1);
 		OAT.Dom.removeClass(self.li,"tree_li_selected");
 		if (self.ul) { OAT.Dom.removeClass(self.ul,"tree_ul_selected"); }
@@ -309,7 +309,7 @@ OAT.TreeNode = function(li,ul,parent,root,value) {
 			var mode = node.gdMode;
 
 			function isLast(n) {
-				return (n.parent.children.find(n) == n.parent.children.length-1);
+				return (n.parent.children.indexOf(n) == n.parent.children.length-1);
 			}
 
 			if (mode == 1) {
@@ -317,7 +317,7 @@ OAT.TreeNode = function(li,ul,parent,root,value) {
 				while (node.parent && node.parent.parent && isLast(node)) { node = node.parent; }
 
 				/* reposition after target */
-				var index = node.parent.children.find(node)+1;
+				var index = node.parent.children.indexOf(node)+1;
 				node.parent.appendChild(self,index);
 			} else {
 				/* append */
@@ -490,7 +490,7 @@ OAT.TreeNode = function(li,ul,parent,root,value) {
 		/* insert before node at position [index] */
 		var idx = ( (index || index == 0) ? index : self.children.length);
 		var oldParent = oldNode.parent;
-		if (!ignoreOldParent) { var oldIdx = oldParent.children.find(oldNode); }
+		if (!ignoreOldParent) { var oldIdx = oldParent.children.indexOf(oldNode); }
 		/* basic check */
 		if (!self.ul) {
 			self.ul = OAT.Dom.create("ul",{margin:"0px",padding:"0px"});
@@ -519,7 +519,7 @@ OAT.TreeNode = function(li,ul,parent,root,value) {
 	}
 
 	this.deleteChild = function(oldNode) {
-		var index = self.children.find(oldNode);
+		var index = self.children.indexOf(oldNode);
 		if (index == -1) { return; }
 		self.children.splice(index,1);
 		OAT.Dom.unlink(oldNode.li);
