@@ -1,14 +1,14 @@
 /*
  *  ScrollDemo2.java
  *
- *  $Id: ScrollDemo2.java,v 1.1.1.1 2006/04/11 17:56:11 source Exp $
+ *  $Id$
  *
  *  Sample JDBC program
  *  
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *  
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *  
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -276,9 +276,12 @@ class ScrollDemo2 extends Frame implements Runnable
       conn = null;
     }
     textStatus.setText("Connecting to :"+connectionURL);
+    if (userName != null || password != null)
+      conn = DriverManager.getConnection (connectionURL,userName, password);
+    else
+      conn = DriverManager.getConnection (connectionURL);
 
-    conn = DriverManager.getConnection (connectionURL);
-
+    // Execute Query
     // Execute Query
     stmt = conn.createStatement (ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_UPDATABLE);
     textStatus.setText("Executing query");
@@ -398,7 +401,7 @@ class ScrollDemo2 extends Frame implements Runnable
     panel3.setBackground(new Color(12632256));
     add("North", panel3);
     textQuery = new java.awt.TextField();
-    textQuery.setText("SELECT * FROM \"Customers\"");
+    textQuery.setText("select * from Customers");
     textQuery.reshape(0,0,600,21);
     textQuery.setBackground(new Color(16777215));
     panel3.add("Center", textQuery);
@@ -501,12 +504,11 @@ class ScrollDemo2 extends Frame implements Runnable
     textStatus.reshape(0,62,600,30);
     textStatus.setFont(new Font("Dialog", Font.PLAIN, 10));
     panel4.add(textStatus);
-    setTitle("OpenLink JDBC 2.0 Scrollable Cursor Demo");
+    setTitle("OpenLink Virtuoso JDBC 2.0 Scrollable Cursor Demo");
 
     Info.remove(label1);
     Info.remove(textField1);
-
-    loadDriver("virtuoso.jdbc3.Driver");
+    loadDriver("virtuoso.jdbc2.Driver");
   }
 
   public void show()
@@ -515,24 +517,6 @@ class ScrollDemo2 extends Frame implements Runnable
     super.show();
   }
 
-  public String MakeConnectURL(String inurl, String inkey, String invalue)
-  {
-    int inkeypos, endpos;
-
-	inkeypos = inurl.indexOf(inkey+"=",0);
-	if (inkeypos <0)
-	{
-		return inurl + "/" + inkey + "=" + invalue;
-	}
-	endpos = inurl.indexOf("/",inkeypos);
-	if (endpos <0)
-	{
-		return inurl.substring(0,inkeypos) + inkey + "=" + invalue;
-	}
-
-	return inurl.substring(0,inkeypos) + inkey + "=" + invalue + inurl.substring(endpos);
-  }
-
   public boolean handleEvent(Event event) {
     if (event.id == -1)
     {
@@ -547,25 +531,14 @@ class ScrollDemo2 extends Frame implements Runnable
       loadDriver(driverName);
       return true;
     }
-    else if (event.id == -5)
-    {
-      DbName = (String)(event.arg);
-      connectionURL = MakeConnectURL(connectionURL, "DATABASE", DbName);
-      textStatus.setText("Connection URL is  :"+connectionURL);
-      return true;
-    }
     else if (event.id == -3)
     {
       userName = (String)(event.arg);
-      connectionURL = MakeConnectURL(connectionURL, "UID", userName);
-      textStatus.setText("Connection URL is  :"+connectionURL);
       return true;
     }
     else if (event.id == -4)
     {
       password = (String)(event.arg);
-      connectionURL = MakeConnectURL(connectionURL, "PWD", password);
-      textStatus.setText("Connection URL is  :"+connectionURL);
       return true;
     }
     if (event.id == Event.WINDOW_DESTROY)
@@ -748,7 +721,7 @@ class ScrollDemo2 extends Frame implements Runnable
   static int numThreadsInvoked = 1;
   int myThreadNum;
   String[] urlList;
-  String userName, password, driverName, DbName;
+  String userName, password, driverName;
   boolean isApplet=false;
   String jdkVersion;
 }
