@@ -1,10 +1,10 @@
 /*
- *  $Id: VirtuosoTest.java,v 1.11.2.3 2010/03/01 10:06:02 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2010 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -61,8 +61,8 @@ import org.openrdf.rio.RDFFormat;
 import org.openrdf.rio.RDFHandler;
 import org.openrdf.rio.ntriples.NTriplesWriter;
 
-import virtuoso.jdbc3.VirtuosoExtendedString;
-import virtuoso.jdbc3.VirtuosoRdfBox;
+import virtuoso.jdbc4.VirtuosoExtendedString;
+import virtuoso.jdbc4.VirtuosoRdfBox;
 import virtuoso.sesame2.driver.*;
 
 public class VirtuosoTest {
@@ -120,7 +120,7 @@ public class VirtuosoTest {
 			// test add data to the repository
 			boolean ok = true;
 			String query = null;
-			String strurl = "http://www.openlinksw.com/dataspace/person/kidehen@openlinksw.com/foaf.rdf";
+			String strurl = "http://dbpedia.org/data/Berlin.rdf";
 			URL url = new URL(strurl);
 			URI context = repository.getValueFactory().createURI("http://demo.openlinksw.com/demo#this");
 			Value[][] results = null;
@@ -598,7 +598,7 @@ public class VirtuosoTest {
 		try {
 			String url;
 			url = "jdbc:virtuoso://localhost:1111";
-			Class.forName("virtuoso.jdbc3.Driver");
+			Class.forName("virtuoso.jdbc4.Driver");
 			Connection connection = DriverManager.getConnection(url, "dba", "123456");
 			java.sql.Statement stmt = connection.createStatement();
 
