@@ -1,10 +1,10 @@
 /*
- *  $Id: VirtuosoRepositoryConnection.java,v 1.1.2.9 2010/07/05 12:42:23 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2007 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -171,8 +171,8 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 	private static Resource nilContext;
 	private Connection quadStoreConnection;
 	protected VirtuosoRepository repository;
-	static final String S_INSERT = "sparql define output:format '_JAVA_'  insert into graph iri(??) { `iri(??)` `iri(??)` `bif:__rdf_long_from_batch_params(??,??,??)` }";
-        static final String S_DELETE = "sparql define output:format '_JAVA_' delete from graph iri(??) {`iri(??)` `iri(??)` `bif:__rdf_long_from_batch_params(??,??,??)`}";
+	static final String S_INSERT = "sparql insert into graph iri(??) { `iri(??)` `iri(??)` `bif:__rdf_long_from_batch_params(??,??,??)` }";
+        static final String S_DELETE = "sparql delete from graph iri(??) {`iri(??)` `iri(??)` `bif:__rdf_long_from_batch_params(??,??,??)`}";
 	static final int BATCH_SIZE = 5000;
 	private PreparedStatement psInsert;
 	private int psInsertCount = 0;
@@ -672,7 +672,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
 		String query = "DB.DBA.SPARQL_SELECT_KNOWN_GRAPHS()";
 		try {
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(query);
 
 			// begin at onset one
@@ -1078,7 +1078,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		boolean result = false;
 		String query = "sparql define input:storage \"\" select * where {?s ?o ?p} limit 1";
 		try {
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(query);
 			result = !rs.next();
                         rs.close();
@@ -1335,16 +1335,23 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 			parser.setVerifyData(true);
 			parser.setStopAtFirstError(true);
 			parser.setDatatypeHandling(RDFParser.DatatypeHandling.IGNORE);
-			final PreparedStatement ps = quadStoreConnection.prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
-			final Resource[] _contexts = checkDMLContext(contexts);
 
                         RDFHandlerBase rdfInserter = new RDFHandlerBase() {
 				
 				int count = 0;
+				PreparedStatement ps = null;
+				Resource[] _contexts = checkDMLContext(contexts);
 
-				public void startRDF() {
+				public void startRDF() throws RDFHandlerException {
+					if (ps == null) 
+						try {
+				        		ps = prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
+				        	} catch (java.sql.SQLException e) {
+							throw new RDFHandlerException("Problem PrepareStatement: ", e);
+				        	}
 				}
 
+
 				public void endRDF() throws RDFHandlerException {
 					try {
 						if (count > 0) {
@@ -1352,6 +1359,10 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 							ps.clearBatch();
 							count = 0;
 						}
+						if (ps != null) {
+							ps.close();
+							ps = null;
+						}
 					}
 					catch (SQLException e) {
 						throw new RDFHandlerException("Problem executing query: ", e);
@@ -1361,10 +1372,11 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 				public void handleNamespace(String prefix, String name) throws RDFHandlerException {
 					String query = "DB.DBA.XML_SET_NS_DECL(?, ?, 1)";
 					try {
-						PreparedStatement psn = getQuadStoreConnection().prepareStatement(query);
+						PreparedStatement psn = prepareStatement(query);
 						psn.setString(1, prefix);
 						psn.setString(2, name);
 						psn.execute();
+						psn.close();
 					}
 					catch (SQLException e) {
 						throw new RDFHandlerException("Problem executing query: " + query, e);
@@ -1606,7 +1618,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		}
 
 		try {
-			PreparedStatement ps = quadStoreConnection.prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
+			PreparedStatement ps = prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
 			int count = 0;
 
 			while (it.hasNext()) {
@@ -1637,6 +1649,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 				ps.executeBatch();
 				ps.clearBatch();
 			}
+			ps.close();
 			if (autoCommit) {
 				commit();
 			}
@@ -1697,7 +1710,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		}
   
 		try {
-			PreparedStatement ps = quadStoreConnection.prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
+			PreparedStatement ps = prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
 			int count = 0;
 
 			Statement st;
@@ -1729,6 +1742,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 				ps.executeBatch();
 				ps.clearBatch();
 			}
+			ps.close();
 
 			if (autoCommit) {
 				commit();
@@ -2083,7 +2097,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		List<Namespace> namespaceList = new LinkedList<Namespace>();
 		String query = "DB.DBA.XML_SELECT_ALL_NS_DECLS (3)";
 		try {
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(query);
 
 			// begin at onset one
@@ -2120,7 +2134,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		String retVal = null;
 		String query = "SELECT __xml_get_ns_uri (?, 3)";
 		try {
-			PreparedStatement ps = getQuadStoreConnection().prepareStatement(query);
+			PreparedStatement ps = prepareStatement(query);
 			ps.setString(1, prefix);
 			ResultSet rs = ps.executeQuery();
 
@@ -2158,10 +2172,11 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		}
 		String query = "DB.DBA.XML_SET_NS_DECL(?, ?, 1)";
 		try {
-			PreparedStatement ps = getQuadStoreConnection().prepareStatement(query);
+			PreparedStatement ps = prepareStatement(query);
 			ps.setString(1, prefix);
 			ps.setString(2, name);
 			ps.execute();
+			ps.close();
 		}
 		catch (SQLException e) {
 			throw new StoreException("Problem executing query: " + query, e);
@@ -2189,9 +2204,10 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		flushDelayAdd();
 		String query = "DB.DBA.XML_REMOVE_NS_BY_PREFIX(?, 1)";
 		try {
-			PreparedStatement ps = getQuadStoreConnection().prepareStatement(query);
+			PreparedStatement ps = prepareStatement(query);
 			ps.setString(1, prefix);
 			ps.execute(query);
+			ps.close();
 		}
 		catch (SQLException e) {
 			throw new StoreException("Problem executing query: " + query, e);
@@ -2216,8 +2232,9 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		flushDelayAdd();
 		String query = "DB.DBA.XML_CLEAR_ALL_NS_DECLS()";
 		try {
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+			java.sql.Statement stmt = createStatement();
 			stmt.execute(query);
+			stmt.close();
 		}
 		catch (SQLException e) {
 			throw new StoreException("Problem executing query: " + query, e);
@@ -2231,8 +2248,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		try {
 			verifyIsOpen();
 			flushDelayAdd();
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
-			stmt.setFetchSize(prefetchSize);
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(fixQuery(query, dataset, includeInferred, bindings));
 
 			ResultSetMetaData rsmd = rs.getMetaData();
@@ -2255,8 +2271,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		try {
 			verifyIsOpen();
 			flushDelayAdd();
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
-			stmt.setFetchSize(prefetchSize);
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(fixQuery(query, dataset, includeInferred, bindings));
 			return new GraphResultImpl(new HashMap<String,String>(), new CursorGraphResult(rs));
 		}
@@ -2272,7 +2287,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		try {
 			verifyIsOpen();
 			flushDelayAdd();
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+			java.sql.Statement stmt = createStatement();
 			ResultSet rs = stmt.executeQuery(fixQuery(query, dataset, includeInferred, bindings));
 
 			while(rs.next())
@@ -2302,6 +2317,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 	 */
 	public int executeSPARUL(String query) throws StoreException 
 	{
+		java.sql.Statement stmt = null;
 		verifyIsOpen();
 		verifyNotReadOnly();
 
@@ -2311,13 +2327,19 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
 		try {
 			flushDelayAdd();
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
-			stmt.execute("sparql\n define output:format '_JAVA_'\n " + query);
+			stmt = createStatement();
+			stmt.execute("sparql\n " + query);
 			return stmt.getUpdateCount();
 		}
 		catch (SQLException e) {
 			throw new StoreException(e);
 		}
+		finally {
+			try {
+			  if (stmt != null)
+				stmt.close();
+			} catch (Exception e) {}
+		}
 	}
 
 	/**
@@ -2341,6 +2363,28 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		this.quadStoreConnection = quadStoreConnection;
 	}
 
+
+	private java.sql.Statement createStatement() throws java.sql.SQLException
+	{
+	  java.sql.Statement stmt = quadStoreConnection.createStatement();
+	  int timeout = repository.getQueryTimeout();
+          if (timeout > 0)
+            stmt.setQueryTimeout(timeout);
+          stmt.setFetchSize(prefetchSize);
+          return stmt;
+	}
+
+	private java.sql.PreparedStatement prepareStatement(String sql) throws java.sql.SQLException
+	{
+	  java.sql.PreparedStatement stmt = quadStoreConnection.prepareStatement(sql);
+	  int timeout = repository.getQueryTimeout();
+          if (timeout > 0)
+            stmt.setQueryTimeout(timeout);
+          stmt.setFetchSize(prefetchSize);
+          return stmt;
+	}
+	
+	
 	private String substBindings(String query, BindingSet bindings) 
 	{
 		StringBuffer buf = new StringBuffer();
@@ -2389,29 +2433,11 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
 	private String fixQuery(String query, Dataset dataset, boolean includeInferred, BindingSet bindings) 
 	{
-		StringTokenizer tok = new StringTokenizer(query);
-		String s = "";
-
-/*****
-		while(tok.hasMoreTokens()) {
-		    s = tok.nextToken().toLowerCase();
-		    if (s.equals("describe") || s.equals("construct") || s.equals("ask")) 
-			break;
-		}
-
-		StringBuffer ret = new StringBuffer();
-		if (s.equals("describe") || s.equals("construct") || s.equals("ask")) 
-			ret.append("sparql\n define output:format '_JAVA_'\n ");
-		else 
-			ret.append("sparql\n ");
-*******/
 		StringBuffer ret = new StringBuffer("sparql\n ");
 
 		if (includeInferred && repository.ruleSet!=null && repository.ruleSet.length() > 0)
 		  ret.append("define input:inference '"+repository.ruleSet+"'\n ");
 
-		ret.append("define output:format '_JAVA_'\n ");
-
 		if (dataset != null)
 		{
 		   Set<URI> list = dataset.getDefaultGraphs();
@@ -2453,10 +2479,10 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
 		try {
 			boolean isAutoCommit = getQuadStoreConnection().getAutoCommit();
-		        if (!isAutoCommit && useLazyAdd) {
-		        	synchronized(this) {
+		        synchronized(this) {
+		        	if (!isAutoCommit && useLazyAdd) {
 		        		if (psInsert == null)
-						psInsert = getQuadStoreConnection().prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
+						psInsert = prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
 
 					for (int i = 0; i < contexts.length; i++) {
 						psInsert.setString(1, contexts[i].stringValue());
@@ -2472,20 +2498,21 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 						psInsert.clearBatch();
 						psInsertCount = 0;
 					}
-				}
-		        } else {
-				PreparedStatement ps = getQuadStoreConnection().prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
+		        	} else {
+		        		if (psInsert == null)
+						psInsert = prepareStatement(VirtuosoRepositoryConnection.S_INSERT);
 
-				for (int i = 0; i < contexts.length; i++) {
-					ps.setString(1, contexts[i].stringValue());
-					bindResource(ps, 2, subject);
-					bindURI(ps, 3, predicate);
-					bindValue(ps, 4, object);
+					for (int i = 0; i < contexts.length; i++) {
+						psInsert.setString(1, contexts[i].stringValue());
+						bindResource(psInsert, 2, subject);
+						bindURI(psInsert, 3, predicate);
+						bindValue(psInsert, 4, object);
 
-					ps.addBatch();
-				}
-				ps.executeBatch();
-				ps.clearBatch();
+						psInsert.addBatch();
+					}
+					psInsert.executeBatch();
+					psInsert.clearBatch();
+		        	}
 		        }
 		}
 		catch (SQLException e) {
@@ -2541,11 +2568,12 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
                 if (contexts!=null && contexts.length > 0)
 		  try {
-			PreparedStatement ps = quadStoreConnection.prepareStatement(query);
+			PreparedStatement ps = prepareStatement(query);
 			for (int i = 0; i < contexts.length; i++) {
 				ps.setString(1, contexts[i].stringValue());
 				ps.execute();
 			}
+			ps.close();
 		  }
 		  catch (SQLException e) {
 			throw new StoreException(e);
@@ -2590,7 +2618,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 
 
 		try {
-		        java.sql.Statement st = getQuadStoreConnection().createStatement();
+		        java.sql.Statement st = createStatement();
 		        ResultSet rs = st.executeQuery(query.toString());
 
 			if (rs.next())
@@ -2642,8 +2670,7 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 			query.append(" LIMIT 1");
 
 		try {
-			java.sql.Statement stmt = getQuadStoreConnection().createStatement();
-			stmt.setFetchSize(prefetchSize);
+			java.sql.Statement stmt = createStatement();
 			rs = stmt.executeQuery(query.toString());
 		}
 		catch (SQLException e) {
@@ -2657,8 +2684,6 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 	private void removeContext(Resource subject, URI predicate, Value object, Resource context) 
 		throws StoreException 
 	{
-		PreparedStatement ps = null;
-
 		String S = "?s";
 		String P = "?p";
 		String O = "?o";
@@ -2668,19 +2693,21 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
 		    if (subject == null && predicate == null && object == null && context != null) {
 			String  query = "sparql clear graph iri(??)";
 
-			ps = getQuadStoreConnection().prepareStatement(query);
+			PreparedStatement ps = prepareStatement(query);
 			ps.setString(1, context.stringValue());
 			ps.execute();
+			ps.close();
 
 		    } else if (subject != null && predicate != null && object != null && context != null) {
 
-		    	ps = getQuadStoreConnection().prepareStatement(VirtuosoRepositoryConnection.S_DELETE);
+		    	PreparedStatement ps = prepareStatement(VirtuosoRepositoryConnection.S_DELETE);
 
 			ps.setString(1, context.stringValue());
 			bindResource(ps, 2, subject);
 			bindURI(ps, 3, predicate);
 			bindValue(ps, 4, object);
 			ps.execute();
+			ps.close();
 
 		    } else {
 
@@ -2702,8 +2729,9 @@ public class VirtuosoRepositoryConnection implements RepositoryConnection {
   				"> { "+S+" "+P+" "+O+" } from <"+context+
   				"> where { "+S+" "+P+" "+O+" }";
 
-		    	java.sql.Statement stmt = getQuadStoreConnection().createStatement();
+		    	java.sql.Statement stmt = createStatement();
 		    	stmt.execute(query);
+		    	stmt.close();
 		    }
 		}
 		catch (SQLException e) {
