@@ -1,14 +1,14 @@
 --
 --  sqlo2.sql
 --
---  $Id: sqlo2.sql,v 1.4.2.5 2010/12/10 12:24:41 source Exp $
+--  $Id$
 --
 --  Various SQL optimized compiler tests, part 2.
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -1341,8 +1341,8 @@ create view t1order as select row_no, string1, string2 from t1 order by row_no;
 
 select top 2 * from t1order where row_no is null or row_no > 111;
 
-echo both $if $equ $last[1] 113 "PASSED" "***FAILED";
-echo both "or of known false in dt predf import\n";
+ECHO BOTH $IF $EQU $LAST[1] 113 "PASSED" "***FAILED";
+ECHO BOTH ": or of known false in dt predf import\n";
 
 select top 2 * from t1order a where row_no is null or exists (select 1 from t1order b where a.row_no = 1 + b.row_no);
 
@@ -1353,12 +1353,12 @@ select count (*) from t1 where row_no = 111 or (row_no  is null and (row_no is n
 -- hash fillers with hash joined existences
 explain ('select count (*) from t1 a, t1 b where a.row_no = b.row_no and exists (select * from t1 c table option (hash) where c.row_no = b.row_no and c.string1 like ''1%'') option (order, hash)');
 select count (*) from t1 a, t1 b where a.row_no = b.row_no and exists (select * from t1 c table option (hash) where c.row_no = b.row_no and c.string1 like '1%') option (order, hash);
-echo both $if $equ $last[1] 353 "PASSED" "***FAILED";
-echo both ": hash join with filter with hash filler with hashed exists\n";
+ECHO BOTH $IF $EQU $LAST[1] 353 "PASSED" "***FAILED";
+ECHO BOTH ": hash join with filter with hash filler with hashed exists\n";
 
 select count (*) from t1 a, t1 b where a.row_no = b.row_no and exists (select * from t1 c table option (loop) where c.row_no = b.row_no and c.string1 like '1%') option (order, loop);
-echo both $if $equ $last[1] 353 "PASSED" "***FAILED";
-echo both ": verify above with ibid with loop\n";
+ECHO BOTH $IF $EQU $LAST[1] 353 "PASSED" "***FAILED";
+ECHO BOTH ": verify above with ibid with loop\n";
 
 
 
