@@ -1,10 +1,10 @@
 --  
---  $Id: tdav_meta_rdf.sql,v 1.2 2006/08/16 07:58:11 source Exp $
+--  $Id$
 --  
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -41,7 +41,7 @@ create procedure TDAV_RDF_QUAD_CHECK (in resname varchar, in propuri varchar, in
   full_res_uri := DB.DBA.DAV_FULL_PATH_TO_IRI (new_dav_graph, '/DAV/tdav_meta_home/zip_samples/' || resname);
   dbg_obj_princ ('TDAV_RDF_QUAD_CHECK: ', full_res_uri, propuri, encoded_propval);
   if (crop <> 0)
-    return
+    return;
   if (exists (sparql ask where { graph ?:new_dav_graph {
           ?:full_res_uri ?:propuri ?:propval } } ))
     actual := 1;
