@@ -1,11 +1,11 @@
 #!/bin/sh
 #
-#  $Id: thttp.sh,v 1.18.2.5 2010/01/28 14:33:20 source Exp $
+#  $Id$
 #
 #  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 #  project.
 #
-#  Copyright (C) 1998-2009 OpenLink Software
+#  Copyright (C) 1998-2013 OpenLink Software
 #
 #  This project is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -744,30 +744,48 @@ case $1 in
    if [ "$MAKE_VAD" = "yes" ] ; then
        if [ $do_mappers_only -eq 1 ]
        then
-	   (cd ../../../rdf_mappers; make)
-	   cp ../../../rdf_mappers/rdf_mappers_dav.vad ./
+	   LOG "Create RDF mappers Package"
+	   (cd ../../../../binsrc/rdf_mappers && $MAKE rdf_mappers_dav.vad)
+	   cp ../../../../binsrc/rdf_mappers/rdf_mappers_dav.vad .
+       elif [ -f ../../../../autogen.sh ]
+       then
+	   LOG "Create RDF mappers Package"
+	   (cd ../../../../binsrc/rdf_mappers && $MAKE rdf_mappers_dav.vad)
+	   cp ../../../../binsrc/rdf_mappers/rdf_mappers_dav.vad .
+
+	   LOG "Create ODS Framework VAD Package"
+	   (cd ../../../../appsrc/ODS-Framework && $MAKE ods_framework_dav.vad)
+	   cp ../../../../appsrc/ODS-Framework/ods_framework_dav.vad .
+
+	   LOG "Create ODS Blog VAD Package"
+	   (cd ../../../../appsrc/ODS-Blog && $MAKE ods_blog_dav.vad)
+	   cp ../../../../appsrc/ODS-Blog/ods_blog_dav.vad .
+
+	   LOG "Create SyncML VAD Package"
+	   (cd ../../../../binsrc/sync && $MAKE syncml_dav.vad)
+	   cp ../../../../binsrc/sync/syncml_dav.vad .
        elif [ "x$HOST_OS" = "x" ]
        then
 	   LOG "Create ODS VAD Package"
-	   (cd ../../../samples/wa/; make)
+	   (cd ../../../samples/wa/; $MAKE)
 	   cp ../../../samples/wa/ods_framework_dav.vad ./
 	   LOG "Create BLOG VAD Package"
-	   (cd ../../../weblog2/; make)
+	   (cd ../../../weblog2/; $MAKE)
 	   cp ../../../weblog2/ods_blog_dav.vad ./
 	   LOG "Create SyncML VAD Package"
-	   (cd ../../../sync/; make)
+	   (cd ../../../sync/; $MAKE)
 	   cp ../../../sync/syncml_dav.vad ./
-	   (cd ../../../rdf_mappers; make)
+	   (cd ../../../rdf_mappers; $MAKE)
 	   cp ../../../rdf_mappers/rdf_mappers_dav.vad ./
        elif [ "x$SRC" != "x" ]
        then
 	   LOG "Create ODS VAD Package"
-	   (cd "$SRC/binsrc/samples/wa/" ; ./make_vad.sh)
+	   (cd "$SRC/binsrc/samples/wa/" ; $MAKE)
 	   cp "$SRC/binsrc/samples/wa/ods_framework_dav.vad" .
 	   LOG "Create BLOG VAD Package"
-	   (cd "$SRC/binsrc/weblog2/" ; ./make_vad.sh)
+	   (cd "$SRC/binsrc/weblog2/" ; $MAKE)
 	   cp "$SRC/binsrc/weblog2/ods_blog_dav.vad" .
-	   (cd "$SRC/binsrc/rdf_mappers"; make)
+	   (cd "$SRC/binsrc/rdf_mappers"; $MAKE)
 	   cp "$SRC/binsrc/rdf_mappers/rdf_mappers_dav.vad" .
        elif [ ! -f ../../../../autogen.sh ]
        then
@@ -1040,6 +1058,68 @@ fi
       exit 1
    fi
 
+  LOG 'Now starting graph CRUD tests...'
+rm http/_virtrdf_log*.ttl
+rm http/graphcrud*.log
+
+#
+#  Make sure curl is installed
+#
+RUN curl --version
+
+if test $STATUS -eq 0
+then
+    curl --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud?graph-uri=http://www.openlinksw.com/schemas/virtrdf%23" > http/_virtrdf_log1.ttl 2> http/graphcrud_get0.log
+    if grep 'virtrdf:' http/_virtrdf_log1.ttl > /dev/null ; then
+      LOG 'PASSED: http/_virtrdf_log1.ttl contains data'
+    else
+      LOG '***FAILED: http/_virtrdf_log1.ttl does not contains virtrdf: string, but it should'
+    fi
+
+    curl --digest --user dba:dba --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud-auth?graph-uri=http://example.com/crud1" -X DELETE > http/graphcrud_1.log 2>&1
+    if grep 'HTTP/1.1 404' http/graphcrud_1.log > /dev/null ; then
+      LOG 'PASSED: http/graphcrud_1.log contains 404 error for missing graph'
+    else
+      LOG '***FAILED: http/graphcrud_1.log does not contains 404 error for missing graph, but it should'
+    fi
+
+    curl --digest --user dba:dba --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud-auth?graph-uri=http://example.com/crud1" -T http/_virtrdf_log1.ttl > http/graphcrud_2.log 2>&1
+    if grep  'HTTP/1.1 201' http/graphcrud_2.log > /dev/null ; then
+      LOG 'PASSED: http/graphcrud_2.log contains 201 for newly created graph'
+    else
+      LOG '***FAILED: http/graphcrud_2.log does not contain 201 for newly created graph, but it should'
+    fi
+
+    curl --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud?graph-uri=http://example.com/crud1" > http/_virtrdf_log2.ttl 2> http/graphcrud_get2.log
+    if grep 'virtrdf:' http/_virtrdf_log2.ttl > /dev/null ; then
+      LOG 'PASSED: http/_virtrdf_log2.ttl contains data'
+    else
+      LOG '***FAILED: http/_virtrdf_log2.ttl does not contains virtrdf: string, but it should'
+    fi
+
+    curl --digest --user dba:dba --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud-auth?graph-uri=http://example.com/crud1" -T http/_virtrdf_log1.ttl > http/graphcrud_2.log 2>&1
+    if grep  'HTTP/1.1 200' http/graphcrud_2.log > /dev/null ; then
+      LOG 'PASSED: http/graphcrud_2.log contains 200 for recreated graph'
+    else
+      LOG '***FAILED: http/graphcrud_2.log does not contains 200 for newly created graph, but it should'
+    fi
+
+    curl --digest --user dba:dba --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud-auth?graph-uri=http://example.com/crud1" -X DELETE > http/graphcrud_3.log 2>&1
+    if grep 'HTTP/1.1 200' http/graphcrud_3.log > /dev/null ; then
+      LOG 'PASSED: http/graphcrud_3.log contains 200 for successful graph removal'
+    else
+      LOG '***FAILED: http/graphcrud_3.log does not contain 200 for successful graph removal, but it should'
+    fi
+
+    curl --verbose --url "http://localhost:$HTTPPORT/sparql-graph-crud?graph-uri=http://example.com/crud1" > http/graphcrud_4.log 2>&1
+    if grep 'HTTP/1.1 404' http/graphcrud_4.log > /dev/null ; then
+      LOG 'PASSED: http/graphcrud_4.log contains 404 for deleted graph'
+    else
+      LOG '***FAILED: http/graphcrud_4.log does not contain 404 for deleted graph, but it should'
+    fi
+fi
+
+
    SHUTDOWN_SERVER
 
    # 
