@@ -1,10 +1,10 @@
 --  
---  $Id: tsoap_new.sql,v 1.2 2006/08/16 07:58:12 source Exp $
+--  $Id$
 --  
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -363,6 +363,8 @@ create procedure DB.DBA.SOAPTEST ()
 ECHO BOTH $IF $EQU $STATE OK "PASSED" "*** FAILED";
 ECHO BOTH ": creating VSP SOAP server procedure STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
 
+grant execute on DB.DBA.SOAPTEST to SOAP_SRV;
+
 soap_call_new ('localhost:$U{HTTPPORT}', '/vspsoap.vsp', 'fake', 'test', vector(), 11);
 ECHO BOTH $IF $EQU $STATE OK "PASSED" "*** FAILED";
 ECHO BOTH ": calling the VSP SOAP server with fake#test STATE=" $STATE " MESSAGE=" $MESSAGE "\n";
@@ -380,6 +382,7 @@ create module DB.DBA.SOAPMOD {
   procedure SOAPMOD2(inout par2 int) { return par2 + 1; };
   procedure SOAPMOD3() { return 14; };
 };
+GRANT EXECUTE ON DB.DBA.SOAPMOD TO SOAP_SRV;
 
 select xml_tree_doc (soap_sdl ('DB.DBA.SOAPMOD', 'URL'));
 ECHO BOTH $IF $EQU $STATE OK "PASSED" "*** FAILED";
