@@ -1,10 +1,10 @@
 --  
---  $Id: vad_make.sql,v 1.23.2.2 2010/04/06 16:47:38 source Exp $
+--  $Id$
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -726,6 +726,7 @@ create procedure "VAD"."DBA"."VAD_TEST_CREATE" (
   {
     s2 := cast (xpath_eval ('@type', aref (items, j)) as varchar);
     s6 := cast (xpath_eval ('@source', aref (items, j)) as varchar);
+    s5 := cast (xpath_eval ('@source_uri', aref (items, j)) as varchar);
     s3 := cast (xpath_eval ('@target_uri', aref (items, j)) as varchar);
     s4 := get_keyword (s6, iniarr);
     if (s4 is null or length(s4)=0)
@@ -733,11 +734,14 @@ create procedure "VAD"."DBA"."VAD_TEST_CREATE" (
       _err_message := sprintf ('Illegal resource type:%s for %s', s2,s3);
       goto error_fin;
     }
-    s4 := sprintf ('%s/%s', s4, s3);
+    if (s5 is null or length(s5)=0)
+    {
+      s5 := sprintf ('%s/%s', s4, s3);
+    }
     if (neq (s6, 'dav'))
-      "VAD"."DBA"."VAD_OUT_ROW_FILE" (fname, pos, s3, s4, ctx);
+      "VAD"."DBA"."VAD_OUT_ROW_FILE" (fname, pos, s3, s5, ctx);
     else
-      "VAD"."DBA"."VAD_OUT_ROW_DAV" (fname, pos, s3, s4, ctx, iniarr);
+      "VAD"."DBA"."VAD_OUT_ROW_DAV" (fname, pos, s3, s5, ctx, iniarr);
     j := j + 1;
   }
   data := md5_final (ctx);
@@ -881,7 +885,7 @@ create procedure "VAD"."DBA"."VAD_CHECK_STICKER_DETAILS" (
         tid := "VAD"."DBA"."VAD_MKNODE" (parr, tid, s2, 'STRING', s3);
       }
       if (not "VAD"."DBA"."VAD_TEST_PACKAGE_LT" (parr, s2, s3))
-        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('required package %s with version less than %s hasn\'t yet installed', s2, s3));
+        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('This package requires %s version less than %s', s2, s3));
     }
     s3 := cast (xpath_eval ('version/@package', aref (items, j)) as varchar);
     if (s2 is not null and length(s3))
@@ -892,7 +896,7 @@ create procedure "VAD"."DBA"."VAD_CHECK_STICKER_DETAILS" (
         tid := "VAD"."DBA"."VAD_MKNODE" (parr, tid, s2, 'STRING', s3);
       }
       if (not "VAD"."DBA"."VAD_TEST_PACKAGE_EQ" (parr, s2, s3))
-        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('required package %s with version equal to %s hasn\'t yet installed', s2, s3));
+        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('This package requires %s version %s', s2, s3));
     }
     s3 := cast (xpath_eval ('versions_later/@package', aref (items, j)) as varchar);
     if (s2 is not null and length(s3))
@@ -903,7 +907,7 @@ create procedure "VAD"."DBA"."VAD_CHECK_STICKER_DETAILS" (
         tid := "VAD"."DBA"."VAD_MKNODE" (parr, tid, s2, 'STRING', s3);
       }
       if (not "VAD"."DBA"."VAD_TEST_PACKAGE_GT" (parr, s2, s3))
-        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('required package %s with version greater than %s hasn\'t yet installed', s2, s3));
+        "VAD"."DBA"."VAD_FAIL_CHECK" (sprintf ('This package requires %s version greater than %s', s2, s3));
     }
     j := j + 1;
   }
@@ -1136,20 +1140,33 @@ create procedure "DB"."DBA"."VAD_INSTALL" (
   declare qual varchar;
   declare SQL_STATE, SQL_MESSAGE varchar;
 
+  qual := dbname ();
+  parr := null;
+  SQL_STATE := '00000';
+  SQL_MESSAGE := '';
+  result_names (SQL_STATE, SQL_MESSAGE);
+
+  if ((is_dav = 0 and 0 = file_stat (fname)) 
+      or 
+      (is_dav <> 0 and (not exists (select 1 from WS.WS.SYS_DAV_RES where RES_FULL_PATH = fname))))
+    {
+      registry_set ('VAD_wet_run', '0');
+      registry_set ('VAD_errcount', '1');
+      result ('42VAD', concat ('Could not open ',
+      case when is_dav = 0 then 'filesystem' else 'DAV' end, 
+      ' resource ', fname, ' Reason: File not found'));
+      goto failure;
+    }
+
   commit work;
   if (sys_stat ('cl_run_local_only') = 1)
     exec ('checkpoint');
   else
     {
-      cl_exec ('rdf_check_init()');
+      exec ('rdf_check_init()');
     cl_exec ('checkpoint');
     }
 
-  qual := dbname ();
-  parr := null;
-  SQL_STATE := '00000';
-  SQL_MESSAGE := '';
-
   "VAD"."DBA"."VAD_ATOMIC" (1);
 
   registry_set ('VAD_msg', '');
@@ -1158,7 +1175,6 @@ create procedure "DB"."DBA"."VAD_INSTALL" (
   registry_set ('VAD_is_run', '1');
   connection_set ('vad_pkg_fullname', null);
 
-  result_names (SQL_STATE, SQL_MESSAGE);
   {
     declare exit handler for sqlstate '*'
     {
@@ -2255,31 +2271,26 @@ create procedure DB.DBA.VAD_DEPS_CHECK (in parr any, in name varchar, in version
 }
 ;
 
--- checks if x is less than y
+--!
+-- \brief Compare two version strings.
+--
+-- An arbitrary number of version components are supported in addition to an
+-- optional suffix like "_xxxNN" where "xxx" is a word and "NN" is the suffix
+-- number.
+--
+-- \b Examples:
+-- \code
+-- 1.0
+-- 1.0.0
+-- 1.0.2
+-- 2.3_git12
+-- \endcode
+--
+-- \return \p 1 if \p x is smaller than \p y, \p 0 otherwise.
+--/
 create procedure VAD.DBA.VER_LT (in x varchar, in y varchar)
 {
-  declare xx, yy any;
-  if (x is null)
-    return 0;
-  if (length (x) = 0 and length (y) > 0)
-    return 1;
-  xx := split_and_decode (x, 0, '\0\0.');
-  yy := split_and_decode (y, 0, '\0\0.');
-  if (length (xx) > length (yy))
-    return 0;
-  if (length (xx) < length (yy))
-    return 1;
-  for (declare i, l int, i := 0, l := length (xx); i < l; i := i + 1)
-    {
-      declare xi, yi int;
-      xi := atoi (xx[i]);
-      yi := atoi (yy[i]);
-      if (xi < yi)
-	return 1;
-      if (xi > yi)
-	return 0;
-    }
-  return 0;
+  return (case when "VAD"."DBA"."VERSION_COMPARE" (x, y) = -1 then 1 else 0 end);
 }
 ;
 
@@ -2333,7 +2344,7 @@ create procedure "VAD"."DBA"."VAD_AUTO_UPGRADE" ()
 	   if (ver is null and pname = 'conductor')
 	     {
 		  isdav := 1;
-		ver := '';
+		ver := '0.0.0';
 	     }
 
 	   -- Only upgrade if package exists in database with older version
