@@ -2,14 +2,14 @@
 --
 --  vsp_auth.vsp
 --
---  $Id: vsp_auth.sql,v 1.2 2008/03/24 17:25:12 source Exp $
+--  $Id$
 --
 --  Virtuoso vsp stored procedures for digest and basic authentication
 --  
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
 --  
---  Copyright (C) 1998-2006 OpenLink Software
+--  Copyright (C) 1998-2013 OpenLink Software
 --  
 --  This project is free software; you can redistribute it and/or modify it
 --  under the terms of the GNU General Public License as published by the
@@ -331,18 +331,6 @@ vsp_ua_match_hdr (inout lines any, in match_str varchar)
 
 
 
-create procedure
-vsp_ua_prop_init ()
-{
---  dbg_printf ('vsp_ua_prop_init');
-  return (vector ('has_basic_auth', 'has_digest_auth',
-		  'is_msie', 'has_frames',
-		  'has_tables', 'has_css1',
-		  'has_css2', 'has_xml',
-		  'browser_level', 'ua_platform',
-		  'is_robot', 'is_dav_client'));
-}
-;
 
 --!AWK PUBLIC
 create procedure
@@ -378,6 +366,19 @@ vsp_calculate_digest (in username varchar, in pass varchar, in auth_vec any)
 }
 ;
 
+create procedure
+vsp_ua_prop_init ()
+{
+--  dbg_printf ('vsp_ua_prop_init');
+  return (vector ('has_basic_auth', 'has_digest_auth',
+		  'is_msie', 'has_frames',
+		  'has_tables', 'has_css1',
+		  'has_css2', 'has_xml',
+		  'browser_level', 'ua_platform',
+		  'is_robot', 'is_dav_client'));
+}
+;
+
 create procedure
 vsp_ua_vec_init ()
 {
@@ -418,14 +419,14 @@ vsp_ua_vec_init ()
 			  2, 'Win32',
 			  'False', 'False'),
 		  'Mozilla/4.*Windows*',
-		  vector ('True', 'False',
+		  vector ('True', 'True',
 			  'False', 'True',
 			  'True', 'True',
 			  'False', 'False',
 			  4, 'Win32',
 			  'False', 'False'),
 		  'Mozilla/5.*Windows*',
-		  vector ('True', 'False',
+		  vector ('True', 'True',
 			  'False', 'True',
 			  'True', 'True',
 			  'False', 'True',
@@ -438,8 +439,15 @@ vsp_ua_vec_init ()
                           'False', 'False',
                           4, 'Linux',
                           'False', 'False'),
+		  'Mozilla/5.*AppleWebKit/5*',
+		  vector ('True', 'True',
+			  'False', 'True',
+			  'True', 'True',
+			  'True', 'True',
+			  5, 'NA',
+			  'False', 'False'),
                   'Mozilla/5.*Linux*',
-                  vector ('True', 'False',
+                  vector ('True', 'True',
                           'False', 'True',
                           'True', 'True',
                           'False', 'True',
@@ -453,7 +461,7 @@ vsp_ua_vec_init ()
                           4, 'Mac',
                           'False', 'False'),
                   'Mozilla/5.*Macintosh*',
-                  vector ('True', 'False',
+                  vector ('True', 'True',
                           'False', 'True',
                           'True', 'True',
                           'False', 'True',
@@ -494,6 +502,13 @@ vsp_ua_vec_init ()
 			  'False', 'False',
 			  0, 'NA',
 			  'False', 'False'),
+    		  'WebDAVFS/*',
+		  vector ('True', 'True',
+			  'False', 'True',
+			  'True',  'True',
+			  'False', 'False',
+			  0, 'Mac',
+			  'False', 'True'),
 		  '*',
 		  vector ('False', 'False',
 			  'False', 'False',
@@ -723,7 +738,7 @@ vsp_auth_get (in realm varchar, in domain varchar,
 
   http_request_status ('HTTP/1.1 401 Unauthorized');
   require_encrypted := sys_stat ('sql_encryption_on_password');
-  if (require_encrypted = 1 or require_encrypted = 2)
+  if (allow_basic and (require_encrypted = 1 or require_encrypted = 2))
     require_encrypted := 0;
   else
     require_encrypted := 1;
