@@ -1,14 +1,14 @@
 /*
  *  Dkmarshal.c
  *
- *  $Id: Dkmarshal.c,v 1.16.2.4 2009/12/09 10:58:42 source Exp $
+ *  $Id$
  *
  *  Marshalling on top of sessions
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -345,7 +345,8 @@ box_read_flags (dk_session_t * session, dtp_t dtp)
 {
   uint32 flags = (uint32) read_long (session);
   char *string = scan_session_boxing (session);
-  box_flags (string) = flags;
+  if (IS_BOX_POINTER (string))
+    box_flags (string) = flags;
   return (void *) string;
 }
 
@@ -799,6 +800,15 @@ scan_session_boxing (dk_session_t * session)
       *box = (boxint) (ptrlong) result;
       result = box;
     }
+  else if (next_char == DV_CHARACTER)
+    {
+      char * box;
+      if (!IS_POINTER (result))
+	return (void *) (ptrlong) result;
+      MARSH_CHECK_BOX (box = (char *) dk_try_alloc_box (sizeof (char), DV_CHARACTER));
+      *box = (char) (ptrlong) result;
+      result = box;
+    }
 
   return result;
 }
@@ -891,7 +901,7 @@ double_to_buf (double d, char *buf)
   unsigned char buf2[8];
   XDR x;
 
-  xdrmem_create (&x, (char *) buf, sizeof (buf2), XDR_ENCODE);
+  xdrmem_create (&x, (char *) buf2, sizeof (buf2), XDR_ENCODE);
   xdr_double (&x, &d);
   memcpy (buf, buf2, sizeof (double));
 #endif
