@@ -1,14 +1,14 @@
 /*
  *  Dkparam.h
  *
- *  $Id: Dkparam.h,v 1.4 2009/04/07 22:00:54 source Exp $
+ *  $Id$
  *
  *  Global parameters
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -25,6 +25,9 @@
  *
  */
 
+#ifndef _DKPARAM_H
+#define _DKPARAM_H
+
 #define MAX_NESTED_FUTURES		20
 #define MAX_INTERRUPTS			20
 #define MAX_THREADS			4096			   /* 512 */
@@ -38,3 +41,5 @@
 #define DKSES_OUT_BUFFER_LENGTH		(4096 * 8)
 
 #define ATOMIC_TIMEOUT			2
+
+#endif
