@@ -1,10 +1,10 @@
 /*
- *  $Id: CallableStatementWrapper.java,v 1.2.2.2 2009/08/20 20:13:28 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2009 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -1302,6 +1302,32 @@ public class CallableStatementWrapper
     }
   }
 
+#if JDK_VER >= 17
+    //--------------------------JDBC 4.1 -----------------------------
+  public <T> T getObject(int parameterIndex, Class<T> type) throws SQLException
+  {
+    check_close();
+    try {
+      return ((CallableStatement)stmt).getObject(parameterIndex, type);
+    } catch (SQLException ex) {
+      exceptionOccurred(ex);
+      throw ex;
+    }
+  }
+
+  public <T> T getObject(String parameterName, Class<T> type) throws SQLException
+  {
+    check_close();
+    try {
+      return ((CallableStatement)stmt).getObject(parameterName, type);
+    } catch (SQLException ex) {
+      exceptionOccurred(ex);
+      throw ex;
+    }
+  }
+
+#endif
+
 #endif
 #endif
 
