@@ -1,5 +1,5 @@
 /*
- *  $Id: tidy.c,v 1.1.1.1.2.2 2010/01/25 23:14:14 source Exp $
+ *  $Id$
  *
  *  tidy.c - HTML parser and pretty printer
  *
@@ -215,7 +215,7 @@ StreamIn *OpenInputBoxString(caddr_t html_input)
 {
     StreamIn *in;
     in = (StreamIn *)MemAlloc(sizeof(StreamIn));
-    ClearMemory(in, sizeof (StreamIn *));
+    ClearMemory(in, sizeof (StreamIn));
     in->input.tio_data.lm_memblock = html_input;
     in->input.tio_data.lm_length = box_length (html_input)-1;
     in->input.tio_pos = 0;
