@@ -1,12 +1,12 @@
 /*
  *  2pc.c
  *
- *  $Id: 2pc.c,v 1.15.2.15 2010/05/21 12:40:47 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -556,11 +556,6 @@ lt_2pc_prepare (lock_trx_t * lt)
   _2pc_printf (("lt_2pc_prepare\n"));
   lt->lt_status = LT_PREPARE_PENDING;
 
-  if (LTE_OK != lt_log_replication (lt))
-    {
-      rc = LTE_LOG_FAILED;
-      goto failed;
-    }
 
   LEAVE_TXN;
   mutex_enter (log_write_mtx);
@@ -612,7 +607,6 @@ lt_2pc_commit (lock_trx_t * lt)
   lt->lt_status = LT_COMMITTED;
 
   log_final_transact (lt, 1);
-  lt_send_repl_cast (lt);
   if (lt->lt_mode == TM_SNAPSHOT)
     {
       lt_close_snapshot (lt);
@@ -1390,7 +1384,7 @@ virt_xa_set_client (void *xid, struct client_connection_s *cli)
 	  return rc;
 #else
 	  mutex_leave (global_xa_map->xm_mtx);
-	  return VXA_AGAIN;
+	  return VXA_ERROR;
 #endif
 	}
     }
