@@ -1,12 +1,12 @@
 /*
  *  CLIodbc3.c
  *
- *  $Id: CLIodbc3.c,v 1.26.2.2 2010/07/06 09:58:07 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -717,12 +717,10 @@ virtodbc__SQLGetStmtAttr (SQLHSTMT statementHandle,
     SQLINTEGER BufferLength,
     SQLINTEGER * StringLengthPtr)
 {
-  SQLINTEGER dummy;
+  SQLLEN dummy = 0;
   STMT (stmt, statementHandle);
-
   if (!stmt)
     return (SQL_INVALID_HANDLE);
-
   if (!ValuePtr)
     ValuePtr = &dummy;
 
@@ -1336,6 +1334,7 @@ get_rdf_literal_prop (cli_connection_t * con, SQLSMALLINT ftype, short key)
     }
   else
     ret = gethash ((void *)(ptrlong) key, ht);
+  LEAVE_CON (con);
 
   if (!ret) /* not in cache */
     {
@@ -1351,7 +1350,6 @@ get_rdf_literal_prop (cli_connection_t * con, SQLSMALLINT ftype, short key)
       rc = virtodbc__SQLAllocHandle (SQL_HANDLE_STMT, con, &hstmt);
       if (rc != SQL_SUCCESS)
 	{
-	  LEAVE_CON (con);
 	  return NULL;
 	}
       rc = virtodbc__SQLBindParameter (hstmt, 1, SQL_PARAM_INPUT, SQL_C_SSHORT,
@@ -1370,12 +1368,13 @@ get_rdf_literal_prop (cli_connection_t * con, SQLSMALLINT ftype, short key)
       if (SQL_SUCCESS != rc)
 	goto err_cleanup;
       ret = box_dv_short_string (buf);
+      IN_CON (con);
       sethash ((void*)(ptrlong)key, ht, (void*) ret);
+      LEAVE_CON (con);
 err_cleanup:
       virtodbc__SQLFreeStmt (hstmt, SQL_CLOSE);
       virtodbc__SQLFreeHandle (SQL_HANDLE_STMT, (SQLHANDLE) hstmt);
     }
-  LEAVE_CON (con);
   return ret;
 }
 
