@@ -1,14 +1,14 @@
 /*
  *  CLIuti.c
  *
- *  $Id: CLIuti.c,v 1.35.2.5 2010/12/20 18:09:05 source Exp $
+ *  $Id$
  *
  *  Auxiliary functions for the ODBC driver
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -205,7 +205,7 @@ dv_to_sql_type (dtp_t dv, int cli_binary_timestamp)
       return SQL_WVARCHAR;
 
     case DV_ANY:
-      return SQL_VARCHAR;
+      return SQL_LONGVARCHAR;
 
     case DV_INT64:
       return SQL_INTEGER;
@@ -2214,9 +2214,9 @@ dv_to_str_place (caddr_t it, dtp_t dtp, SQLLEN max, caddr_t place,
         {
           iri_id_t iid = unbox_iri_id (it);
           if (iid >= MIN_64BIT_BNODE_IRI_ID)
-	    snprintf (temp, sizeof (temp), "#ib" BOXINT_FMT, (boxint)(iid-MIN_64BIT_BNODE_IRI_ID));
+	    snprintf (temp, sizeof (temp), "#ib" IIDBOXINT_FMT, (boxint)(iid-MIN_64BIT_BNODE_IRI_ID));
           else
-	    snprintf (temp, sizeof (temp), "#i" BOXINT_FMT, (boxint)(iid));
+	    snprintf (temp, sizeof (temp), "#i" IIDBOXINT_FMT, (boxint)(iid));
           break;
         }
 
