@@ -1,7 +1,7 @@
 /*
  *  auxfiles.c
  *
- *  $Id: auxfiles.c,v 1.20.2.8 2010/09/17 16:35:46 source Exp $
+ *  $Id$
  *
  *  License file (license.dat)
  *  Error logging (wi.err)
@@ -10,7 +10,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -39,6 +39,7 @@ char * http_port;
 char * https_port;
 char * https_cert;
 char * https_key;
+char * https_extra;
 int32 https_client_verify = 0;
 int32 https_client_verify_depth = 0;
 char * https_client_verify_file = NULL;
@@ -85,7 +86,7 @@ int n_oldest_flushable;
 int null_bad_dtp;
 int atomic_dive = 0;
 int dive_pa_mode = PA_READ;
-int c_compress_mode = 0;
+int32 c_compress_mode = 0;
 int default_txn_isolation = ISO_REPEATABLE;
 int prefix_in_result_col_names;
 int disk_no_mt_write;
@@ -127,6 +128,7 @@ int c_stripe_unit = 256;
 extern int32 sqlo_compiler_exceeds_run_factor;
 
 int32 c_dense_page_allocation = 0;
+int32 log_proc_overwrite = 1;
 
 void _db_read_cfg (dbe_storage_t * dbs, char *mode);
 dk_set_t _cfg_read_storages (caddr_t **temp_storage);
