@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2012 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -692,11 +692,58 @@ pkcs7_signer_info_to_array (PKCS7 * p7)
   return (caddr_t) ret;
 }
 
+BIO *
+strses_to_bio (dk_session_t * ses)
+{
+  BIO * in_bio;
+  int len = strses_length (ses), to_read = len, readed = 0;
+  char buf[4096];
+  char err_buf[512];
+
+  in_bio = BIO_new (BIO_s_mem ());
+  CATCH_READ_FAIL (ses)
+    {
+      do {
+	readed = session_buffered_read (ses, buf, MIN (sizeof (buf), to_read));
+	if (readed && readed != BIO_write (in_bio, buf, readed))
+	  sqlr_new_error ("42000", "CR003", "Can not write to BIO. SSL Error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+	to_read -= readed;
+      } while (to_read > 0);
+    }
+  END_READ_FAIL (ses);
+  return in_bio;
+}
+
+dk_session_t * 
+bio_to_strses (BIO * out_bio)
+{
+  dk_session_t * ses = strses_allocate ();
+  char buf[4096], *to_free;
+  char *ptr = NULL;
+  int len = BIO_get_mem_data (out_bio, &ptr);
+  int to_read = len, readed = 0;
+
+  to_free = ((BUF_MEM *) out_bio->ptr)->data;
+  BIO_set_flags (out_bio, BIO_FLAGS_MEM_RDONLY);
+  CATCH_WRITE_FAIL (ses)
+    {
+      do {
+	readed = BIO_read (out_bio, buf, MIN (sizeof (buf), to_read));
+	if (readed > 0)
+	  session_buffered_write (ses, buf, readed);
+	to_read -= readed;
+      } while (to_read > 0);
+    }
+  END_WRITE_FAIL (ses);
+  ((BUF_MEM *) out_bio->ptr)->data = to_free;
+  BIO_clear_flags (out_bio, BIO_FLAGS_MEM_RDONLY);
+  return ses;
+}
 
 static caddr_t
 bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  caddr_t msg = bif_string_arg (qst, args, 0, "smime_verify");
+  caddr_t msg = bif_arg (qst, args, 0, "smime_verify");
   caddr_t certs = bif_array_arg (qst, args, 1, "smime_verify");
   int flags = 0;
   caddr_t ret = NULL;
@@ -705,22 +752,37 @@ bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   BIO *out_bio = NULL, *in_bio = NULL, *data_bio = NULL;
   PKCS7 *p7 = NULL;
   X509_STORE *store = NULL;
+  char * to_free = NULL;
   int res;
   char err_buf[512];
 
   if (BOX_ELEMENTS (args) > 3)
     flags = (int) bif_long_arg (qst, args, 3, "smime_verify");
 
+  if (!IS_BOX_POINTER (msg) || (DV_TYPE_OF (msg) != DV_STRING && DV_TYPE_OF (msg) != DV_STRING_SESSION))
+     msg = bif_string_arg (qst, args, 0, "smime_verify");
+
   store = smime_get_store_from_array (certs, &err);
   if (err)
     sqlr_resignal (err);
   if (!store)
     sqlr_new_error ("42000", "CR003", "No CA certificates. SSL Error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
-
-  in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
+  if (DV_TYPE_OF (msg) == DV_STRING_SESSION)
+    {
+      in_bio = strses_to_bio ((dk_session_t *) msg);
+      to_free = ((BUF_MEM *) in_bio->ptr)->data;
+      BIO_set_flags (in_bio, BIO_FLAGS_MEM_RDONLY);
+    }
+  else
+    in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
   if (in_bio)
     {
       p7 = SMIME_read_PKCS7 (in_bio, &data_bio);
+      if (to_free)
+	{
+	  ((BUF_MEM *) in_bio->ptr)->data = to_free;
+	  BIO_clear_flags (in_bio, BIO_FLAGS_MEM_RDONLY);
+	}
       BIO_free (in_bio);
     }
 
@@ -753,9 +815,17 @@ bif_smime_verify (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   if (res)
     {
       char *ptr = NULL;
-      ret = dk_alloc_box (BIO_get_mem_data (out_bio, &ptr) + 1, DV_SHORT_STRING);
-      memcpy (ret, ptr, box_length (ret) - 1);
-      ret[box_length (ret) - 1] = 0;
+      int len = BIO_get_mem_data (out_bio, &ptr);
+      if (len >= MAX_BOX_LENGTH)
+	{
+	  ret = (caddr_t) bio_to_strses (out_bio);
+	}
+      else
+	{
+	  ret = dk_alloc_box (len + 1, DV_SHORT_STRING);
+	  memcpy (ret, ptr, box_length (ret) - 1);
+	  ret[box_length (ret) - 1] = 0;
+	}
     }
 
   BIO_free (out_bio);
@@ -1552,6 +1622,28 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	ret = list_to_array (dk_set_nreverse (set));
 	break;
       }
+    case 12:
+      {
+	const unsigned char *s;
+	int i, n;
+	const ASN1_STRING *sig = cert->signature;
+	X509_ALGOR *sigalg = cert->sig_alg;
+	char buf[80];
+	caddr_t val;
+
+        i2t_ASN1_OBJECT(buf,sizeof (buf), sigalg->algorithm);
+
+	n = sig->length;
+	s = sig->data;
+	val = dk_alloc_box ((n * 2) + 1, DV_SHORT_STRING);
+	for (i = 0; i < n; i ++)
+	  {
+	    sprintf (&(val[i * 2]), "%02x", s[i]);
+	  }
+	val[n * 2] = 0;
+	ret = list (2, box_dv_short_string (buf), val);
+	break;
+      }
     default:
       {
 	if (!internal)
