@@ -1,14 +1,14 @@
 /*
  *  bif_imap.c
  *
- *  $Id: bif_imap.c,v 1.1.2.8 2011/01/28 13:32:49 source Exp $
+ *  $Id$
  *
  *  IMAP4 client function
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2010 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -70,6 +70,7 @@
 	  } \
 	END_WRITE_FAIL (ses)
 
+
 static int
 is_ok (char *resp)
 {
@@ -231,6 +232,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   char resp[1024];
   char message[128], err_text[512], err_code[6], login_message[512], username[512], password[512];
   char end_msg[1] = ")";
+  char end_msg3[3] = ")\r\n";
   char *s, *ps;
   caddr_t target_folder_id = NULL;
   dk_session_t *msg = NULL;
@@ -313,60 +315,60 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   if (!stricmp ("list", mode))
     {
       if (folder_id && strlen (folder_id) > 0)
-			snprintf (message, sizeof (message), "3 LIST \"\" \"%s\"", folder_id);
+	snprintf (message, sizeof (message), "3 LIST \"\" \"%s\"", folder_id);
       else
-			snprintf (message, sizeof (message), "3 LIST \"\" \"%%\"");
+	snprintf (message, sizeof (message), "3 LIST \"\" \"%%\"");
       SEND (ses, rc, message, "");
       message_begin = 0;
       strses_flush (msg);
       strses_enable_paging (msg, http_ses_size);
       CATCH_READ_FAIL (ses)
-      {
-	rc = dks_read_line (ses, resp, sizeof (resp));
-	while (strlen (resp) > 2 && strncmp ("3 OK", resp, 4))
-	  {
-	    ps = resp;
-	    ps = imap_next_word (ps);
-	    if (!ascii_strncasecmp ("LIST", ps, 4))
+	{
+	  rc = dks_read_line (ses, resp, sizeof (resp));
+	  while (strlen (resp) > 2 && strncmp ("3 OK", resp, 4))
+	    {
+	      ps = resp;
 	      ps = imap_next_word (ps);
-	    else
-	      {
-		strcpy_ck (err_text, "Some error in the list of folders");
-		strcpy_ck (err_code, "IM006");
-		SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-		goto logout;
-	      }
-	    dk_set_push (ret_v, box_dv_short_string (ps));
-	    if (tcpses_check_disk_error (msg, qst, 0))
-	      {
-		strcpy_ck (err_text, "Server error in accessing temp file");
-		strcpy_ck (err_code, "IM007");
-		SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-		goto logout;
-	      }
-	    rc = dks_read_line (ses, resp, sizeof (resp));
-	  }
-	if (tcpses_check_disk_error (msg, NULL, 0))
-	  {
-	    strcpy_ck (err_text, "Server error in accessing temp file");
-	    strcpy_ck (err_code, "IM008");
-	    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-	    goto logout;
-	  }
-	if (!STRSES_CAN_BE_STRING (msg))
-	  {
-	    strcpy_ck (err_text, "Server error in storing data into a string session");
-	    strcpy_ck (err_code, "IM009");
-	    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-	    goto logout;
-	  }
-      }
+	      if (!ascii_strncasecmp ("LIST", ps, 4))
+		ps = imap_next_word (ps);
+	      else
+		{
+		  strcpy_ck (err_text, "Some error in the list of folders");
+		  strcpy_ck (err_code, "IM006");
+		  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+		  goto logout;
+		}
+	      dk_set_push (ret_v, box_dv_short_string (ps));
+	      if (tcpses_check_disk_error (msg, qst, 0))
+		{
+		  strcpy_ck (err_text, "Server error in accessing temp file");
+		  strcpy_ck (err_code, "IM007");
+		  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+		  goto logout;
+		}
+	      rc = dks_read_line (ses, resp, sizeof (resp));
+	    }
+	  if (tcpses_check_disk_error (msg, NULL, 0))
+	    {
+	      strcpy_ck (err_text, "Server error in accessing temp file");
+	      strcpy_ck (err_code, "IM008");
+	      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+	      goto logout;
+	    }
+	  if (!STRSES_CAN_BE_STRING (msg))
+	    {
+	      strcpy_ck (err_text, "Server error in storing data into a string session");
+	      strcpy_ck (err_code, "IM009");
+	      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+	      goto logout;
+	    }
+	}
       FAILED
-      {
-	strcpy_ck (err_code, "IM010");
-	strcpy_ck (err_text, "Failed reading output of FETCH command on remote IMAP server");
-	goto error_end;
-      }
+	{
+	  strcpy_ck (err_code, "IM010");
+	  strcpy_ck (err_text, "Failed reading output of LIST command on remote IMAP server");
+	  goto error_end;
+	}
       END_READ_FAIL (ses);
       goto logout;
     }
@@ -375,7 +377,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   if (!stricmp ("delete", mode))
     {
       if (folder_id && strlen (folder_id) > 0)
-		snprintf (message, sizeof (message), "3 DELETE \"%s\"", folder_id);
+	snprintf (message, sizeof (message), "3 DELETE \"%s\"", folder_id);
       else
 	{
 	  strcpy_ck (err_code, "IM011");
@@ -407,7 +409,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   if (!stricmp ("create", mode))
     {
       if (folder_id && strlen (folder_id) > 0)
-		snprintf (message, sizeof (message), "3 CREATE \"%s\"", folder_id);
+	snprintf (message, sizeof (message), "3 CREATE \"%s\"", folder_id);
       else
 	{
 	  strcpy_ck (err_code, "IM015");
@@ -440,9 +442,9 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   if (!stricmp ("select", mode) || !stricmp ("expunge", mode))
     {
       if (folder_id && strlen (folder_id) > 0)
-		snprintf (message, sizeof (message), "4 SELECT \"%s\"", folder_id);
+	snprintf (message, sizeof (message), "4 SELECT \"%s\"", folder_id);
       else
-		snprintf (message, sizeof (message), "4 SELECT \"INBOX\"");
+	snprintf (message, sizeof (message), "4 SELECT \"INBOX\"");
       SEND (ses, rc, message, "");
       while (1)
 	{
@@ -492,82 +494,82 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
 	      strses_flush (msg2);
 	      strses_enable_paging (msg2, http_ses_size);
 	      CATCH_READ_FAIL (ses)
-	      {
-		rc = dks_read_line (ses, resp, sizeof (resp));
-		if (strlen (resp) > 2 && !strncmp ("5 OK", resp, 4))
-		  break;
-		while (strncmp (end_msg, resp, sizeof (end_msg)))
-		  {
-		    ps = resp;
-		    if (!message_begin)
-		      {
-			message_begin = 1;
-			ps = imap_next_word (ps);
-			ps = imap_next_word (ps);
-			if (!ascii_strncasecmp ("FETCH", ps, 5))
-			  {
-			    ps = imap_next_word (ps);
-			    if (ps[0] == '(')
-			      ps++;
-			    if (ascii_strncasecmp ("UID", ps, 3) == 0)
-			      {
-				ps = imap_next_word (ps);
-				uid = atoi (ps);
-				ps = imap_next_word (ps);
-				SES_PRINT (msg2, ps);
-				if (tcpses_check_disk_error (msg2, qst, 0))
-				  {
-				    strcpy_ck (err_text, "Server error in accessing temp file");
-				    strcpy_ck (err_code, "IM024");
-				    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-				    goto logout;
-				  }
-				goto next_statement;
-			      }
-			  }
-			else
-			  {
-			    strcpy_ck (err_text, "Some error");
-			    strcpy_ck (err_code, "IM023");
-			    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-			    goto logout;
-			  }
-		      }
-		    SES_PRINT (msg, ps);
-		    if (tcpses_check_disk_error (msg, qst, 0))
-		      {
-			strcpy_ck (err_text, "Server error in accessing temp file");
-			strcpy_ck (err_code, "IM024");
-			SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-			goto logout;
-		      }
-		  next_statement:
-		    rc = dks_read_line (ses, resp, sizeof (resp));
-		  }
-		session_flush_1 (msg);
-		session_flush_1 (msg2);
-		if (tcpses_check_disk_error (msg, NULL, 0) || tcpses_check_disk_error (msg2, NULL, 0))
-		  {
-		    strcpy_ck (err_text, "Server error in accessing temp file");
-		    strcpy_ck (err_code, "IM025");
-		    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-		    goto logout;
-		  }
-		if (!STRSES_CAN_BE_STRING (msg) || !STRSES_CAN_BE_STRING (msg2))
-		  {
-		    strcpy_ck (err_text, "Server error in storing data into a string session");
-		    strcpy_ck (err_code, "IM026");
-		    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-		    goto logout;
-		  }
-		dk_set_push (ret_v, list (3, uid, strses_string (msg2), strses_string (msg)));
-	      }
+		{
+		  rc = dks_read_line (ses, resp, sizeof (resp));
+		  if (strlen (resp) > 2 && !strncmp ("5 OK", resp, 4))
+		    break;
+		  while (strncmp (end_msg, resp, sizeof (end_msg)))
+		    {
+		      ps = resp;
+		      if (!message_begin)
+			{
+			  message_begin = 1;
+			  ps = imap_next_word (ps);
+			  ps = imap_next_word (ps);
+			  if (!ascii_strncasecmp ("FETCH", ps, 5))
+			    {
+			      ps = imap_next_word (ps);
+			      if (ps[0] == '(')
+				ps++;
+			      if (ascii_strncasecmp ("UID", ps, 3) == 0)
+				{
+				  ps = imap_next_word (ps);
+				  uid = atoi (ps);
+				  ps = imap_next_word (ps);
+				  SES_PRINT (msg2, ps);
+				  if (tcpses_check_disk_error (msg2, qst, 0))
+				    {
+				      strcpy_ck (err_text, "Server error in accessing temp file");
+				      strcpy_ck (err_code, "IM024");
+				      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+				      goto logout;
+				    }
+				  goto next_statement;
+				}
+			    }
+			  else
+			    {
+			      strcpy_ck (err_text, "Some error");
+			      strcpy_ck (err_code, "IM023");
+			      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+			      goto logout;
+			    }
+			}
+		      SES_PRINT (msg, ps);
+		      if (tcpses_check_disk_error (msg, qst, 0))
+			{
+			  strcpy_ck (err_text, "Server error in accessing temp file");
+			  strcpy_ck (err_code, "IM024");
+			  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+			  goto logout;
+			}
+		    next_statement:
+		      rc = dks_read_line (ses, resp, sizeof (resp));
+		    }
+		  session_flush_1 (msg);
+		  session_flush_1 (msg2);
+		  if (tcpses_check_disk_error (msg, NULL, 0) || tcpses_check_disk_error (msg2, NULL, 0))
+		    {
+		      strcpy_ck (err_text, "Server error in accessing temp file");
+		      strcpy_ck (err_code, "IM025");
+		      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+		      goto logout;
+		    }
+		  if (!STRSES_CAN_BE_STRING (msg) || !STRSES_CAN_BE_STRING (msg2))
+		    {
+		      strcpy_ck (err_text, "Server error in storing data into a string session");
+		      strcpy_ck (err_code, "IM026");
+		      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+		      goto logout;
+		    }
+		  dk_set_push (ret_v, list (3, box_num (uid), strses_string (msg2), strses_string (msg)));
+		}
 	      FAILED
-	      {
-		strcpy_ck (err_code, "IM027");
-		strcpy_ck (err_text, "Failed reading output of FETCH command on remote IMAP server");
-		goto error_end;
-	      }
+		{
+		  strcpy_ck (err_code, "IM027");
+		  strcpy_ck (err_text, "Failed reading output of FETCH command on remote IMAP server");
+		  goto error_end;
+		}
 	      END_READ_FAIL (ses);
 	    }
 	}
@@ -594,7 +596,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
 	  strcpy_ck (err_text, "There must be 2 string items in vector of argument 7 (old folder name to rename and a new name)");
 	  goto logout;
 	}
-		snprintf (message, sizeof (message), "4 RENAME \"%s\" \"%s\"", in[0], in[1]);
+      snprintf (message, sizeof (message), "4 RENAME \"%s\" \"%s\"", in[0], in[1]);
       SEND (ses, rc, message, "");
       while (1)
 	{
@@ -616,7 +618,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
   if (!stricmp ("fetch", mode) || !stricmp ("message_delete", mode) || !stricmp ("message_copy", mode))
     {
       if (folder_id && strlen (folder_id) > 0)
-		snprintf (message, sizeof (message), "4 SELECT \"%s\"", folder_id);
+	snprintf (message, sizeof (message), "4 SELECT \"%s\"", folder_id);
       else
 	snprintf (message, sizeof (message), "4 SELECT INBOX");
       SEND (ses, rc, message, "");
@@ -634,7 +636,7 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
       if (inx_mails > 0)
 	{
 	  volatile int l, br;
-	  int start = 0;
+	  int do_not_read, start = 0;
 	  dtp_t type;
 	  if (in)
 	    l = BOX_ELEMENTS (in);
@@ -660,98 +662,116 @@ imap_get (char *host, caddr_t * err_ret, caddr_t user, caddr_t pass,
 		  goto logout;
 		}
 	      if (!stricmp ("fetch", mode))
-					snprintf (message, sizeof (message), "5 UID FETCH %d BODY.PEEK[]", (int)(in[br]));
+		snprintf (message, sizeof (message), "5 UID FETCH %d BODY.PEEK[]", (int) (in[br]));
 	      if (!stricmp ("message_delete", mode))
-					snprintf (message, sizeof (message), "5 UID STORE %d +FLAGS (\\Deleted)", (int)(in[br]));
+		snprintf (message, sizeof (message), "5 UID STORE %d +FLAGS (\\Deleted)", (int) (in[br]));
 	      if (!stricmp ("message_copy", mode))
-					snprintf (message, sizeof (message), "5 UID COPY %d \"%s\"", (int)(in[br]), target_folder_id);
+		snprintf (message, sizeof (message), "5 UID COPY %d \"%s\"", (int) (in[br]), target_folder_id);
 	      SEND (ses, rc, message, "");
+	      do_not_read = 0;
 	      while (1)
 		{
 		  message_begin = 0;
 		  strses_flush (msg);
 		  strses_enable_paging (msg, http_ses_size);
 		  CATCH_READ_FAIL (ses)
-		  {
-		    rc = dks_read_line (ses, resp, sizeof (resp));
-		    if (strlen (resp) > 2 && !strncmp ("5 OK", resp, 4))
-		      break;
-		    if (strlen (resp) > 2 && !strncmp ("5 BAD", resp, 5))
-		      {
-			strcpy_ck (err_text, "Error in IMAP command UID STORE");
-			strcpy_ck (err_code, "IM035");
-			break;
-		      }
-		    if (strlen (resp) > 2 && !strncmp ("5 NO", resp, 4))
-		      {
-			strcpy_ck (err_text, "Error in IMAP command UID STORE");
-			strcpy_ck (err_code, "IM036");
-			break;
-		      }
-		    while (strncmp (end_msg, resp, sizeof (end_msg)))
-		      {
-			ps = resp;
-			if (!message_begin)
-			  {
-			    message_begin = 1;
-			    ps = imap_next_word (ps);
-			    ps = imap_next_word (ps);
-			    if (!ascii_strncasecmp ("FETCH", ps, 5))
-			      {
-				ps = imap_next_word (ps);
-				if (ps[0] == '(')
-				  ps++;
-				if (ascii_strncasecmp ("UID", ps, 3) == 0)
-				  {
-				    ps = imap_next_word (ps);
-				    uid = atoi (ps);
-				  }
-			      }
-			    else
-			      {
-				strcpy_ck (err_text, "Some error");
-				strcpy_ck (err_code, "IM037");
-				SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-				goto logout;
-			      }
-			    if (!stricmp ("message_delete", mode) || !stricmp ("message_copy", mode))
-			      break;
-			    goto next_message;
-			  }
-			SES_PRINT (msg, ps);
-			if (tcpses_check_disk_error (msg, qst, 0))
-			  {
-			    strcpy_ck (err_text, "Server error in accessing temp file");
-			    strcpy_ck (err_code, "IM038");
-			    SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-			    goto logout;
-			  }
-		      next_message:
+		    {
+		      if (!do_not_read)
 			rc = dks_read_line (ses, resp, sizeof (resp));
-		      }
-		    session_flush_1 (msg);
-		    if (tcpses_check_disk_error (msg, NULL, 0))
-		      {
-			strcpy_ck (err_text, "Server error in accessing temp file");
-			strcpy_ck (err_code, "IM039");
-			SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-			goto logout;
-		      }
-		    if (!STRSES_CAN_BE_STRING (msg))
-		      {
-			strcpy_ck (err_text, "Server error in storing data into a string session");
-			strcpy_ck (err_code, "IM040");
-			SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
-			goto logout;
-		      }
-		    dk_set_push (ret_v, list (2, uid, strses_string (msg)));
-		  }
+		      if (strlen (resp) > 2 && !strncmp ("5 OK", resp, 4))
+			break;
+		      if (strlen (resp) > 2 && !strncmp ("5 BAD", resp, 5))
+			{
+			  strcpy_ck (err_text, "Error in IMAP command UID STORE");
+			  strcpy_ck (err_code, "IM035");
+			  break;
+			}
+		      if (strlen (resp) > 2 && !strncmp ("5 NO", resp, 4))
+			{
+			  strcpy_ck (err_text, "Error in IMAP command UID STORE");
+			  strcpy_ck (err_code, "IM036");
+			  break;
+			}
+		      while (resp)
+			{
+			  ps = resp;
+			  if (!message_begin)
+			    {
+			      message_begin = 1;
+			      if (!ascii_strncasecmp ("* ", ps, 2))
+				{
+				  ps = imap_next_word (ps);
+				  ps = imap_next_word (ps);
+				  if (!ascii_strncasecmp ("FETCH", ps, 5))
+				    {
+				      ps = imap_next_word (ps);
+				      if (ps[0] == '(')
+					ps++;
+				      if (ascii_strncasecmp ("UID", ps, 3) == 0)
+					{
+					  ps = imap_next_word (ps);
+					  uid = atoi (ps);
+					}
+				    }
+				}
+			      else
+				{
+				  strcpy_ck (err_text, "Some error");
+				  strcpy_ck (err_code, "IM037");
+				  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+				  goto logout;
+				}
+			      if (!stricmp ("message_delete", mode) || !stricmp ("message_copy", mode))
+				break;
+			      goto next_message;
+			    }
+			  SES_PRINT (msg, ps);
+			  if (tcpses_check_disk_error (msg, qst, 0))
+			    {
+			      strcpy_ck (err_text, "Server error in accessing temp file");
+			      strcpy_ck (err_code, "IM038");
+			      SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+			      goto logout;
+			    }
+			next_message:
+			  rc = dks_read_line (ses, resp, sizeof (resp));
+			  if (strlen (resp) > 2 && (!strncmp ("5 OK", resp, 4) || !strncmp ("5 BAD", resp, 5)
+				  || !strncmp ("5 NO", resp, 4)))
+			    {
+			      do_not_read = 1;
+			      break;
+			    }
+			}
+		      session_flush_1 (msg);
+		      if (tcpses_check_disk_error (msg, NULL, 0))
+			{
+			  strcpy_ck (err_text, "Server error in accessing temp file");
+			  strcpy_ck (err_code, "IM039");
+			  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+			  goto logout;
+			}
+		      if (!STRSES_CAN_BE_STRING (msg))
+			{
+			  strcpy_ck (err_text, "Server error in storing data into a string session");
+			  strcpy_ck (err_code, "IM040");
+			  SESSION_SCH_DATA (ses)->sio_read_fail_on = 0;
+			  goto logout;
+			}
+		      if (uid > 0)
+			{
+			  caddr_t result = NULL;
+			  result = strses_string (msg);
+			  if (!strncmp (result + strlen (result) - 3, end_msg3, sizeof (end_msg3)))
+			    result[strlen (result) - 3] = 0;
+			  dk_set_push (ret_v, list (2, box_num (uid), result));
+			}
+		    }
 		  FAILED
-		  {
-		    strcpy_ck (err_code, "IM041");
-		    strcpy_ck (err_text, "Failed reading output of FETCH command on remote IMAP server");
-		    goto error_end;
-		  }
+		    {
+		      strcpy_ck (err_code, "IM041");
+		      strcpy_ck (err_text, "Failed reading output of FETCH command on remote IMAP server");
+		      goto error_end;
+		    }
 		  END_READ_FAIL (ses);
 		}
 	    }
