@@ -1,14 +1,14 @@
 /*
  *  bif_intl.c
  *
- *  $Id: bif_intl.c,v 1.11.2.4 2010/02/03 10:29:47 source Exp $
+ *  $Id$
  *
  *  Internationalization functions
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@
 #include <wchar.h>
 #endif
 #include <limits.h>
+#include "http.h" /* For WS_CHARSET */
 #include "wi.h"
 #include "libutil.h"
 #include "sqlnode.h"
@@ -38,6 +39,7 @@
 #include "multibyte.h"
 #include "srvmultibyte.h"
 #include "xml.h"
+#include "security.h"
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -298,7 +300,7 @@ literal_as_utf8 (encoding_handler_t * enc, caddr_t literal, int len)
 }
 
 caddr_t
-charset_recode_from_named_to_named (caddr_t narrow, const char *cs1_uppercase, const char *cs2_uppercase, int *res_is_new_ret, caddr_t *err_ret)
+charset_recode_from_named_to_named (query_instance_t *qi, caddr_t narrow, const char *cs1_uppercase, const char *cs2_uppercase, int *res_is_new_ret, caddr_t *err_ret)
 {
   wcharset_t *cs1, *cs2;
   int bom_skip_offset = 0;
@@ -316,7 +318,18 @@ charset_recode_from_named_to_named (caddr_t narrow, const char *cs1_uppercase, c
     cs1 = CHARSET_WIDE;
   if (cs2_uppercase && !cs2 && !strcmp (cs2_uppercase, "_WIDE_"))
     cs2 = CHARSET_WIDE;
-
+  if (cs1_uppercase && !cs1 && !strcmp (cs1_uppercase, "_WS_"))
+    {
+      cs1 = ((NULL != qi) ? WS_CHARSET (qi->qi_client->cli_ws, qi) : NULL);
+      if (NULL == cs1)
+        cs1 = default_charset;
+    }
+  if (cs2_uppercase && !cs2 && !strcmp (cs2_uppercase, "_WS_"))
+    {
+      cs2 = ((NULL != qi) ? WS_CHARSET (qi->qi_client->cli_ws, qi) : NULL);
+      if (NULL == cs2)
+        cs2 = default_charset;
+    }
   if (!cs1 && cs1_uppercase && box_length (cs1_uppercase) > 1)
     {
       if (!stricmp (cs1_uppercase, "UTF-16") && box_length (narrow) > 2
@@ -429,7 +442,7 @@ bif_charset_recode (caddr_t *qst, caddr_t *err_ret, state_slot_t ** args)
 
   cs1_uname = cs1_name ? sqlp_box_upcase (cs1_name) : NULL;
   cs2_uname = cs2_name ? sqlp_box_upcase (cs2_name) : NULL;
-  res = charset_recode_from_named_to_named (narrow, cs1_uname, cs2_uname, &res_is_new, &err);
+  res = charset_recode_from_named_to_named ((query_instance_t *)qst, narrow, cs1_uname, cs2_uname, &res_is_new, &err);
   dk_free_box (cs1_uname); dk_free_box (cs2_uname);
   if (NULL != err)
     {
@@ -535,7 +548,10 @@ bif_uname (caddr_t *qst, caddr_t *err_ret, state_slot_t ** args)
     default:
     case 3: allow_long = bif_long_arg (qst, args, 2, "uname");
     case 2: cs1_name = bif_string_or_null_arg (qst, args, 1, "uname");
-    case 1: break;
+    case 1:
+      if (DV_UNAME == dtp)
+        return box_copy (narrow);
+      break;
     }
   cs1_uname = cs1_name ? sqlp_box_upcase (cs1_name) : NULL;
 
@@ -1029,6 +1045,15 @@ bif_dbg_assert_encoding (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return box_copy_tree (box);
 }
 
+static
+caddr_t
+bif_dbg_set_lh_xany_normalization_flags (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  sec_check_dba ((query_instance_t *)qst, "dbg_set_lh_xany_normalization_flags");
+  lh_xany_normalization_flags = bif_long_arg (qst, args, 0, "dbg_set_lh_xany_normalization_flags");
+  return box_num (lh_xany_normalization_flags);
+}
+
 wcharset_t *
 wcharset_by_name_or_dflt (ccaddr_t cs_name, query_instance_t *qi)
 {
@@ -1101,7 +1126,6 @@ get_q_of_lang_in_http_accept_language (const char *lang, const char *line)
         }
 garbage_after_q:
       while ((' ' <= tail[0]) && (',' != tail[0])) tail++;
-q_done:
       match_weight = lang_match_to_accept_language_range (lang, key, key_end);
       if (match_weight > best_match_weight)
         {
@@ -1150,6 +1174,7 @@ bif_intl_init (void)
   bif_define ("set_utf8_output", bif_set_utf8_output);
 #endif
   bif_define ("dbg_assert_encoding", bif_dbg_assert_encoding);
+  bif_define ("__dbg_set_lh_xany_normalization_flags", bif_dbg_set_lh_xany_normalization_flags);
   bif_define_typed ("langmatches_pct_http", bif_langmatches_pct_http, &bt_integer);
 }
 
