@@ -1,14 +1,14 @@
 /*
  *  bif_tidy.c
  *
- *  $Id: bif_tidy.c,v 1.2.2.4 2011/02/18 10:37:24 source Exp $
+ *  $Id$
  *
  *  Build in Functions for tidying HTML pages
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -48,7 +48,7 @@
 static dk_mutex_t *tidy_mtx;
 
 #ifndef OLD_TIDY
-static void *
+static void * TIDY_CALL
 tidy_malloc (size_t len)
 {
   if (len >= MAX_BOX_LENGTH)
@@ -56,7 +56,7 @@ tidy_malloc (size_t len)
   return t_alloc_box (len, DV_CUSTOM);
 }
 
-static void *
+static void * TIDY_CALL
 tidy_realloc (void * buf, size_t len)
 {
   int buf_size = IS_BOX_POINTER (buf) ? box_length (buf) : 0;
@@ -70,13 +70,13 @@ tidy_realloc (void * buf, size_t len)
   return new;
 }
 
-static void
+static void TIDY_CALL
 tidy_free (void * buf)
 {
   /* void, will release on MP_DONE */
 }
 
-static void
+static void TIDY_CALL
 tidy_panic (const char * err)
 {
   /* log_error ("Tidy panic: %s", err); */
