@@ -1,7 +1,7 @@
 /*
  *  blobio.c
  *
- *  $Id: blobio.c,v 1.8.2.5 2010/06/25 16:39:58 source Exp $
+ *  $Id$
  *
  *  Marshallers for DV_BLOB_HANDLE and DV_TIMESTAMP_OBJ
  *  Should really be part of Dk
@@ -9,7 +9,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -563,7 +563,9 @@ static void *
 box_read_symbol (dk_session_t *session, dtp_t dtp)
 {
   size_t length = (size_t) read_long (session);
-  char *string = (char *) dk_alloc_box (length + 1, DV_SYMBOL);
+  char *string;
+  MARSH_CHECK_LENGTH (length + 1);
+  MARSH_CHECK_BOX (string = (char *) dk_try_alloc_box (length + 1, DV_SYMBOL));
   session_buffered_read (session, string, (int) length);
   string[length] = 0;
   return (void *) string;
@@ -638,15 +640,6 @@ xx_encrypt_passwd (char *thing, int thing_len, char *user_name)
 }
 
 
-caddr_t
-box_iri_id (int64 n)
-{
-  iri_id_t * box = (iri_id_t*) dk_alloc_box (sizeof (iri_id_t), DV_IRI_ID);
-  *box = n;
-  return (caddr_t) box;
-}
-
-
 void
 iri_id_write (iri_id_t * iid, dk_session_t * ses)
 {
