@@ -1,14 +1,14 @@
 /*
  *  eqlcomp.c
  *
- *  $Id: eqlcomp.c,v 1.16.2.13 2011/03/16 10:34:26 source Exp $
+ *  $Id$
  *
  *  SQL Query Node Constructors.
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -315,8 +315,6 @@ qr_free (query_t * qr)
   while (NULL != qr->qr_used_tables) dk_free_tree (dk_set_pop (&(qr->qr_used_tables)));
   while (NULL != qr->qr_used_udts) dk_free_tree (dk_set_pop (&(qr->qr_used_udts)));
   while (NULL != qr->qr_used_jsos) dk_free_tree (dk_set_pop (&(qr->qr_used_jsos)));
-  if (!qr->qr_text_is_constant)
-    dk_free_box (qr->qr_text);
   dk_free_tree (qr->qr_parse_tree);
   DO_SET (data_source_t *, sr, &qr->qr_nodes)
   {
@@ -393,6 +391,7 @@ qr_free (query_t * qr)
       dk_free_tree (qr->qr_proc_ret_type);
       dk_free_box (qr->qr_trig_table);
       dk_free_box ((box_t) qr->qr_trig_upd_cols);
+      if (qr->qr_pn) qr->qr_pn->pn_query = NULL;
       proc_name_free (qr->qr_pn);
     }
   dk_free_box ((caddr_t) qr->qr_proc_cost);
@@ -423,6 +422,8 @@ qr_free (query_t * qr)
   if ((NULL != qr->qr_static_prev) || (NULL != qr->qr_static_next) || (qr == static_qr_dllist))
     static_qr_dllist_remove (qr);
 #endif
+  if (!qr->qr_text_is_constant)
+    dk_free_box (qr->qr_text);
   dk_free ((caddr_t) qr, sizeof (query_t));
 }
 
@@ -619,6 +620,7 @@ ssl_new_constant (comp_context_t * cc, caddr_t val)
   DO_SET (state_const_slot_t *, ssl, &cc->cc_query->qr_state_map)
     {
       if (SSL_CONSTANT == ssl->ssl_type
+        && (DV_TYPE_OF (val) == DV_TYPE_OF (ssl->ssl_const_val)) /* This check is added due to bug 14773 */
 	  && box_equal (val, ssl->ssl_const_val))
 	return (state_slot_t *)ssl;
     }
@@ -1711,6 +1713,7 @@ key_source_om (comp_context_t * cc, key_source_t * ks)
     return;
   om = (out_map_t *) dk_alloc_box (sizeof (out_map_t) * n_out, DV_BIN);
   memset (om, 0, n_out * sizeof (out_map_t));
+  ks->ks_out_map = om;
   DO_SET (dbe_column_t *, col, &ks->ks_out_cols)
     {
       if (ks->ks_key->key_bit_cl && col->col_id == ks->ks_key->key_bit_cl->cl_col_id)
@@ -1727,7 +1730,6 @@ key_source_om (comp_context_t * cc, key_source_t * ks)
 	}
     }
   END_DO_SET();
-  ks->ks_out_map = om;
 }
 
 
@@ -2455,7 +2457,7 @@ retry_dupe_check:
 	      desc->cd_scale = box_num (6);
 	    }
 	  if (desc->cd_dtp == DV_ANY)
-  	      prec = ROW_MAX_COL_BYTES;
+	    prec = 0x7fffffff;
 	  if (IS_BLOB_DTP (desc->cd_dtp))
 	    prec = 0x7fffffff;
 	  desc->cd_precision = box_num (prec);
