@@ -1,14 +1,14 @@
 /*
  *  hash.c
  *
- *  $Id: hash.c,v 1.19.2.18 2011/03/11 12:17:30 source Exp $
+ *  $Id$
  *
  *  Hash Index
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -714,7 +714,11 @@ hash_cast (query_instance_t * qi, hash_area_t * ha, int inx, state_slot_t * ssl,
       || (DV_IRI_ID_8 == target_dtp && DV_IRI_ID == dtp))
     return data;
   if (SSL_CONSTANT == ssl->ssl_type)
+#ifndef NDEBUG
     GPF_T1 ("constant ssl in hash_cast");
+#else
+    sqlr_new_error ("42000", ".....", "constant ssl in hash_cast, please report statement compiled");
+#endif
   if (IS_BLOB_DTP (target_dtp))
     target_dtp = DV_BLOB_INLINE_DTP (target_dtp);  /* non blob value for blob col. Will be inlined */
   data = box_cast_to (qst, data, dtp,
@@ -1077,9 +1081,10 @@ itc_ha_equal (it_cursor_t * itc, hash_area_t * ha, caddr_t * qst, db_buf_t hash_
 	    }
 	  if (IS_WIDE_STRING_DTP(v_dtp))
 	    {
-	      if (DVC_MATCH == compare_wide_to_utf8 ((caddr_t) (hash_row + h_off),
-		    h_len, value, box_length(value) - sizeof (wchar_t),
-		    ssl->ssl_sqt.sqt_collation))
+	      if (DVC_MATCH == compare_wide_to_utf8_with_collation (
+                  (wchar_t *)value, (box_length(value) / sizeof (wchar_t)) - 1,
+                  (utf8char *) (hash_row + h_off), h_len,
+                  ssl->ssl_sqt.sqt_collation ) )
 		continue;
 	      return DVC_LESS;
 	    }
@@ -1470,6 +1475,7 @@ ha_rehash (caddr_t * inst, hash_area_t * ha, index_tree_t * it)
   END_DO_BOX;
   dk_free_box ((caddr_t)save);
   dk_set_free (ks.ks_out_slots);
+  dk_free_box (ks.ks_out_map);
   ITC_IN_KNOWN_MAP (insert_itc, last_dp);
   page_wait_access (insert_itc, last_dp, NULL, &insert_itc->itc_hash_buf, PA_WRITE, RWG_WAIT_ANY);
   ITC_LEAVE_MAPS (insert_itc);
@@ -1669,7 +1675,7 @@ itc_ha_feed (itc_ha_feed_ret_t *ret, hash_area_t * ha, caddr_t * qst, unsigned l
   END_FAIL (itc);
 
   if (ha_rehash_pct &&  hi->hi_count > (hi->hi_size / 100) * ha_rehash_pct
-      && HI_NEXT_SIZE (hi) < HI_MAX_SIZE)
+      && HI_NEXT_SIZE (hi) < (int64)HI_MAX_SIZE)
     ha_rehash (qst, ha, tree);
 #endif
   ret->ihfr_disk_buf = itc->itc_hash_buf;
