@@ -1,14 +1,14 @@
 /*
  *  lock.c
  *
- *  $Id: lock.c,v 1.20.2.20 2011/03/23 12:25:08 source Exp $
+ *  $Id$
  *
  *  Locking concurrency control
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -368,10 +368,9 @@ void
 lt_restart (lock_trx_t * lt, int leave_flag)
 {
   lt_threads_set_inner (lt, 0);
-
   if (TRX_CONT == leave_flag)
     {
-      lt_wait_checkpoint ();
+      lt_wait_checkpoint_lt (lt);
       lt_threads_set_inner (lt, 1);
     }
   lt->lt_w_id = 0;
@@ -424,6 +423,7 @@ lt_restart (lock_trx_t * lt, int leave_flag)
       }
   }
   IN_TXN;
+  lt_wait_checkpoint_lt (lt);
   if (TRX_CONT == leave_flag || TRX_CONT_LT_LEAVE == leave_flag)
     {
       lt->lt_status = LT_PENDING;
@@ -474,6 +474,8 @@ lt_done (lock_trx_t * lt)
   {
     int64 plt = 0;
     int inx;
+    if (local_cll.cll_id_to_trx)
+      gethash_64 (plt, lt->lt_trx_no, local_cll.cll_id_to_trx);
     if (plt)
       GPF_T1 ("lt in id to trx  at lt_done");
     for (inx = 0; inx < trx_rc->rc_fill; inx++)
@@ -667,22 +669,27 @@ lt_rollback_1 (lock_trx_t * lt, int free_trx)
 void
 lt_ack_freeze_inner (lock_trx_t * lt)
 {
-
+  du_thread_t *self = THREAD_CURRENT_THREAD;
   ASSERT_IN_TXN;
   /* Can be called from lt_leave, no itc and bufs but must ack for cpt to proceed */
   lt->lt_status = LT_FREEZE;
   lt->lt_error = LTE_OK;
-  LT_CLOSE_ACK_THREADS(lt);
+  LT_CLOSE_ACK_THREADS (lt);
   lt->lt_close_ack_threads = 1;
   lt_resume_waiting_end (lt);
-  lt_wait_checkpoint();
-  LT_CLOSE_ACK_THREADS(lt);
-  lt->lt_close_ack_threads = 0;
+  self->thr_lt = lt;
+  lt_wait_checkpoint ();
+  self->thr_lt = NULL;
   if (LT_FREEZE == lt->lt_status)
-    lt->lt_status = LT_PENDING;
+    {
+      LT_CLOSE_ACK_THREADS (lt);
+      lt->lt_close_ack_threads = 0;
+      lt->lt_status = LT_PENDING;
+    }
 }
 
 
+
 void
 lt_ack_freeze (lock_trx_t * lt, it_cursor_t * itc, buffer_desc_t ** buf_ret)
 {
