@@ -1,14 +1,14 @@
 /*
  *  log.c
  *
- *  $Id: log.c,v 1.16.2.21 2011/01/25 18:54:23 source Exp $
+ *  $Id$
  *
  *  Transaction log write and recovery
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -159,6 +159,18 @@ log_set_server_version_check (int in_txn)
 
   sprintf (tmp, "server_version_check ('%s')", DBMS_SRV_VER);
   log_set_compatibility_check (in_txn, tmp);
+  if (wi_inst.wi_master->dbs_id[0])
+    {
+      int inx;
+      char hex[33];
+      for (inx = 0; inx < sizeof (wi_inst.wi_master->dbs_id); inx ++)
+	{
+	  sprintf (hex + (inx * 2), "%02x", (unsigned char) (wi_inst.wi_master->dbs_id[inx]));
+	}
+      hex[32] = 0;
+      sprintf (tmp, "server_id_check ('%s')", hex);
+      log_set_compatibility_check (in_txn, tmp);
+    }
 }
 
 
@@ -1409,7 +1421,7 @@ log_replay_text (lock_trx_t * lt, dk_session_t * in, int is_pushback, int use_st
 cr_done:
   if (use_stmt_cache)
     {
-      sst = cli_get_stmt_access (lt->lt_client, stmt_id, GET_EXCLUSIVE);
+      sst = cli_get_stmt_access (lt->lt_client, stmt_id, GET_EXCLUSIVE, NULL);
       err = stmt_set_query (sst, lt->lt_client, text, opts);
       LEAVE_CLIENT (lt->lt_client);
       if (err != NULL)
