@@ -1,14 +1,14 @@
 /*
  *  obackup.c
  *
- *  $Id: obackup.c,v 1.20.2.12 2010/06/10 18:46:26 source Exp $
+ *  $Id$
  *
  *  Online & Incremental Backup
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -1097,8 +1097,8 @@ long ol_backup (const char* prefix, long pages, long timeout, caddr_t* backup_pa
       fclose (obackup_trace);
       obackup_trace = NULL;
     }
+  log_info ("Backed up pages: [%ld]", ctx->octx_page_count - _pages);
 #ifdef DEBUG
-  log_info ("Backed up pages: [%ld]", ctx->octx_page_count);
   log_info ("Log = %s", wi_inst.wi_master->dbs_log_name);
 #endif
 
@@ -1136,26 +1136,28 @@ void bp_sec_user_check (query_instance_t * qi)
     }
 }
 
-void bp_sec_check_prefix (query_instance_t * qi, char * file_prefix)
+void
+bp_sec_check_prefix (query_instance_t * qi, char *file_prefix)
 {
   char * s;
+
   if (!file_prefix[0])
     sqlr_new_error ("42000", FILE_FORM_ERR_CODE , "Backup prefix must contains at least one char");
 
+  if (file_prefix[0] == '/')
+    sqlr_new_error ("42000", FILE_FORM_ERR_CODE, "Absolute path as backup prefix is not allowed");
 
   s = strchr (file_prefix, ':');
-
   if (s)
     sqlr_new_error ("42000", FILE_FORM_ERR_CODE , "Semicolon in backup prefix is not allowed");
 
-  do { /* .. check */
     s = strchr (file_prefix, '.');
-    if (s && s[1] == '.')
+  while (s)
+    {
+      if (s[1] == '.')
       sqlr_new_error ("42000", FILE_FORM_ERR_CODE , "\"..\" substring in backup prefix is not allowed");
-  } while (s);
-
-  if (file_prefix[0] == '/')
-    sqlr_new_error ("42000", FILE_FORM_ERR_CODE , "Absolute path as backup prefix is not allowed");
+      s = strchr (s + 1, '.');
+    }
 }
 
 
