@@ -1,14 +1,14 @@
 /*
  *  odbcinc.h
  *
- *  $Id: odbcinc.h,v 1.6.2.5 2010/09/17 22:49:23 source Exp $
+ *  $Id$
  *
  *  Include the ODBC header, whichever appropriate
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -138,7 +138,8 @@ typedef void * SQLHANDLE;
 		   strstr (rds_get_info (rds, SQL_DBMS_NAME), "S Q L   S e r v e r") \
 		  ) && \
 		  !(strstr (rds_get_info (rds, SQL_DBMS_NAME), "Microsoft") || \
-		   strstr (rds_get_info (rds, SQL_DBMS_NAME), "M i c r o s o f t") \
+		   strstr (rds_get_info (rds, SQL_DBMS_NAME), "M i c r o s o f t") || \
+		   strstr (rds_get_info (rds, SQL_DBMS_NAME), "MS SQL Server") \
 		  ) \
 		)
 #define IS_VIRTUOSO_RDS(rds) (strstr (rds_get_info (rds, SQL_DBMS_NAME), "Virtuoso") != NULL)
