@@ -1,12 +1,12 @@
 /*
- *  $Id: page.c,v 1.1.2.15 2010/02/23 14:49:24 source Exp $
+ *  $Id$
  *
  *  Page and Row Layout, Key Compression
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2009 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -913,6 +913,8 @@ buf_order_ck (buffer_desc_t * buf)
 {
   int inx;
   page_map_t * pm = buf->bd_content_map;
+  if (buf->bd_tree->it_key->key_id == KI_TEMP)
+    return;
   for (inx = 0; inx < pm->pm_count - 1; inx++)
     {
       if (DVC_LESS != buf_row_compare (buf, inx, inx + 1))
@@ -925,9 +927,20 @@ buf_order_ck (buffer_desc_t * buf)
 #define buf_order_ck(b)
 #endif
 
+#ifdef NDEBUG
+static long
+pf_count_registered (buffer_desc_t * buf)
+{
+  long cr_fill = 0;
+  it_cursor_t * cr = buf->bd_registered;
+  for (cr = cr; cr; cr = cr->itc_next_on_page)
+    cr_fill++;
+  return cr_fill;
+}
+#endif
 
 void
-pf_fill_registered (page_fill_t * pf, buffer_desc_t * buf)
+pf_fill_registered (page_fill_t * pf, buffer_desc_t * buf, it_cursor_t * itc)
 {
   int cr_fill = 0;
   it_cursor_t * cr = buf->bd_registered;
@@ -936,7 +949,14 @@ pf_fill_registered (page_fill_t * pf, buffer_desc_t * buf)
     {
       pf->pf_registered[cr_fill++] = (placeholder_t *) cr;
       if (cr_fill >= MAX_ITCS_ON_PAGE)
-	GPF_T1 ("too many cursors on splitting page.");
+	{
+#ifdef NDEBUG
+	  log_error ("Too many cursors: %ld on splitting page.", pf_count_registered (buf));
+	  itc_bust_this_trx (itc, &buf, ITC_BUST_THROW);
+#endif
+	  /* should never return*/
+	  GPF_T1 ("too many cursors on splitting page.");
+	}
     }
   pf->pf_cr_fill = cr_fill;
 }
@@ -1650,6 +1670,25 @@ row_refit_col (buffer_desc_t * buf, int map_pos, dbe_key_t * key, db_buf_t row,
   return 1;
 }
 
+int
+row_refit_col_space_needed (buffer_desc_t * buf, int map_pos, dbe_key_t * key, db_buf_t row, dbe_col_loc_t * cl, db_buf_t bytes, int new_len, int * space_ret)
+{
+  /* insert the new val if fits.  If If fits, return the space left in space_ret.  If did not fit, return how much extra space is needed in space_ret */
+  int space = *space_ret;
+  short off, len;
+  pg_check_map (buf);
+  if (DV_DB_NULL == DV_TYPE_OF (bytes))
+    new_len = 0;
+  KEY_PRESENT_VAR_COL (key, row, (*cl), off, len);
+  if (new_len - len > space)
+    {
+      *space_ret = new_len - len;
+      return 0;
+    }
+  *space_ret -= new_len - len;
+  return 1;
+}
+
 
 void
 page_row_spacing (buffer_desc_t * buf, short row_gap, short ins_inx, int ins_gap)
@@ -1693,12 +1732,13 @@ pf_rd_refit_1 (page_fill_t * pf, row_delta_t * rd, int recursive)
   db_buf_t row = buf->bd_buffer + pm->pm_entries[rd->rd_map_pos];
   dbe_key_t * key = rd->rd_key->key_versions[IE_KEY_VERSION (row)];
   int inx, rc = 0;
-  int space = row_space_after (buf, rd->rd_map_pos), avail, gap;
+  int space = row_space_after (buf, rd->rd_map_pos), avail, gap = 0, space_needed = 0, space_before = space;
   if (!buf->bd_is_write || !buf->bd_is_dirty) GPF_T1 ("refit1 for non excl or non dirty buffer");
   for (inx = 0; inx < rd->rd_n_values; inx++)
     {
       if (rd->rd_upd_change[inx])
 	{
+	  space_before = space;
 	  rc = row_refit_col (buf, rd->rd_map_pos, key, row, rd->rd_upd_change[inx], rd->rd_values[inx], box_length_on_row (rd->rd_values[inx]), &space);
 	  if (!rc)
 	    break;
@@ -1714,8 +1754,26 @@ pf_rd_refit_1 (page_fill_t * pf, row_delta_t * rd, int recursive)
       return 1;
     }
   /* the cols did not fit.  See if need rearrange or split */
-  if (recursive) GPF_T1 ("still did not refit on recursive call");
-  space = ROW_ALIGN (space);
+  if (recursive)
+    {
+      log_error ("can not refit row key: %s space %d, gap: %d pm free: %d, pm count: %d",
+	  	rd->rd_key ? rd->rd_key->key_name : "no key", space, gap, pm->pm_bytes_free, pm->pm_count);
+      GPF_T1 ("still did not refit on recursive call");
+    }
+  for (space = space_before; inx < rd->rd_n_values; inx++)
+    {
+      if (rd->rd_upd_change[inx])
+	{
+	  space_before = space;
+	  rc = row_refit_col_space_needed (buf, rd->rd_map_pos, key, row, rd->rd_upd_change[inx], rd->rd_values[inx], box_length_on_row (rd->rd_values[inx]), &space);
+	  if (!rc)
+	    {
+	      space_needed += space;
+	      space = space_before;
+	    }
+	}
+    }
+  space = ROW_ALIGN (space_needed);
     if (space > pm->pm_bytes_free)
     {
       rd->rd_op = RD_UPDATE;
@@ -2243,7 +2301,7 @@ page_apply_1 (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t *
   pf.pf_org = buf;
   pf.pf_current = t_buf;
   if (op != PA_REWRITE_ONLY)
-    pf_fill_registered (&pf, buf);
+    pf_fill_registered (&pf, buf, itc);
   if (org_pm->pm_bytes_free < bytes_delta)
     {
       if (!itc->itc_n_pages_on_hold && op != PA_REWRITE_ONLY)
@@ -2348,7 +2406,9 @@ page_apply_1 (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t *
 	    GPF_T1 ("unmoved non-deleted row lock");
 	  PL_RL_ADD (buf->bd_pl, rlocks[inx], ITC_AT_END);
 	  buf->bd_pl->pl_n_row_locks++;
+#ifndef NDEBUG
 	  log_info ("deleted rl kept around for page apply");
+#endif
 	}
     }
   if (pf.pf_left)
@@ -2363,7 +2423,8 @@ page_apply_1 (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t *
   if (t_buf->bd_registered) GPF_T1 ("registrations are not supposed to go to the temp buf");
   if (PA_REWRITE_ONLY == op)
     return;
-  if (PA_AUTOCOMPACT == op)
+  if (PA_AUTOCOMPACT == op
+     )
     first_affected = 0;
   page_apply_parent (buf, &pf, first_affected, op, change, paf);
   dk_set_free (pf.pf_left);
@@ -2376,6 +2437,9 @@ void
 page_apply_s (it_cursor_t * itc, buffer_desc_t * buf, int n_delta, row_delta_t ** delta, int op)
 {
   page_apply_frame_t paf;
+#ifdef VALGRIND
+  memset (&paf, 0, sizeof (page_apply_frame_t));
+#endif
   page_apply_1 (itc, buf, n_delta, delta, op, &paf);
 }
 
