@@ -1,10 +1,10 @@
 /*  */
-/* $Id: rdf_mapping.jso,v 1.24.2.4 2010/09/17 22:49:23 source Exp $ */
+/* $Id$ */
 /*  */
 /* This file is part of the OpenLink Software Virtuoso Open-Source (VOS) */
 /* project. */
 /*  */
-/* Copyright (C) 1998-2009 OpenLink Software */
+/* Copyright (C) 1998-2013 OpenLink Software */
 /*  */
 /* This project is free software; you can redistribute it and/or modify it */
 /* under the terms of the GNU General Public License as published by the */
@@ -24,12 +24,15 @@
 #include "rdf_mapping_jso.h"
 /* Flags listed below this line are for temporary use, they never appear in databases sho they may be changed. */
 /* Flags of actions to be done in case of match of triple pattern to conditions of the quad map */
+/* Flags that are responsible for various serialization features. */
+/* Some features are labeled as "blocking", because if such a feature is required but flag is not set, an error is signaled. */
+/* An occurrence of a non-blocking feature provides some hint to the optimizer of the SPARQL service endpoint; a blocking one alters semantics. */
 
 jso_class_descr_t jso__rdf_val_range_array = {
   JSO_CAT_ARRAY,  "array of struct rdf_val_range_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-ValRange" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-ValRange" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#ValRange" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -51,17 +54,17 @@ jso_class_descr_t jso__rdf_val_range = {
   JSO_CAT_STRUCT,  "struct rdf_val_range_s",
   "http://www.openlinksw.com/schemas/virtrdf#ValRange" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "ValRange" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (rdf_val_range_t),
-      -1, jso_fields__rdf_val_range, NULL },
+      -1, jso_fields__rdf_val_range, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__qm_format_array = {
   JSO_CAT_ARRAY,  "array of struct qm_format_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMapFormat" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMapFormat" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMapFormat" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -79,6 +82,7 @@ jso_field_descr_t jso_fields__qm_format[] = {
   { NULL	, "qmfIsuriOfShortTmpl"	, JSO_STRING	, JSO_REQUIRED	, JSO_FIELD_OFFSET(qm_format_t,qmfIsuriOfShortTmpl)	, NULL },
   { NULL	, "qmfIsblankOfShortTmpl"	, JSO_STRING	, JSO_REQUIRED	, JSO_FIELD_OFFSET(qm_format_t,qmfIsblankOfShortTmpl)	, NULL },
   { NULL	, "qmfIslitOfShortTmpl"	, JSO_STRING	, JSO_REQUIRED	, JSO_FIELD_OFFSET(qm_format_t,qmfIslitOfShortTmpl)	, NULL },
+  { NULL	, "qmfIsnumericOfShortTmpl"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfIsnumericOfShortTmpl)	, NULL },
 /* Templates of 0/1 integer expressions that tell whether the short is of some sort: */
   { NULL	, "qmf01uriOfShortTmpl"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmf01uriOfShortTmpl)	, NULL },
   { NULL	, "qmf01blankOfShortTmpl"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmf01blankOfShortTmpl)	, NULL },
@@ -106,6 +110,7 @@ jso_field_descr_t jso_fields__qm_format[] = {
   { NULL	, "qmfExistingShortOfNiceSqlvalTmpl"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfExistingShortOfNiceSqlvalTmpl)	, NULL },
 /* Misc */
   { NULL	, "qmfIsBijection"	, JSO_BOOLEAN	, JSO_REQUIRED	, JSO_FIELD_OFFSET(qm_format_t,qmfIsBijection)	, NULL },
+  { NULL	, "qmfMapsOnlyNullToNull"	, JSO_BOOLEAN	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfMapsOnlyNullToNull)	, NULL },
   { NULL	, "qmfDerefFlags"	, JSO_BOOLEAN	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfDerefFlags)	, NULL },
   { NULL	, "qmfIsStable"	, JSO_BOOLEAN	, JSO_REQUIRED	, JSO_FIELD_OFFSET(qm_format_t,qmfIsStable)	, NULL },
   { NULL	, "qmfIsSubformatOfLong"	, JSO_BOOLEAN	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfIsSubformatOfLong)	, NULL },
@@ -121,6 +126,7 @@ jso_field_descr_t jso_fields__qm_format[] = {
   { NULL	, "qmfUriIdOffset"	, JSO_INTEGER	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfUriIdOffset)	, NULL },
   { NULL	, "qmfCustomString1"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfCustomString1)	, NULL },
   { NULL	, "qmfArgDtps"	, JSO_STRING	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfArgDtps)	, NULL },
+  { NULL	, "qmfSubFormatForRefs"	, JSO_IRI_OF_qm_format	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfSubFormatForRefs)	, NULL },
   { NULL	, "qmfSuperFormats"	, JSO_IRI_OF_qm_format_array	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfSuperFormats)	, NULL },
   { NULL	, "qmfValRange-rvrRestrictions"	, JSO_BITMASK	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfValRange.rvrRestrictions)	, NULL },
   { NULL	, "qmfValRange-rvrDatatype"	, JSO_ANY_URI	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_format_t,qmfValRange.rvrDatatype)	, NULL },
@@ -134,17 +140,17 @@ jso_class_descr_t jso__qm_format = {
   JSO_CAT_STRUCT,  "struct qm_format_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMapFormat" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMapFormat" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (qm_format_t),
-      -1, jso_fields__qm_format, NULL },
+      -1, jso_fields__qm_format, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__qm_atable_array = {
   JSO_CAT_ARRAY,  "array of struct qm_atable_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMapATable" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMapATable" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMapATable" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -157,17 +163,17 @@ jso_class_descr_t jso__qm_atable = {
   JSO_CAT_STRUCT,  "struct qm_atable_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMapATable" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMapATable" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (qm_atable_t),
-      -1, jso_fields__qm_atable, NULL },
+      -1, jso_fields__qm_atable, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__qm_column_array = {
   JSO_CAT_ARRAY,  "array of struct qm_column_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMapColumn" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMapColumn" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMapColumn" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -181,17 +187,17 @@ jso_class_descr_t jso__qm_column = {
   JSO_CAT_STRUCT,  "struct qm_column_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMapColumn" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMapColumn" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (qm_column_t),
-      -1, jso_fields__qm_column, NULL },
+      -1, jso_fields__qm_column, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__qm_ftext_array = {
   JSO_CAT_ARRAY,  "array of struct qm_ftext_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMapFText" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMapFText" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMapFText" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -209,17 +215,17 @@ jso_class_descr_t jso__qm_ftext = {
   JSO_CAT_STRUCT,  "struct qm_ftext_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMapFText" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMapFText" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (qm_ftext_t),
-      -1, jso_fields__qm_ftext, NULL },
+      -1, jso_fields__qm_ftext, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__qm_value_array = {
   JSO_CAT_ARRAY,  "array of struct qm_value_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMapValue" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMapValue" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMapValue" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -238,23 +244,24 @@ jso_field_descr_t jso_fields__qm_value[] = {
   { NULL	, "qmvRange-rvrDatatype"	, JSO_ANY_URI	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_value_t,qmvRange.rvrDatatype)	, NULL },
   { NULL	, "qmvRange-rvrLanguage"	, JSO_ANY_URI	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_value_t,qmvRange.rvrLanguage)	, NULL },
   { NULL	, "qmvRange-rvrFixedValue"	, JSO_ANY	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(qm_value_t,qmvRange.rvrFixedValue)	, NULL },
+  { NULL	, "qmvHash"	, JSO_STRING	, JSO_PRIVATE	, JSO_FIELD_OFFSET(qm_value_t,qmvHash)	, NULL },
   { NULL	, NULL	, NULL	,0xdeadce11	,0xdeadce11	,NULL } };
 
 jso_class_descr_t jso__qm_value = {
   JSO_CAT_STRUCT,  "struct qm_value_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMapValue" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMapValue" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (qm_value_t),
-      -1, jso_fields__qm_value, NULL },
+      -1, jso_fields__qm_value, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__quad_map_array = {
   JSO_CAT_ARRAY,  "array of struct quad_map_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMap" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadMap" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadMap" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -297,17 +304,42 @@ jso_class_descr_t jso__quad_map = {
   JSO_CAT_STRUCT,  "struct quad_map_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadMap" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadMap" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (quad_map_t),
-      -1, jso_fields__quad_map, NULL },
+      -1, jso_fields__quad_map, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
+    { NULL, 0, 0} } };
+
+jso_class_descr_t jso__sparql_macro_library_array = {
+  JSO_CAT_ARRAY,  "array of struct sparql_macro_library_s *",
+  "http://www.openlinksw.com/schemas/virtrdf#array-of-SparqlMacroLibrary" ,
+  "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-SparqlMacroLibrary" ,
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
+    { 0, -1, NULL, NULL },
+    {  "http://www.openlinksw.com/schemas/virtrdf#SparqlMacroLibrary" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
+
+jso_field_descr_t jso_fields__sparql_macro_library[] = {
+  { NULL	, "smlSourceText"	, JSO_STRING	, JSO_REQUIRED	, JSO_FIELD_OFFSET(sparql_macro_library_t,smlSourceText)	, NULL },
+  { NULL	, "smlCompilationState"	, JSO_INTEGER	, JSO_PRIVATE	, JSO_FIELD_OFFSET(sparql_macro_library_t,smlCompilationState)	, NULL },
+  { NULL	, "smlIncludes"	, JSO_ANY	, JSO_PRIVATE	, JSO_FIELD_OFFSET(sparql_macro_library_t,smlIncludes)	, NULL },
+  { NULL	, "smlList"	, JSO_ANY	, JSO_PRIVATE	, JSO_FIELD_OFFSET(sparql_macro_library_t,smlList)	, NULL },
+  { NULL	, NULL	, NULL	,0xdeadce11	,0xdeadce11	,NULL } };
+
+jso_class_descr_t jso__sparql_macro_library = {
+  JSO_CAT_STRUCT,  "struct sparql_macro_library_s",
+  "http://www.openlinksw.com/schemas/virtrdf#SparqlMacroLibrary" ,
+  "http://www.openlinksw.com/schemas/virtrdf#" , "SparqlMacroLibrary" ,
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
+    {
+      sizeof (sparql_macro_library_t),
+      -1, jso_fields__sparql_macro_library, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 jso_class_descr_t jso__quad_storage_array = {
   JSO_CAT_ARRAY,  "array of struct quad_storage_s *",
   "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadStorage" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "array-of-QuadStorage" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     { 0, -1, NULL, NULL },
     {  "http://www.openlinksw.com/schemas/virtrdf#QuadStorage" , 0, (SMALLEST_POSSIBLE_POINTER-2)} } };
 
@@ -315,6 +347,7 @@ jso_field_descr_t jso_fields__quad_storage[] = {
   { NULL	, "qsMjvMaps"	, JSO_IRI_OF_quad_map_array	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsMjvMaps)	, NULL },
   { NULL	, "qsUserMaps"	, JSO_IRI_OF_quad_map_array	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsUserMaps)	, NULL },
   { NULL	, "qsDefaultMap"	, JSO_IRI_OF_quad_map	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsDefaultMap)	, NULL },
+  { NULL	, "qsMacroLibrary"	, JSO_IRI_OF_sparql_macro_library	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsMacroLibrary)	, NULL },
   { NULL	, "qsMatchingFlags"	, JSO_BITMASK	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsMatchingFlags)	, NULL },
   { NULL	, "qsAlterInProgress"	, JSO_ANY	, JSO_OPTIONAL	, JSO_FIELD_OFFSET(quad_storage_t,qsAlterInProgress)	, NULL },
   { NULL	, NULL	, NULL	,0xdeadce11	,0xdeadce11	,NULL } };
@@ -323,10 +356,10 @@ jso_class_descr_t jso__quad_storage = {
   JSO_CAT_STRUCT,  "struct quad_storage_s",
   "http://www.openlinksw.com/schemas/virtrdf#QuadStorage" ,
   "http://www.openlinksw.com/schemas/virtrdf#" , "QuadStorage" ,
-  NULL, {
+  NULL /* jsocd_validation_cbk */, NULL /* jsocd_rttis */, {
     {
       sizeof (quad_storage_t),
-      -1, jso_fields__quad_storage, NULL },
+      -1, jso_fields__quad_storage, NULL /* jsosd_field_hash */, NULL /* jsosd_fields_by_idx */ },
     { NULL, 0, 0} } };
 
 void
@@ -355,6 +388,23 @@ rdf_mapping_jso_init (void)
   jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SPART_QM_SOFT_EXCLUSIVE", SPART_QM_SOFT_EXCLUSIVE);
   jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SPART_QS_NO_IMPLICIT_USER_QM", SPART_QS_NO_IMPLICIT_USER_QM);
   jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SPART_QS_BLOCK_DEFAULT_QM", SPART_QS_BLOCK_DEFAULT_QM);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_QUAD_MAP", SSG_SD_QUAD_MAP);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_OPTION", SSG_SD_OPTION);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_BREAKUP", SSG_SD_BREAKUP);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_PKSELFJOIN", SSG_SD_PKSELFJOIN);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_RVR", SSG_SD_RVR);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_IN", SSG_SD_IN);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_LIKE", SSG_SD_LIKE);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_GLOBALS", SSG_SD_GLOBALS);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_BI", SSG_SD_BI);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_VIRTSPECIFIC", SSG_SD_VIRTSPECIFIC);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_VOS_509", SSG_SD_VOS_509);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_SERVICE", SSG_SD_SERVICE);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_VOS_5_LATEST", SSG_SD_VOS_5_LATEST);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_TRANSIT", SSG_SD_TRANSIT);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_VOS_6", SSG_SD_VOS_6);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_SPARQL11_DRAFT", SSG_SD_SPARQL11_DRAFT);
+  jso_define_const( "http://www.openlinksw.com/schemas/virtrdf#SSG_SD_SPARQL11_MORE", SSG_SD_SPARQL11_MORE);
   jso_define_class(&jso__rdf_val_range_array);
   jso_define_class(&jso__rdf_val_range);
   jso_define_class(&jso__qm_format_array);
@@ -369,6 +419,8 @@ rdf_mapping_jso_init (void)
   jso_define_class(&jso__qm_value);
   jso_define_class(&jso__quad_map_array);
   jso_define_class(&jso__quad_map);
+  jso_define_class(&jso__sparql_macro_library_array);
+  jso_define_class(&jso__sparql_macro_library);
   jso_define_class(&jso__quad_storage_array);
   jso_define_class(&jso__quad_storage);
 }
