@@ -6,7 +6,7 @@
 /* This file is part of the OpenLink Software Virtuoso Open-Source (VOS) */
 /* project. */
 /*  */
-/* Copyright (C) 1998-2012 OpenLink Software */
+/* Copyright (C) 1998-2013 OpenLink Software */
 /*  */
 /* This project is free software; you can redistribute it and/or modify it */
 /* under the terms of the GNU General Public License as published by the */
@@ -138,6 +138,7 @@ typedef struct qm_format_s
   ccaddr_t	qmfExistingShortOfNiceSqlvalTmpl;	/*!< ... works like qmfExistingShortOfSqlvalTmpl for values of one selected data type, not suitable for other types */
 /* Misc */
   ptrlong	qmfIsBijection;	/*!< Nonzero if the format is bijection so long values are equal IFF short values of the type are equal */
+  ptrlong	qmfMapsOnlyNullToNull;	/*!< Nonzero if the format gives NULL IFF short value is NULL */
   ptrlong	qmfDerefFlags;	/*!< Flags of IRI dereferencing */
   ptrlong	qmfIsStable;	/*!< Nonzero if sql values are in order IFF of short values are in order */
   ptrlong	qmfIsSubformatOfLong;	/*!< Nonzero if the format is subformat of LONG valmode */
@@ -153,6 +154,7 @@ typedef struct qm_format_s
   ptrlong	qmfUriIdOffset;	/*!< The value that should be added to locally stored ref id in order to get portable ref id */
   ccaddr_t	qmfCustomString1;	/*!< The value that can be used as a value of ^{custom-string-1}^ in templates */
   ccaddr_t	qmfArgDtps;	/*!< String of dtp_t bytes, can be used as a value of ^{opt-comma-arg-dtps}^ in templates */
+  struct qm_format_s *	qmfSubFormatForRefs;	/*!< Subformat that supports all refs (IRIs and bnodes) that are supported by this format (and NULLs if they're supported by this formats) */
   qm_format_array_t	qmfSuperFormats;	/*!< Array of formats that can store any value that current format can store */
   rdf_val_range_t qmfValRange;	/*! Natural restrictions on values stored in the source column of format */
 } qm_format_t;
@@ -214,6 +216,7 @@ typedef struct qm_value_s
   ccaddr_t	qmvColumnsToIRI;	/*!<  */
   ccaddr_t	qmvIriClass;	/*!< Known IRI class, if the IRI is calculable */
   rdf_val_range_t qmvRange;	/*! Natural restrictions on values stored in the source column of format */
+  ccaddr_t	qmvHash;	/*!< Hash that uniquely identifies the qmv structure (its more reproducible than an address) */
 } qm_value_t;
 
 #define JSO_IRI_OF_quad_map_array  "http://www.openlinksw.com/schemas/virtrdf#array-of-QuadMap"
