@@ -1,14 +1,14 @@
 /*
  *  rdfinf.c
  *
- *  $Id: rdfinf.c,v 1.19.2.16 2010/02/01 12:53:43 source Exp $
+ *  $Id$
  *
  *  RDF Inference
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -235,9 +235,9 @@ ri_list (rdf_inf_pre_node_t * ri, caddr_t iri, rdf_sub_t ** sub_ret)
     {
       switch (ri->ri_mode)
 	{
-	case RI_SUBCLASS: return sub->rs_sub;
+	  case RI_SUBCLASS: return sub->rs_sub ? sub->rs_sub : sub->rs_equiv;
 	case RI_SUPERCLASS: return sub->rs_super;
-	case RI_SUBPROPERTY: return sub->rs_sub;
+	  case RI_SUBPROPERTY: return sub->rs_sub ? sub->rs_sub : sub->rs_equiv;
 	case RI_SUPERPROPERTY: return sub->rs_super;
 	}
     }
@@ -320,20 +320,37 @@ iri_ensure (caddr_t * qst, caddr_t name, int flag, caddr_t * err_ret)
 }
 
 
-char * sas_1_text = "select s from rdf_quad where g = ? and o = ? and p = ? option (quietcast)";
-char * sas_2_text = "select o from rdf_quad where g = ? and s = ? and p = ? option (quietcast)";
-char * sas_tn_text = "select O from RDF_QUAD where S = :0 and P = rdf_sas_iri () and G = :1 union all select S from RDF_QUAD where O = :0 and P = rdf_sas_iri () and G = :1 option (quietcast, array)";
-char * sas_tn_no_graph_text = "select O from RDF_QUAD where S = :0 and P = rdf_sas_iri () union all select S from RDF_QUAD where O = :0 and P = rdf_sas_iri () option (quietcast, array)";
-char * tn_ifp_text = "select s from rdf_quad where p in (rdf_inf_ifp_list (:1)) and o = :0 and not isiri_id (:0) and G in (:2) and not rdf_inf_ifp_is_excluded (:1, P, :0) "
-  " union all select syn.s from rdf_quad org, rdf_quad syn where org.p in (rdf_inf_ifp_list (:1)) and syn.p = org.p and org.s = :0 and isiri_id (:0) and syn.o = org.o and org.G in (:2) and syn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) option (any order)";
-char * tn_ifp_no_graph_text = "select s from rdf_quad where p in (rdf_inf_ifp_list (:1)) and o = :0 and not isiri_id (:0) and not rdf_inf_ifp_is_excluded (:1, P, :0) "
-" union all select syn.s from rdf_quad org, rdf_quad syn where org.p in (rdf_inf_ifp_list (:1)) and syn.p = org.p and org.s = :0 and isiri_id (:0) and syn.o = org.o and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) option (any order)";
+char * sas_1_text = "select S from DB.DBA.RDF_QUAD where G = ? and O = ? and P = ? option (quietcast)";
+char * sas_2_text = "select O from DB.DBA.RDF_QUAD where G = ? and S = ? and P = ? option (quietcast)";
+char * sas_tn_text = "select O from DB.DBA.RDF_QUAD where S = :0 and P = rdf_sas_iri () and G = :1 and isiri_id (O) union all select S from DB.DBA.RDF_QUAD where O = :0 and P = rdf_sas_iri () and G = :1 option (quietcast, array)";
+char * sas_tn_no_graph_text = "select O from DB.DBA.RDF_QUAD where S = :0 and P = rdf_sas_iri () union all select S from DB.DBA.RDF_QUAD where O = :0 and P = rdf_sas_iri () option (quietcast, array)";
+char * tn_ifp_text =
+  " select S from DB.DBA.RDF_QUAD "
+  " where P in (rdf_inf_ifp_list (:1)) and O = :0 and not isiri_id (:0) and G in (:2) and not rdf_inf_ifp_is_excluded (:1, P, :0) "
+  " union all select syn.S from DB.DBA.RDF_QUAD org, DB.DBA.RDF_QUAD syn "
+  " where org.P in (rdf_inf_ifp_list (:1)) and syn.P = org.P and org.S = :0 and isiri_id (:0) and syn.O = org.O and org.G in (:2) and syn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, org.P, org.O)"
+  " union all select rsyn.S from DB.DBA.RDF_QUAD rorg, DB.DBA.RDF_QUAD rsyn "
+  " where rorg.P in (rdf_inf_ifp_rel_list (:1)) and rsyn.P in (rdf_inf_ifp_rel_list (:1, rorg.P)) and rorg.S = :0 and isiri_id (:0) and rsyn.O = rorg.O and rorg.G in (:2) and rsyn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, rorg.P, rorg.O) "
+  " option (any order)";
+char * tn_ifp_no_graph_text =
+  "select S from DB.DBA.RDF_QUAD "
+  " where P in (rdf_inf_ifp_list (:1)) and o = :0 and not isiri_id (:0) and not rdf_inf_ifp_is_excluded (:1, P, :0) "
+  " union all select syn.s from DB.DBA.RDF_QUAD org, DB.DBA.RDF_QUAD syn "
+  " where org.P in (rdf_inf_ifp_list (:1)) and syn.P = org.P and org.S = :0 and isiri_id (:0) and syn.o = org.O and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) "
+  " union all select rsyn.s from DB.DBA.RDF_QUAD rorg, DB.DBA.RDF_QUAD rsyn "
+  " where rorg.P in (rdf_inf_ifp_rel_list (:1)) and rsyn.P in (rdf_inf_ifp_rel_list (:1, rorg.P)) and rorg.S = :0 and isiri_id (:0) and rsyn.o = rorg.O and not rdf_inf_ifp_is_excluded (:1, rorg.P, rorg.O) "
+  " option (any order)";
 char * tn_ifp_dist_text =
-  " select syn.s from rdf_quad org, rdf_quad syn where org.p in (rdf_inf_ifp_list (:1)) and syn.p = org.p and org.s = :0 and isiri_id (:0) and syn.o = org.o and org.G in (:2) and syn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) option (any order)";
+  " select syn.s from DB.DBA.RDF_QUAD org, DB.DBA.RDF_QUAD syn "
+  " where org.P in (rdf_inf_ifp_list (:1)) and syn.P = org.P and org.s = :0 and isiri_id (:0) and syn.O = org.O and org.G in (:2) and syn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) "
+  " union all select rsyn.S from DB.DBA.RDF_QUAD rorg, DB.DBA.RDF_QUAD rsyn "
+  " where rorg.P in (rdf_inf_ifp_rel_list (:1)) and rsyn.P in (rdf_inf_ifp_rel_list (:1, rorg.P)) and rorg.S = :0 and isiri_id (:0) and rsyn.O = rorg.O and rorg.G in (:2) and rsyn.G in (:2) and not rdf_inf_ifp_is_excluded (:1, rorg.P, rorg.O) "
+  " option (any order)";
 char * tn_ifp_dist_no_graph_text =
-" select syn.s from rdf_quad org, rdf_quad syn where org.p in (rdf_inf_ifp_list (:1)) and syn.p = org.p and org.s = :0 and isiri_id (:0) and syn.o = org.o and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) option (any order)";
-
-
+  " select syn.s from DB.DBA.RDF_QUAD org, DB.DBA.RDF_QUAD syn where org.p in (rdf_inf_ifp_list (:1)) and syn.p = org.p and org.s = :0 and isiri_id (:0) and syn.o = org.o and not rdf_inf_ifp_is_excluded (:1, org.P, org.O) "
+  " union all select rsyn.s from DB.DBA.RDF_QUAD rorg, DB.DBA.RDF_QUAD rsyn "
+  " where rorg.P in (rdf_inf_ifp_rel_list (:1)) and rsyn.P in (rdf_inf_ifp_rel_list (:1, rorg.P)) and rorg.S = :0 and isiri_id (:0) and rsyn.o = rorg.O and not rdf_inf_ifp_is_excluded (:1, rorg.P, rorg.O) "
+  " option (any order)";
 
 query_t * sas_1_qr;
 query_t * sas_2_qr;
@@ -344,6 +361,12 @@ query_t * tn_ifp_no_graph_qr;
 query_t * tn_ifp_dist_qr;
 query_t * tn_ifp_dist_no_graph_qr;
 
+id_hash_t * sas_tn_ht;
+id_hash_t * sas_tn_no_graph_ht;
+id_hash_t * tn_ifp_ht;
+id_hash_t * tn_ifp_no_graph_ht;
+dk_mutex_t * tn_cache_mtx;
+
 void
 sas_ensure ()
 {
@@ -358,9 +381,24 @@ sas_ensure ()
       tn_ifp_no_graph_qr = sql_compile (tn_ifp_no_graph_text, bootstrap_cli, &err, SQLC_DEFAULT);
       tn_ifp_dist_qr = sql_compile (tn_ifp_dist_text, bootstrap_cli, &err, SQLC_DEFAULT);
       tn_ifp_dist_no_graph_qr = sql_compile (tn_ifp_dist_no_graph_text, bootstrap_cli, &err, SQLC_DEFAULT);
+
+      sas_tn_ht = id_hash_allocate (31, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      sas_tn_no_graph_ht = id_hash_allocate (31, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      tn_ifp_ht = id_hash_allocate (31, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      tn_ifp_no_graph_ht = id_hash_allocate (31, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      tn_cache_mtx = mutex_allocate ();
     }
 }
 
+id_hash_t *
+tn_hash_table_get (trans_node_t * tn)
+{
+  if (sas_tn_qr == tn->tn_prepared_step) return sas_tn_ht;
+  if (sas_tn_no_graph_qr == tn->tn_prepared_step) return sas_tn_no_graph_ht;
+  if (tn_ifp_qr == tn->tn_prepared_step) return tn_ifp_ht;
+  if (tn_ifp_no_graph_qr == tn->tn_prepared_step) return tn_ifp_no_graph_ht;
+  return NULL;
+}
 
 caddr_t same_as_iri;
 caddr_t owl_sub_class_iri = NULL;
@@ -407,20 +445,22 @@ hash_queue_get (id_hash_t * ht, query_instance_t * qi, int next, int last)
 }
 
 
-void rdf_sas_ensure (caddr_t * qst);
+void rdf_sas_ensure (caddr_t * qst, caddr_t * err_ret);
 
 void
 ri_same_as_iri (rdf_inf_pre_node_t * ri, query_instance_t * qi, caddr_t iri, query_t * qr)
 {
   caddr_t * qst = (caddr_t *)qi;
   ptrlong one = 1;
-  caddr_t err;
+  caddr_t err = NULL;
   local_cursor_t * lc;
   id_hash_t * reached = (id_hash_t *) QST_GET (qi, ri->ri_sas_reached);
   id_hash_t * out = (id_hash_t *) QST_GET (qi, ri->ri_sas_out);
   id_hash_t * follow = (id_hash_t *) QST_GET (qi, ri->ri_sas_follow);
   int ginx;
-  rdf_sas_ensure (qst);
+  rdf_sas_ensure (qst, &err);
+  if (err)
+    sqlr_resignal (err);
   DO_BOX (state_slot_t *, g_ssl, ginx, ri->ri_sas_g)
     {
       caddr_t g = qst_get (qst, g_ssl);
@@ -718,32 +758,41 @@ rdf_inf_ctx (char * name)
 }
 
 
+rdf_inf_ctx_t *
+ric_allocate (caddr_t n2)
+    {
+  NEW_VARZ (rdf_inf_ctx_t, ctx);
+  ctx->ric_name = n2;
+  id_hash_set (rdf_name_to_ric, (caddr_t)&n2, (caddr_t)&ctx);
+      ctx->ric_iri_to_subclass = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      ctx->ric_iri_to_subproperty = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      ctx->ric_iid_to_rel_ifp = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      ctx->ric_samples = id_hash_allocate (61, sizeof (caddr_t), sizeof (text_count_t), treehash, treehashcmp);
+      /*ctx->ric_prop_props = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);*/
+      ctx->ric_ifp_exclude = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+      id_hash_set_rehash_pct (ctx->ric_iri_to_subclass, 200);
+      id_hash_set_rehash_pct (ctx->ric_iri_to_subproperty, 200);
+      id_hash_set_rehash_pct (ctx->ric_iid_to_rel_ifp, 200);
+      id_hash_set_rehash_pct (ctx->ric_samples, 200);
+      id_hash_set_rehash_pct (ctx->ric_ifp_exclude, 200);
+  ctx->ric_ifp_exclude = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);;
+      ctx->ric_mtx = mutex_allocate ();
+  ctx->ric_samples = id_hash_allocate (61, sizeof (caddr_t), sizeof (text_count_t), treehash, treehashcmp);
+  id_hash_set_rehash_pct (ctx->ric_samples, 200);
+      return ctx;
+    }
+
 rdf_inf_ctx_t *
 bif_ctx_arg (caddr_t * qst, state_slot_t ** args, int nth, char * name, int create)
 {
   caddr_t ctx_name = bif_string_arg (qst, args, nth, name);
-  rdf_inf_ctx_t ** place = (rdf_inf_ctx_t **) id_hash_get (rdf_name_to_ric, (caddr_t)&ctx_name), * ctx;
+  rdf_inf_ctx_t ** place = (rdf_inf_ctx_t **) id_hash_get (rdf_name_to_ric, (caddr_t)&ctx_name);
   if (!place && !create)
-    sqlr_new_error ("42000", "RDFI.", "No rdf inf ctx %.200s", ctx_name);
+    sqlr_new_error ("42000", "RDFI.", "No RDF inference rule set '%.200s' specified as argument #%d of %.200s()", ctx_name, nth, name);
   if (!place)
     {
       caddr_t n2 = box_copy (ctx_name);
-      NEW_VARZ (rdf_inf_ctx_t, c1);
-      c1->ric_name = n2;
-      id_hash_set (rdf_name_to_ric, (caddr_t)&n2, (caddr_t)&c1);
-      ctx = c1;
-      ctx->ric_iri_to_subclass = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);;
-      ctx->ric_iri_to_subproperty = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);;
-      ctx->ric_samples = id_hash_allocate (61, sizeof (caddr_t), sizeof (text_count_t), treehash, treehashcmp);
-      /*ctx->ric_prop_props = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);*/
-      ctx->ric_ifp_exclude = id_hash_allocate (61, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);;
-      id_hash_set_rehash_pct (ctx->ric_iri_to_subclass, 200);
-      id_hash_set_rehash_pct (ctx->ric_iri_to_subproperty, 200);
-      id_hash_set_rehash_pct (ctx->ric_samples, 200);
-      id_hash_set_rehash_pct (ctx->ric_ifp_exclude, 200);
-      /*id_hash_set_rehash_pct (ctx->ric_prop_props, 200);*/
-      ctx->ric_mtx = mutex_allocate ();
-      return ctx;
+      return ric_allocate (n2);
     }
   else
     return * place;
@@ -754,7 +803,7 @@ bif_rdf_super_sub_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t iri = bif_arg (qst, args, 1, "rdf_super_sub_list");
   int mode = bif_long_arg (qst, args, 2, "rdf_super_sub_list");
-  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "", 0);
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_super_sub_list", 0);
   rdf_sub_t * sub;
   if (mode < RI_SUBCLASS || mode > RI_SUPERPROPERTY)
     sqlr_new_error ("42000", "RDF..", "RDF inference type must be between 1 and 4");
@@ -780,11 +829,11 @@ bif_rdf_is_sub (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   ri_iterator_t * rit;
   caddr_t sub_iri = bif_arg (qst, args, 1, "rdf_is_sub");
   caddr_t iri = bif_arg (qst, args, 2, "rdf_is_sub");
-  int mode = bif_long_arg (qst, args, 3, "rdf_is_sub_list");
-  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "", 0);
+  int mode = bif_long_arg (qst, args, 3, "rdf_is_sub");
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_is_sub", 0);
   rdf_sub_t * sub;
   if (mode != RI_SUBCLASS && mode != RI_SUBPROPERTY)
-    sqlr_new_error ("42000", "RDF..", "RDF inference type for rdf_is_sub must be  1 for subclass  or 3 for subproperty");
+    sqlr_new_error ("42000", "RDF..", "RDF inference type for rdf_is_sub() must be 1 for subclass  or 3 for subproperty");
   mode = mode == RI_SUBCLASS ? RI_SUPERCLASS : RI_SUPERPROPERTY;
   sub = ric_iri_to_sub (ctx, sub_iri, mode, 0);
   if (sub)
@@ -832,27 +881,271 @@ bif_rdf_inf_dir (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return box_num (0);
 }
 
+caddr_t
+bif_rdf_inf_dump (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_dump", 0);
+  int ctr;
+  dk_set_t res_triples = NULL;
+  id_hash_iterator_t hiter;
+  caddr_t *key_ptr, *data_ptr;
+  rdf_sub_t **rsub_ptr;
+  iri_id_t **rels_ptr;
+  id_hash_iterator (&hiter, ctx->ric_iri_to_subclass);
+  while (hit_next (&hiter, (char **)(&key_ptr), (char **)(&rsub_ptr)))
+    {
+      rdf_sub_t *rsub = rsub_ptr[0];
+      DO_SET (rdf_sub_t *, sub_rs, &(rsub->rs_sub))
+        {
+          dk_set_push (&res_triples, list (3, box_copy_tree (sub_rs->rs_iri), box_dv_uname_string ("http://www.w3.org/2000/01/rdf-schema#subClassOf"), box_copy_tree(key_ptr[0])));
+        }
+      END_DO_SET ()
+      DO_SET (rdf_sub_t *, equiv_rs, &(rsub->rs_equiv))
+        {
+          dk_set_push (&res_triples, list (3, box_copy_tree (equiv_rs->rs_iri), box_dv_uname_string ("http://www.w3.org/2002/07/owl#equivalentClass"), box_copy_tree(key_ptr[0])));
+        }
+      END_DO_SET ()
+    }
+  id_hash_iterator (&hiter, ctx->ric_iri_to_subproperty);
+  while (hit_next (&hiter, (char **)(&key_ptr), (char **)(&rsub_ptr)))
+    {
+      rdf_sub_t *rsub = rsub_ptr[0];
+      DO_SET (rdf_sub_t *, sub_rs, &(rsub->rs_sub))
+        {
+          dk_set_push (&res_triples, list (3, box_copy_tree (sub_rs->rs_iri), box_dv_uname_string ("http://www.w3.org/2000/01/rdf-schema#subPropertyOf"), box_copy_tree(key_ptr[0])));
+        }
+      END_DO_SET ()
+      DO_SET (rdf_sub_t *, equiv_rs, &(rsub->rs_equiv))
+        {
+          dk_set_push (&res_triples, list (3, box_copy_tree (equiv_rs->rs_iri), box_dv_uname_string ("http://www.w3.org/2002/07/owl#equivalentProperty"), box_copy_tree(key_ptr[0])));
+        }
+      END_DO_SET ()
+    }
+  DO_BOX_FAST_REV (caddr_t, iid, ctr, ctx->ric_ifp_list)
+    {
+      dk_set_push (&res_triples, list (3, box_copy_tree(iid), uname_rdf_ns_uri_type, box_dv_uname_string ("http://www.w3.org/2002/07/owl#InverseFunctionalProperty"), box_dv_uname_string ("http://www.w3.org/2002/07/owl#InverseFunctionalProperty")));
+    }
+  END_DO_BOX_FAST;
+  data_ptr = ctx->ric_inverse_prop_pair_sortedalist;
+  for (ctr = BOX_ELEMENTS_0 (data_ptr) - 2; ctr >= 0; ctr -= 2)
+    {
+      dk_set_push (&res_triples, list (3, box_copy_tree (data_ptr[ctr]), box_dv_uname_string ("http://www.w3.org/2002/07/owl#inverseOf"), box_copy_tree (data_ptr[ctr+1])));
+    }
+  data_ptr = ctx->ric_prop_props;
+  for (ctr = BOX_ELEMENTS_0 (data_ptr) - 2; ctr >= 0; ctr -= 2)
+    {
+      if (0x1 & (ptrlong)(data_ptr[ctr+1]))
+        dk_set_push (&res_triples, list (3, box_copy_tree (data_ptr[ctr]), uname_rdf_ns_uri_type, box_dv_uname_string ("http://www.w3.org/2002/07/owl#InverseFunctionalProperty"), box_dv_uname_string ("http://www.w3.org/2002/07/owl#TransitiveProperty")));
+    }
+  id_hash_iterator (&hiter, ctx->ric_ifp_exclude);
+  while (hit_next (&hiter, (char **)(&key_ptr), (char **)(&data_ptr)))
+    {
+      caddr_t *data = (caddr_t *)data_ptr;
+      DO_BOX_FAST_REV (caddr_t, iid, ctr, data)
+        {
+          dk_set_push (&res_triples, list (3, box_copy_tree(key_ptr[0]), box_dv_uname_string ("http://www.openlinksw.com/schemas/virtrdf#nullIFPValue"), box_copy_tree (iid)));
+        }
+      END_DO_BOX_FAST;
+    }
+#ifndef NDEBUG
+  id_hash_iterator (&hiter, ctx->ric_iid_to_rel_ifp);
+  while (hit_next (&hiter, (char **)(&key_ptr), (char **)(&rels_ptr)))
+    {
+      iri_id_t *rels = rels_ptr[0];
+      int rel_ctr, rels_count = box_length (rels) / sizeof (iri_id_t);
+      for (rel_ctr = rels_count; rel_ctr--; /*no step*/)
+        dk_set_push (&res_triples, list (3, box_copy_tree(key_ptr[0]), box_dv_uname_string ("http://www.openlinksw.com/schemas/virtrdf#relatedIFP"), box_iri_id (rels[rel_ctr])));
+    }
+#endif
+  return list_to_array (res_triples);
+}
 
 caddr_t
 bif_rdf_inf_ifp_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   rdf_inf_ctx_t * ctx;
   ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_ifp_list", 0);
-  if (ctx->ric_ifp_list)
-    return box_copy_tree (ctx->ric_ifp_list);
+  if (NULL == ctx->ric_ifp_list)
   return list (0);
+  return box_copy_tree (ctx->ric_ifp_list);
 }
 
+caddr_t
+bif_rdf_inf_ifp_rel_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  rdf_inf_ctx_t * ctx;
+  ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_ifp_rel_list", 0);
+  if (NULL == ctx->ric_ifp_rel_list)
+    return NULL;
+  if (1 == BOX_ELEMENTS (args))
+    return box_copy_tree (ctx->ric_ifp_rel_list);
+  else
+    {
+      caddr_t arg = bif_arg (qst, args, 1, "rdf_inf_ifp_rel_list");
+      iri_id_t arg_iid, **rels_ptr, *rels;
+      caddr_t *res_boxes;
+      int rel_ctr, rels_count, res_boxes_ctr;
+      if (DV_IRI_ID != DV_TYPE_OF (arg))
+        return NEW_DB_NULL;
+      rels_ptr = (iri_id_t **)id_hash_get (ctx->ric_iid_to_rel_ifp, (caddr_t)(&arg));
+      if (NULL == rels_ptr)
+        return NEW_DB_NULL;
+      rels = rels_ptr[0];
+      rels_count = box_length (rels) / sizeof (iri_id_t);
+      arg_iid = unbox_iri_int64 (arg);
+      res_boxes = dk_alloc_box ((rels_count-1) * sizeof (caddr_t), DV_ARRAY_OF_POINTER);
+      for (rel_ctr = res_boxes_ctr = 0; rel_ctr < rels_count; rel_ctr++)
+        {
+          if (rels[rel_ctr] != arg_iid)
+            res_boxes [res_boxes_ctr++] = box_iri_id (rels[rel_ctr]);
+        }
+      if (res_boxes_ctr != (rels_count-1))
+        sqlr_new_error ("22023", "SR634", "Corrupted inference rule set '%.200s', please report the ontology and the query", ctx->ric_name);
+      return (caddr_t)res_boxes;
+    }
+}
 
 caddr_t
 bif_rdf_inf_set_ifp_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_ifp_list", 1);
+#ifdef RDFINF_DEBUG
+  query_instance_t *qi = (query_instance_t *)qst;
+#endif
+  int ifp_count, ifp_ctr;
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_ifp_list", 1);
   caddr_t * arr = bif_array_of_pointer_arg (qst, args, 1, "rdf_inf_set_ifp_list");
+  int dirt;
+  caddr_t * grps;
+  dk_set_t ifp_rel_acc = NULL;
+
+  sec_check_dba ((query_instance_t *)qst, "rdf_inf_set_ifp_list");
+  grps = box_copy /*_tree*/ (arr);
   ctx->ric_ifp_list = box_copy_tree ((caddr_t)arr);
+  ifp_count = BOX_ELEMENTS_0 (arr);
+  if (ctx->ric_iid_to_rel_ifp->ht_inserts != ctx->ric_iid_to_rel_ifp->ht_deletes)
+    {
+      caddr_t *keyp, *valp;
+      id_hash_iterator_t hit;
+      id_hash_iterator (&hit, ctx->ric_iid_to_rel_ifp);
+      while (hit_next (&hit, (char **)&keyp, (char **)&valp))
+        {
+           dk_free_tree (keyp[0]);
+           /*dk_free_tree (valp[0]);*/
+        }
+      id_hash_clear (ctx->ric_iid_to_rel_ifp);
+    }
+  do {
+      dirt = 0;
+      for (ifp_ctr = ifp_count; ifp_ctr--; /*no step*/)
+        {
+          rdf_sub_t **rsub_ptr;
+          caddr_t ifp_boxed_iid = arr[ifp_ctr];
+          iri_id_t ifp_iid = unbox_iri_int64 (ifp_boxed_iid);
+          iri_id_t grp_iid = unbox_iri_int64 (grps[ifp_ctr]);
+          dk_set_t subs;
+          rsub_ptr = (rdf_sub_t **)id_hash_get (ctx->ric_iri_to_subproperty, (char *)(&ifp_boxed_iid));
+          if (NULL == rsub_ptr)
+            continue;
+          subs = rsub_ptr[0]->rs_sub;
+          DO_SET (rdf_sub_t *, sub_rsub, &subs)
+            {
+              int sub_ifp_pos;
+              iri_id_t sub_iid, sub_grp_iid;
+              sub_iid = unbox_iri_int64 (sub_rsub->rs_iri);
+              for (sub_ifp_pos = ifp_count; sub_ifp_pos--; /*no step*/)
+                {
+                  if (unbox_iri_int64 (arr[sub_ifp_pos]) == sub_iid)
+                    break;
+                }
+              if (0 > sub_ifp_pos)
+                sqlr_new_error ("22023", "SR633", "Inconsistent inference rule set '%.200s' in rdf_inf_set_ifp_list(), loading is aborted, please report the ontology", ctx->ric_name);
+              sub_grp_iid = unbox_iri_int64 (grps[sub_ifp_pos]);
+              if (sub_grp_iid != grp_iid)
+                {
+                  int ctr;
+                  dirt = 1;
+                  for (ctr = ifp_count; ctr--; /*no step*/)
+                    {
+                      if (unbox_iri_int64 (grps[ctr]) == sub_grp_iid)
+                        {
+#ifdef RDFINF_DEBUG
+                          query_instance_t *qi = (query_instance_t *)qst;
+                          caddr_t main_iri = key_id_to_iri (qi, ifp_iid);
+                          caddr_t sub_iri = key_id_to_iri (qi, ifp_iid);
+                          caddr_t attach_iri = key_id_to_iri (qi, unbox_iri_int64 (arr[ctr]));
+                          printf ("INF '%s' loading: IFP <%s> is attached to <%s> via <%s>\n", ctx->ric_name, attach_iri, main_iri, sub_iri);
+                          dk_free_box (main_iri);
+                          dk_free_box (sub_iri);
+                          dk_free_box (attach_iri);
+
+#endif
+                          grps[ctr] = grps[ifp_ctr];
+                        }
+                    }
+                }
+            }
+          END_DO_SET()
+        }
+    } while (dirt);
+#ifdef RDFINF_DEBUG
+  for (ifp_ctr = ifp_count; ifp_ctr--; /*no step*/)
+    {
+      caddr_t ifp_iri = key_id_to_iri (qi, unbox_iri_int64 (arr[ifp_ctr]));
+      caddr_t grp_iri = key_id_to_iri (qi, unbox_iri_int64 (grps[ifp_ctr]));
+      printf ("INF '%s' loading: IFP <%s> belongs to <%s> group\n", ctx->ric_name, ifp_iri, grp_iri);
+    }
+#endif
+
+/* At this point, every group in grps[ctr] is labeled by exactly one its representative. Now it's easy to fill in the hashtable */
+  for (ifp_ctr = ifp_count; ifp_ctr--; /*no step*/)
+    {
+      caddr_t ifp_boxed_iid = arr[ifp_ctr];
+      dk_set_t acc;
+      int grpsize, ctr;
+      iri_id_t *val;
+      iri_id_t ifp_iid = unbox_iri_int64 (ifp_boxed_iid);
+      iri_id_t grp_iid = unbox_iri_int64 (grps[ifp_ctr]);
+      if (ifp_iid != grp_iid)
+        continue; /* This is not a canonical rep of the group */
+      acc = NULL;
+      grpsize = 0;
+      for (ctr = ifp_count; ctr--; /*no step*/)
+        if (unbox_iri_int64 (grps[ctr]) == grp_iid)
+          {
+            dk_set_push (&acc, arr[ctr]);
+            grpsize++;
+          }
+      if (1 == grpsize)
+        {
+          dk_set_pop (&acc);
+          continue;
+        }
+      val = dk_alloc_box (grpsize * sizeof (iri_id_t), DV_ARRAY_OF_LONG);
+      for (ctr = 0; ctr < grpsize; ctr++)
+        {
+          caddr_t member_boxed_iid = box_copy_tree (dk_set_pop (&acc));
+          dk_set_push (&ifp_rel_acc, member_boxed_iid);
+          val[ctr] = unbox_iri_int64 (member_boxed_iid);
+          id_hash_set (ctx->ric_iid_to_rel_ifp, (caddr_t)(&member_boxed_iid), (caddr_t)(&val));
+        }
+    }
+  ctx->ric_ifp_rel_list = (caddr_t *) revlist_to_array (ifp_rel_acc);
   return NULL;
 }
 
+caddr_t
+bif_rdf_inf_ifp_exclude_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_ifp_exclude_list", 0);
+  iri_id_t ifp = bif_iri_id_arg (qst, args, 1, "rdf_inf_ifp_exclude_list");
+  caddr_t box = box_iri_id (ifp);
+  caddr_t *place = (caddr_t *)id_hash_get (ctx->ric_ifp_exclude, (void *)(&box));
+  dk_free_tree (box);
+  if (NULL == place)
+    return list (0);
+  return box_copy_tree (place[0]);
+}
+
 
 caddr_t
 bif_rdf_inf_set_ifp_exclude_list (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -860,8 +1153,10 @@ bif_rdf_inf_set_ifp_exclude_list (caddr_t * qst, caddr_t * err_ret, state_slot_t
   rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_ifp_exclude_list", 1);
   iri_id_t ifp = bif_iri_id_arg (qst, args, 1, "rdf_inf_set_ifp_exclude_list");
   caddr_t * arr = bif_array_of_pointer_arg (qst, args, 2, "rdf_inf_set_ifp_exclude_list");
-  caddr_t box = box_iri_id (ifp);
-  caddr_t copy = box_copy_tree (arr);
+  caddr_t box, copy;
+  sec_check_dba ((query_instance_t *)qst, "rdf_inf_set_ifp_exclude_list");
+  box = box_iri_id (ifp);
+  copy = box_copy_tree (arr);
   id_hash_set (ctx->ric_ifp_exclude, (caddr_t)&box, (caddr_t)&copy);
   return NULL;
 }
@@ -893,9 +1188,15 @@ caddr_t
 bif_rdf_inf_set_inverses (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t *lst = bif_array_of_pointer_arg (qst, args, 1, "rdf_inf_set_inverses");
-  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_inverses", 0);
-  dk_free_tree (ctx->ric_inverse_prop_pair_sortedalist);
-  ctx->ric_inverse_prop_pair_sortedalist = box_copy_tree (lst);
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_inverses", 1);
+  int ctr, len = BOX_ELEMENTS_0 (lst);
+  caddr_t *uname_lst;
+  sec_check_dba ((query_instance_t *)qst, "rdf_inf_set_inverses");
+  uname_lst = dk_alloc_list (len);
+  for (ctr = len; ctr--; /* no step */)
+    uname_lst[ctr] = box_dv_uname_string (lst[ctr]);
+  dk_free_tree ((caddr_t)(ctx->ric_inverse_prop_pair_sortedalist));
+  ctx->ric_inverse_prop_pair_sortedalist = uname_lst;
   return NULL;
 }
 
@@ -903,9 +1204,18 @@ caddr_t
 bif_rdf_inf_set_prop_props (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t *lst = bif_array_of_pointer_arg (qst, args, 1, "rdf_inf_set_prop_props");
-  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_prop_props", 0);
-  dk_free_tree (ctx->ric_prop_props);
-  ctx->ric_prop_props = box_copy_tree (lst);
+  rdf_inf_ctx_t * ctx = bif_ctx_arg (qst, args, 0, "rdf_inf_set_prop_props", 1);
+  int ctr, len = BOX_ELEMENTS_0 (lst);
+  caddr_t *uname_flags_lst;
+  sec_check_dba ((query_instance_t *)qst, "rdf_inf_set_prop_props");
+  uname_flags_lst = dk_alloc_list (len);
+  for (ctr = len - 2; ctr >= 0; ctr -= 2)
+    {
+      uname_flags_lst[ctr] = box_dv_uname_string (lst[ctr]);
+      uname_flags_lst[ctr+1] = (caddr_t)((ptrlong)(unbox (lst[ctr+1])));
+    }
+  dk_free_tree ((caddr_t)(ctx->ric_prop_props));
+  ctx->ric_prop_props = uname_flags_lst;
   return NULL;
 }
 
@@ -913,10 +1223,18 @@ caddr_t
 bif_rdf_inf_clear (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   caddr_t ctx_name = bif_string_arg (qst, args, 0, "rdf_inf_clear");
+  sec_check_dba ((query_instance_t *)qst, "rdf_inf_clear");
   id_hash_remove (rdf_name_to_ric, (caddr_t)&ctx_name);
   return 0;
 }
 
+caddr_t
+bif_rdf_inf_is_loaded (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t ctx_name = bif_string_arg (qst, args, 0, "rdf_inf_is_loaded");
+  return box_num ((NULL != id_hash_get (rdf_name_to_ric, (caddr_t)&ctx_name)) ? 1 : 0);
+}
+
 
 caddr_t
 bif_rdf_inf_const_init (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -973,7 +1291,7 @@ bif_rdf_owl_iri (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 
 
 void
-rdf_sas_ensure (caddr_t * qst)
+rdf_sas_ensure (caddr_t * qst, caddr_t * err_ret)
 {
 }
 
@@ -981,7 +1299,7 @@ rdf_sas_ensure (caddr_t * qst)
 caddr_t
 bif_rdf_sas_iri (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  rdf_sas_ensure (qst);
+  rdf_sas_ensure (qst, err_ret);
   return box_copy_tree (same_as_iri);
 }
 
@@ -1045,7 +1363,8 @@ char * cl_rdf_init_srv =
 "   DB.DBA.RDF_MAKE_IID_OF_QNAME_SAFE (null); \n"
 "   DB.DBA.RDF_QNAME_OF_IID (null); \n"
 "   rdf_inf_const_init (); \n"
-"   select  count (*) into c from sys_rdf_schema where 0 = rdfs_load_schema (rs_name, rs_uri); \n"
+"   select count (*) into c from (select distinct s.RS_NAME from sys_rdf_schema s) sub where 0 = rdfs_load_schema (sub.RS_NAME); \n"
+"   DB.DBA.RDF_QUAD_LOAD_CACHE (); \n"
 "   JSO_LOAD_AND_PIN_SYS_GRAPH_RO  (); \n"
 "   commit work; \n"
 "   rdf_init_thread (0); \n"
@@ -1077,6 +1396,7 @@ cl_rdf_inf_init (client_connection_t * cli, caddr_t * err_ret)
   /* called from compiler when an inf ctx is found in the parse tree. Must complete before the compilation can proceed.  For cluster only */
   int lt_threads = cli->cli_trx->lt_threads;
   static query_t * qr;
+  caddr_t err = NULL;
   IN_TXN;
   if (1 == cl_rdf_inf_inited)
     {
@@ -1092,7 +1412,7 @@ cl_rdf_inf_init (client_connection_t * cli, caddr_t * err_ret)
       semaphore_enter (self->thr_sem);
       vdb_leave_lt (cli->cli_trx, err_ret);
       if (1 != cl_rdf_inf_inited)
-	*err_ret = srv_make_new_error ("42000", "CL...", "Rdf inf init failed while waiting for other thread to do the rdf inf init");
+	*err_ret = srv_make_new_error ("42000", "CLRI1", "Rdf inf init failed while waiting for other thread to do the rdf inf init");
       return;
     }
   cl_rdf_inf_init_user = cli->cli_user;
@@ -1111,7 +1431,7 @@ cl_rdf_inf_init (client_connection_t * cli, caddr_t * err_ret)
       rc = lt_enter (cli->cli_trx);
       if (LTE_OK != rc)
 	{
-	  *err_ret = srv_make_new_error ("42000", "CL...", "Cluster rrdf inf init failed because bad transaction state.  Rollback and retry.");
+	  *err_ret = srv_make_new_error ("42000", "CLRI2", "Cluster rdf inf init failed because bad transaction state.  Rollback and retry.");
 	  goto init_error;
 	}
     }
@@ -1128,6 +1448,8 @@ cl_rdf_inf_init (client_connection_t * cli, caddr_t * err_ret)
   cl_rdf_inf_init_done (cli, 1);
   return;
  init_error:
+  err = *err_ret;
+  log_error ("Error executing a cluster RDF inf init: %s: %s", ((caddr_t *) err)[QC_ERRNO], ((caddr_t *) err)[QC_ERROR_STRING]);
   cl_rdf_inf_init_done (cli, 0);
 }
 
@@ -1142,11 +1464,31 @@ cl_rdf_inf_init_1 (caddr_t * qst)
     sqlr_resignal (err);
 }
 
+#define TN_HASH_CLEANUP(ht) \
+      DO_IDHASH (caddr_t, k, caddr_t, d, ht) \
+	{ \
+	  dk_free_tree (k); \
+	  dk_free_tree (d); \
+	} \
+      END_DO_IDHASH; \
+      id_hash_clear (ht)
+
+caddr_t
+bif_tn_cache_clear (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  mutex_enter (tn_cache_mtx);
+  TN_HASH_CLEANUP (sas_tn_ht);
+  TN_HASH_CLEANUP (sas_tn_no_graph_ht);
+  TN_HASH_CLEANUP (tn_ifp_ht);
+  TN_HASH_CLEANUP (tn_ifp_no_graph_ht);
+  mutex_leave (tn_cache_mtx);
+  return NULL;
+}
 
 void
 cl_rdf_bif_check_init (bif_t bif)
 {
-  if (cl_rdf_inf_inited)
+  if (1 == cl_rdf_inf_inited)
     return;
   if  (bif_rdf_inf_ifp_list == bif || bif_rdf_inf_ifp_is_excluded == bif
        || bif_rdf_is_sub == bif || bif_rdf_super_sub_list == bif)
@@ -1162,7 +1504,7 @@ cl_rdf_bif_check_init (bif_t bif)
 	}
     }
 }
-
+rdf_inf_ctx_t * empty_ric;
 
 void
 rdf_inf_init ()
@@ -1173,11 +1515,13 @@ rdf_inf_init ()
   bif_define ("rdf_inf_dir", bif_rdf_inf_dir);
   bif_define ("rdf_inf_const_init", bif_rdf_inf_const_init);
   bif_define ("rdf_inf_clear", bif_rdf_inf_clear);
+  bif_define ("rdf_inf_is_loaded", bif_rdf_inf_is_loaded);
   bif_define ("rdf_sas_iri", bif_rdf_sas_iri);
   bif_set_uses_index (bif_rdf_sas_iri);
   bif_define ("rdf_owl_iri", bif_rdf_owl_iri);
   bif_set_uses_index (bif_rdf_owl_iri);
   bif_define ("rdf_inf_ifp_list", bif_rdf_inf_ifp_list);
+  bif_define ("rdf_inf_ifp_rel_list", bif_rdf_inf_ifp_rel_list);
   bif_define ("rdf_inf_set_ifp_list", bif_rdf_inf_set_ifp_list);
   bif_define ("rdf_inf_set_ifp_exclude_list", bif_rdf_inf_set_ifp_exclude_list);
   bif_define ("rdf_inf_ifp_is_excluded", bif_rdf_inf_ifp_is_excluded);
@@ -1187,7 +1531,10 @@ rdf_inf_init ()
   bif_set_uses_index (bif_rdf_check_init);
   bif_define ("rdf_super_sub_list", bif_rdf_super_sub_list);
   bif_define ("rdf_is_sub", bif_rdf_is_sub);
+  bif_define ("rdf_inf_dump", bif_rdf_inf_dump);
+  bif_define ("tn_cache_clear", bif_tn_cache_clear);
   dk_mem_hooks (DV_RI_ITERATOR, box_non_copiable, rit_free, 0);
+  empty_ric = ric_allocate (box_dv_short_string ("__ empty"));
   sas_init ();
 }
 
@@ -1601,6 +1948,40 @@ sqlg_rdf_inf_same_as_opt (df_elt_t * tb_dfe)
   return NULL;
 }
 
+static void
+sqlg_ts_copies_search_pars (table_source_t * ts)
+{
+  if (ts->ts_inx_op)
+    {
+      inx_op_t *iop = ts->ts_inx_op;
+      int inx;
+      DO_BOX (inx_op_t *, term, inx, iop->iop_terms)
+        {
+          term->iop_ks->ks_copy_search_pars = 1;
+        }
+      END_DO_BOX
+    }
+  if (ts->ts_order_ks)
+    ts->ts_order_ks->ks_copy_search_pars = 1;
+  if (ts->ts_main_ks)
+    ts->ts_main_ks->ks_copy_search_pars = 1;
+}
+
+#define TS_COPIES_SEARCH_PARS do { \
+    sqlg_ts_copies_search_pars ((table_source_t *)(ts)); \
+    if (NULL != q_head[0]) \
+      { \
+        data_source_t *h = q_head[0]; \
+        if (IS_TS (h)) \
+          sqlg_ts_copies_search_pars ((table_source_t *)h); \
+        while (h->src_continuations) \
+          { \
+            h = (data_source_t *) h->src_continuations->data; \
+            if (h && IS_TS (h)) \
+              sqlg_ts_copies_search_pars ((table_source_t *)h); \
+          } \
+      } \
+  } while (0);
 
 #define LEADING_SUBCLASS \
   sqlg_leading_subclass_inf (tb_dfe->dfe_sqlo, q_head, ts, p_dfe, const_p, o_dfe, const_o, ctx, tb_dfe, inxop_inx, sas_o)
@@ -1713,7 +2094,7 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
   caddr_t ctx_name = sqlo_opt_value (tb_dfe->_.table.ot->ot_opts, OPT_RDF_INFERENCE);
   rdf_inf_ctx_t * ctx, **place, *sas_ctx;
   rdf_inf_pre_node_t * sas_s = NULL, * sas_o = NULL, * sas_p = NULL;
-  caddr_t const_s = NULL, const_p = NULL, const_o = NULL;
+  caddr_t /*const_s = NULL,*/ const_p = NULL, const_o = NULL;
   df_elt_t * g_dfe = NULL, * s_dfe = NULL, * p_dfe = NULL, * o_dfe = NULL;
   if (!IS_TS (((table_source_t*)ts))
       && (qn_input_fn)hash_source_input != ts->src_input)
@@ -1756,13 +2137,14 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
 	}
     }
   END_DO_SET();
-  const_s = dfe_iri_const (s_dfe);
+  /*const_s = dfe_iri_const (s_dfe);*/
   const_p = dfe_iri_const (p_dfe);
   const_o = dfe_iri_const (o_dfe);
   memset (&ris, 0, sizeof (ris));
   sc->sc_rdf_inf_slots = &ris;
   if (!s_dfe && !p_dfe && !o_dfe)
     {
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBCLASS;
       TRAILING_SUBP;
     }
@@ -1771,12 +2153,14 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
       LEADING_IFP_O;
       LEADING_SAME_AS_O;
       LEADING_SUBCLASS;
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBP;
     }
   else if (!s_dfe && p_dfe && !o_dfe)
     {
       LEADING_SAME_AS_P;
       LEADING_SUBP;
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBCLASS;
     }
   else if (!s_dfe && p_dfe && o_dfe)
@@ -1791,6 +2175,7 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
     {
       LEADING_IFP_S;
       LEADING_SAME_AS_S;
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBCLASS;
       TRAILING_SUBP;
     }
@@ -1801,6 +2186,7 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
       LEADING_SAME_AS_O;
       LEADING_SAME_AS_S;
       LEADING_SUBCLASS;
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBP;
     }
   else if (s_dfe && p_dfe && !o_dfe)
@@ -1809,6 +2195,7 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
       LEADING_SAME_AS_S;
       LEADING_SAME_AS_P;
       LEADING_SUBP;
+      TS_COPIES_SEARCH_PARS;
       TRAILING_SUBCLASS;
     }
   else if (s_dfe && p_dfe && o_dfe)
@@ -1823,6 +2210,8 @@ sqlg_rdf_inf_1 (df_elt_t * tb_dfe, data_source_t * ts, data_source_t ** q_head,
     }
   else
     GPF_T1 (" all possibilities of spo already covered");
+  dk_free_box (const_p);
+  dk_free_box (const_o);
   sc->sc_rdf_inf_slots = NULL;
 }
 
@@ -1895,7 +2284,9 @@ sqlg_cl_bracket_outer (sqlo_t * so, data_source_t * first)
   sql_comp_t * sc = so->so_sc;
   outer_seq_end_node_t * ose;
   dk_hash_t * res = hash_table_allocate (11);
-  data_source_t * first1 = qn_next (first);
+  data_source_t * first1 = first ? qn_next (first) : NULL;
+  data_source_t * org_first = first1;
+  void * dp;
   SET_THR_ATTR (THREAD_CURRENT_THREAD, TA_SQLC_ASG_SET, res);
   sc->sc_any_clb = 1;
   while (first1)
@@ -1928,6 +2319,11 @@ sqlg_cl_bracket_outer (sqlo_t * so, data_source_t * first)
     ose->ose_sctr = sctr;
     for (qn = qn_next ((data_source_t*)sctr); qn != (data_source_t*)ose && qn; qn = qn_next (qn))
       dk_set_push (&sctr->sctr_continuable, (void*)qn);
+    if (sc->sc_qn_to_dpipe && (dp = (data_source_t*)gethash ((void*)org_first, sc->sc_qn_to_dpipe)))
+      {
+	remhash ((void*)org_first, sc->sc_qn_to_dpipe);
+	sethash ((void*)sctr, sc->sc_qn_to_dpipe, (void*)dp);
+      }
   }
   return ose;
 }
