@@ -1,14 +1,14 @@
 /*
  *  regist.c
  *
- *  $Id: regist.c,v 1.13.2.5 2010/02/02 14:16:14 source Exp $
+ *  $Id$
  *
  *  Database Registry
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -549,7 +549,7 @@ log_registry_set (lock_trx_t * lt, char * k, const char * d)
   int llen = 0;
   char temp[2000];
 
-  if (!lt || lt->lt_replicate == REPL_NO_LOG || !d || !k || in_log_replay)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || !d || !k || in_log_replay || cl_non_logged_write_mode)
     return;
 
   ASSERT_IN_TXN;
