@@ -1,10 +1,10 @@
 /*
- *  $Id: sparql_sff.c,v 1.19.2.4 2011/02/04 15:36:04 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2009 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -35,10 +35,6 @@ extern "C" {
 #endif
 
 
-#define isdatechar(c) (('\0' != (c)) && (NULL != strchr ("0123456789 GMTZ:-", (c))))
-/* This was: #define isplainURIchar(c) (('\0' != (c)) && ('/' != (c)) && ('?' != (c)) && ('=' != (c)) && ('#' != (c))) */
-#define isplainURIchar(c) ('\0' == DKS_ESC_CHARCLASS_ACTION((unsigned)(c), DKS_ESC_URI))
-
 int
 sprintff_is_proven_bijection (const char *f)
 {
@@ -69,7 +65,7 @@ sprintff_is_proven_bijection (const char *f)
           continue;
         }
       tail++;
-      if (NULL == strchr ("DUdusc", tail[0]))
+      if (NULL == strchr ("DUdgusc", tail[0]))
         return 0; /* Unknown format so the bijection is not proven */
       fmt_type = tail[0];
       tail++;
@@ -95,6 +91,10 @@ sprintff_is_proven_bijection (const char *f)
             if (isplainURIchar (next) || (next & ~ 0x7F))
               return 0;
             continue;
+          case 'g':
+            if (isfloatchar (next))
+              return 0;
+            continue;
         }
     }
   return 1;
@@ -136,7 +136,7 @@ sprintff_is_proven_unparseable (const char *f)
           continue;
         }
       tail++;
-      if (NULL == strchr ("DUdusc", tail[0]))
+      if (NULL == strchr ("DUduscg", tail[0]))
         return 1; /* Unknown format so the format is unparseable OR the filed has modifiers for length and the like */
       fmt_type = tail[0];
       tail++;
@@ -148,11 +148,13 @@ sprintff_is_proven_unparseable (const char *f)
       if ('s' == left_fmt_type)
         return 1; /* %s at left can not be a bijection */
       if (('U' == left_fmt_type) && ('s' != fmt_type))
-        return 1; /* %U can eat anything except unescaped chars like '/' that might be pronted by '%s' */
+        return 1; /* %U can eat anything except unescaped chars like '/' that might be printed by '%s' */
       if ('u' == fmt_type)
         return 1; /* Any character %u can eat is good for any format at left */
       if ('D' == fmt_type)
         return 1; /* Any character %D can eat as first character is good for any format at left */
+      if ('g' == fmt_type)
+        return 1; /* Any character %D can eat as first character is good for any format at left */
    }
   return 0;
 }
@@ -499,7 +501,7 @@ again:
     }
   switch (f1_v)
     {
-    case 'D': case 'U': case 'd': case 's': case 'u': break;
+    case 'D': case 'U': case 'd': case 'g': case 's': case 'u': break;
     default: goto generic_tails; /* see below */
     }
 /* Now we slightly normalize the rest of processing to write code only for a half of f1_v and f2_v combinations */
@@ -521,6 +523,7 @@ again:
     case 'D': goto f1_v_is_D; /* see below */
     case 'U': goto f1_v_is_U; /* see below */
     case 'd': goto f1_v_is_d; /* see below */
+    case 'g': goto f1_v_is_g; /* see below */
     case 's': goto f1_v_is_s; /* see below */
     case 'u': goto f1_v_is_u; /* see below */
     default: GPF_T;
@@ -556,7 +559,7 @@ f1_v_is_D:
           goto res_tail_gets_f1_v; /* see below */
         }
       goto res_tail_gets_f1_v; /* see below */
-    case 'd': case 'u':
+    case 'd': case 'g': case 'u':
       goto res_tail_gets_f2_v; /* see below */
     case 's': goto generic_tails; /* see below */
     goto res_tail_gets_f2_v; /* see below */
@@ -577,13 +580,14 @@ f1_v_is_U:
           goto tails_are_in_sync; /* see below */
         }
       if ('\0' == f1_tail[0])
-        return SFF_ISECT_DIFF_END; /* One string ends with %D, other with non-%D fixed char */
+        return SFF_ISECT_DIFF_END; /* One string ends with %U, other with non-%U fixed char */
       return SFF_ISECT_DISJOIN;
     }
   switch (f2_v)
     {
     case 'U': goto res_tail_gets_f2_v; /* see below */
     case 'd': goto res_tail_gets_f2_v; /* see below */
+    case 'g': goto res_tail_gets_f2_v; /* see below */
     case 's': goto generic_tails; /* see below */
     case 'u': goto res_tail_gets_f2_v; /* see below */
     default: goto generic_tails; /* see below */
@@ -607,12 +611,42 @@ f1_v_is_u:
           goto tails_are_in_sync; /* see below */
         }
       if ('\0' == f1_tail[0])
-        return SFF_ISECT_DIFF_END; /* One string ends with %D, other with non-%D fixed char */
+        return SFF_ISECT_DIFF_END; /* One string ends with %d, other with non-%d fixed char */
       return SFF_ISECT_DISJOIN;
     }
   switch (f2_v)
     {
     case 'd': goto res_tail_gets_f2_v; /* see below */
+    case 'g': goto res_tail_gets_f1_v; /* see below */
+    case 's': goto generic_tails; /* see below */
+    case 'u': goto res_tail_gets_f2_v; /* see below */
+    default: goto generic_tails; /* see below */
+    }
+
+f1_v_is_g:
+  if (isfloatchar (f1_tail[0]))
+    goto generic_tails; /* see below */
+  /* The unambiguous '%g' in f1 may match any %g-like chars, f2 vars (%u to %u) */
+  if ('\0' == f2_v)
+    {
+      if (isdigit (f2_fix) || ('+' == f2_fix) || ('-' == f2_fix))
+        goto res_tail_gets_f2_fix; /* see below */
+      if ((f1_last_replaced != f1) && (('-' == f2_fix) || ('+' == f2_fix) || ('.' == f2_fix)))
+        goto res_tail_gets_f2_fix; /* see below */
+      if ((f1_tail[0] == f2_fix) && /* unambiguous synchronisation between f1 and f2... */
+        (f1_last_replaced == f1) ) /*... but integer output can not be empty so should make if result contains something for the field */
+        {
+          f2_tail = f2;
+          goto tails_are_in_sync; /* see below */
+        }
+      if ('\0' == f1_tail[0])
+        return SFF_ISECT_DIFF_END; /* One string ends with %g, other with non-%g fixed char */
+      return SFF_ISECT_DISJOIN;
+    }
+  switch (f2_v)
+    {
+    case 'd': goto res_tail_gets_f2_v; /* see below */
+    case 'g': goto res_tail_gets_f2_v; /* see below */
     case 's': goto generic_tails; /* see below */
     case 'u': goto res_tail_gets_f2_v; /* see below */
     default: goto generic_tails; /* see below */
@@ -849,6 +883,7 @@ again:
     case 'D': goto f2_v_is_D; /* see below */
     case 'U': goto f2_v_is_U; /* see below */
     case 'd': goto f2_v_is_d; /* see below */
+    case 'g': goto f2_v_is_g; /* see below */
     case 's': goto f2_v_is_s; /* see below */
     case 'u': goto f2_v_is_u; /* see below */
     default: goto generic_tails; /* see below */
@@ -928,7 +963,7 @@ f2_v_is_u:
           s1_tail += 2;
           s1_shifted = 1; continue;
         }
-      else if (f2_tail[0] == s1_tail[0]) /* unambiguous synchronisation between f2 and s1 */
+      else if (s1_shifted && (f2_tail[0] == s1_tail[0])) /* unambiguous synchronisation between f2 and s1 */
         goto tails_are_in_sync; /* see below */
       else if ('\0' == f2_tail[0])
         return SFF_ISECT_DIFF_END; /* One string ends with %d, other with non-%d fixed char */
@@ -937,6 +972,44 @@ f2_v_is_u:
     }
   GPF_T; /* never reached */
 
+f2_v_is_g:
+  if (isfloatchar (f2_tail[0]) || ('%' == f2_tail[0]))
+    goto generic_tails; /* see below */
+  /* The unambiguous '%d' in f2 may match any %d-like chars */
+  s1_tail = s1;
+  s1_shifted = 0;
+  if (('-' == s1_tail[0]) && isdigit (s1_tail[0]))
+    {
+      s1_tail += 2;
+      s1_shifted = 1;
+    }
+  while (isdigit (s1_tail[0]))
+    {
+      s1_tail++;
+      s1_shifted = 1;
+    }
+  if (s1_shifted && ('.' == s1_tail[0]) && isdigit (s1_tail[0]))
+    {
+      s1_tail += 2;
+      while (isdigit (s1_tail[0])) s1_tail++;
+    }
+  if (s1_shifted && (('e' == s1_tail[0]) || ('E' == s1_tail[0])))
+    {
+      if (('-' == s1_tail[1]) && isdigit (s1_tail[2]))
+        s1_tail += 3;
+      else if (isdigit (s1_tail[1]))
+        s1_tail += 2;
+      else
+        return SFF_ISECT_DIFF_END; /* 'E' in string does not match to f2_tail[0] char */
+      while (isdigit (s1_tail[0])) s1_tail++;
+    }
+  if (s1_shifted && (f2_tail[0] == s1_tail[0])) /* unambiguous synchronisation between f2 and s1 */
+    goto tails_are_in_sync; /* see below */
+  else if ('\0' == f2_tail[0])
+    return SFF_ISECT_DIFF_END; /* One string ends with %g, other with non-%g fixed char */
+  else
+    return SFF_ISECT_DISJOIN;
+
 f2_v_is_s:
   if ('\0' == f2_tail[0])
     { /* %s at the end of f2 intersects with any s1 */
@@ -956,7 +1029,7 @@ generic_tails:
 int
 sprintff_like (ccaddr_t s1, ccaddr_t f2)
 {
-  int s1_chk_pos, f2_chk_pos, s1_strlen;
+  int s1_chk_pos, f2_chk_pos /* s1_strlen */;
 /* Very basic check for disjoint beginnings. */
   s1_chk_pos = f2_chk_pos = 0;
   for (;;)
@@ -979,7 +1052,7 @@ sprintff_like (ccaddr_t s1, ccaddr_t f2)
       s1_chk_pos++; f2_chk_pos++;
     }
 /* Here we start actual calculation of a new result, starting from s1_chk_pos and f2_chk_pos offset. Then we cache it and return */
-  s1_strlen = s1_chk_pos + strlen (s1 + s1_chk_pos);
+  /* s1_strlen = s1_chk_pos + strlen (s1 + s1_chk_pos); */
   if (SFF_ISECT_OK == sff_dislike (s1 + s1_chk_pos, f2 + f2_chk_pos))
     return 1;
   return 0;
@@ -1108,9 +1181,8 @@ skip_save_f12: ;
   if (oldsize < res_count)
     {
       int newsize = oldsize ? oldsize : 1;
-      ccaddr_t *new_buf;
       do newsize *= 2; while (newsize < max_reslen);
-      new_buf = rvr->rvrSprintffs = (ccaddr_t *)t_alloc_box (newsize * sizeof (caddr_t), DV_ARRAY_OF_LONG);
+      rvr->rvrSprintffs = (ccaddr_t *)t_alloc_box (newsize * sizeof (caddr_t), DV_ARRAY_OF_LONG);
     }
 #ifdef MALLOC_DEBUG
   memset (rvr->rvrSprintffs, 0, box_length (rvr->rvrSprintffs));
