@@ -4,7 +4,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2012 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -42,13 +42,15 @@ extern "C" {
 #endif
 #include "xml_ecm.h"
 
+/* The name becomes misleading because "limofs" subquery can now be used for CONSTRUCT {...} WHERE {...} GROUP BY..., not only for LIMIT...OFFSET... */
 #define CTOR_NEEDS_LIMOFS_TRICK(top) ( \
  (DV_LONG_INT != DV_TYPE_OF (top->_.req_top.offset)) || \
  (DV_LONG_INT != DV_TYPE_OF (top->_.req_top.limit)) || \
  (0 != unbox ((caddr_t)(top->_.req_top.offset))) || \
  ((NULL != top->_.req_top.limit) && \
      ((1 != unbox ((caddr_t)(top->_.req_top.limit))) || \
-       (0 != BOX_ELEMENTS (top->_.req_top.pattern->_.gp.members)) ) ) )
+       (0 != BOX_ELEMENTS (top->_.req_top.pattern->_.gp.members)) ) ) || \
+ (NULL != top->_.req_top.groupings) )
 
 #define CTOR_DISJOIN_WHERE 1
 #define CTOR_MAY_INTERSECTS_WHERE 0
@@ -59,32 +61,31 @@ spar_compose_report_flag (sparp_t *sparp)
   sparp_env_t *spare = sparp->sparp_env;
   const char *fmtname;
   caddr_t res;
-  if (NULL != spare->spare_output_compose_report)
-    return spare->spare_output_compose_report;
-  fmtname = spare->spare_output_format_name; /* Report is always a result-set, so no spare_output_XXX_format name */
-  if ((NULL == spare->spare_output_format_name)
-    && (NULL == sparp->sparp_env->spare_parent_env)
+  if (NULL != sparp->sparp_sg->sg_output_compose_report)
+    return sparp->sparp_sg->sg_output_compose_report;
+  fmtname = (sparp->sparp_env->spare_disable_output_formatting ? NULL : sparp->sparp_sg->sg_output_format_name); /* Report is always a result-set, so no spare_output_XXX_format name */
+  if ((NULL == fmtname) && (NULL == sparp->sparp_env->spare_parent_env)
     && ssg_is_odbc_cli () )
     {
       if (ssg_is_odbc_msaccess_cli ())
-        fmtname = spare->spare_output_format_name = t_box_dv_short_string ("_MSACCESS_");
+        fmtname = sparp->sparp_sg->sg_output_format_name = t_box_dv_short_string ("_MSACCESS_");
       else
-        fmtname = spare->spare_output_format_name = t_box_dv_short_string ("_UDBC_");
+        fmtname = sparp->sparp_sg->sg_output_format_name = t_box_dv_short_string ("_UDBC_");
     }
   res = t_box_num_nonull (((NULL != fmtname) && (!strcmp (fmtname, "_JAVA_") || !strcmp (fmtname, "_UDBC_") || !strcmp (fmtname, "_MSACCESS_"))) ? 0 : 1);
-  spare->spare_output_compose_report = res;
+  sparp->sparp_sg->sg_output_compose_report = res;
   return res;
 }
 
-extern int sparp_ctor_fields_are_disjoin_with_where_fields (sparp_t *sparp, SPART **ctor_fields, SPART **where_fields);
+extern int sparp_ctor_fields_are_disjoin_with_where_fields (sparp_t *sparp, SPART *top, SPART **ctor_fields, SPART **where_fields);
 extern int sparp_ctor_fields_are_disjoin_with_data_gathering (sparp_t *sparp, SPART **ctor_fields, SPART *req, int the_query_is_topmost);
 
 int
-sparp_ctor_fields_are_disjoin_with_where_fields (sparp_t *sparp, SPART **ctor_fields, SPART **where_fields)
+sparp_ctor_fields_are_disjoin_with_where_fields (sparp_t *sparp, SPART *top, SPART **ctor_fields, SPART **where_fields)
 {
   sparp_equiv_t **top_eqs = sparp->sparp_sg->sg_equivs;
   int top_eq_count = sparp->sparp_sg->sg_equiv_count;
-  SPART *top_gp = sparp->sparp_expr->_.req_top.pattern;
+  SPART *top_gp = top->_.req_top.pattern;
   int fld_ctr;
   for (fld_ctr = SPART_TRIPLE_FIELDS_COUNT; fld_ctr--; /*no step*/)
     {
@@ -154,7 +155,7 @@ sparp_gp_trav_find_isect_with_ctor (sparp_t *sparp, SPART *curr, sparp_trav_stat
     {
     case SPAR_TRIPLE:
       if (CTOR_MAY_INTERSECTS_WHERE ==
-        sparp_ctor_fields_are_disjoin_with_where_fields (sparp, (SPART **)common_env, curr->_.triple.tr_fields) )
+        sparp_ctor_fields_are_disjoin_with_where_fields (sparp, sparp->sparp_stp->stp_trav_req_top, (SPART **)common_env, curr->_.triple.tr_fields) )
         return SPAR_GPT_COMPLETED;
       return SPAR_GPT_NODOWN;
     case SPAR_GP:
@@ -166,9 +167,9 @@ sparp_gp_trav_find_isect_with_ctor (sparp_t *sparp, SPART *curr, sparp_trav_stat
             return SPAR_GPT_COMPLETED;
 #else
             int isect_res;
-            sparp_t *sub_sparp = sparp_down_to_sub (sparp, curr);
+            sparp_gp_trav_suspend (sparp);
             isect_res = sparp_ctor_fields_are_disjoin_with_data_gathering (sub_sparp, (SPART **)common_env, curr->_.gp.subquery, 1);
-            sparp_up_from_sub (sparp, curr, sub_sparp);
+            sparp_gp_trav_resume (sparp);
             if (CTOR_MAY_INTERSECTS_WHERE == isect_res)
               return SPAR_GPT_COMPLETED;
             return SPAR_GPT_NODOWN;
@@ -176,6 +177,8 @@ sparp_gp_trav_find_isect_with_ctor (sparp_t *sparp, SPART *curr, sparp_trav_stat
           }
         case SERVICE_L:
           return 0; /* remote triples can not interfere with local data manipulations. This may change in the future if SERVICE group ctor template is added in SPARUL */
+        case VALUES_L:
+          return 0; /* no variables --- no interference */
         }
       break;
     }
@@ -187,9 +190,9 @@ int
 sparp_ctor_fields_are_disjoin_with_data_gathering (sparp_t *sparp, SPART **ctor_fields, SPART *req, int the_query_is_topmost)
 {
   int res;
-  SPART **saved_orig_retvals = NULL;
+  /*SPART **saved_orig_retvals = NULL;*/
   SPART **saved_retvals = NULL;
-  res = sparp_gp_trav (sparp, req->_.req_top.pattern, ctor_fields,
+  res = sparp_gp_trav_top_pattern (sparp, req, ctor_fields,
     sparp_gp_trav_find_isect_with_ctor, NULL,
     NULL, NULL, sparp_gp_trav_find_isect_with_ctor,
     NULL );
@@ -197,8 +200,8 @@ sparp_ctor_fields_are_disjoin_with_data_gathering (sparp_t *sparp, SPART **ctor_
     return CTOR_MAY_INTERSECTS_WHERE;
   if (the_query_is_topmost)
     {
-      saved_orig_retvals = req->_.req_top.orig_retvals;
-      req->_.req_top.orig_retvals = NULL;
+      /*saved_orig_retvals = req->_.req_top.orig_retvals;
+      req->_.req_top.orig_retvals = NULL;*/
       saved_retvals = req->_.req_top.retvals;
       req->_.req_top.retvals = NULL;
     }
@@ -208,7 +211,7 @@ sparp_ctor_fields_are_disjoin_with_data_gathering (sparp_t *sparp, SPART **ctor_
     NULL );
   if (the_query_is_topmost)
     {
-      req->_.req_top.orig_retvals = saved_orig_retvals;
+      /*req->_.req_top.orig_retvals = saved_orig_retvals;*/
       req->_.req_top.retvals = saved_retvals;
     }
   if (res & SPAR_GPT_COMPLETED)
@@ -225,7 +228,8 @@ typedef struct ctor_var_enumerator_s
   SPART *cve_limofs_var;		/*!< Variable that is passed from limit-offset subselect */
   caddr_t cve_limofs_var_alias;		/*!< Alias used for cve_limofs_var */
   int cve_make_quads;			/*!< Contructor should make quads */
-  SPART *cve_default_graph;		/*!< An expression for the dafult graph */
+  SPART *cve_default_graph;		/*!< An expression for the default graph. It is not used in the results ATM because we can generate a mix of triples and quads. */
+  int cve_graphs_should_be_set;		/*!< If \c cve_default_graph is NULL and \c cve_graphs_should_be_set is true and a ctor tmpl has no explicit graph then the default graph should be set from context or an error should be signalled */
 }
 ctor_var_enumerator_t;
 
@@ -251,7 +255,7 @@ spar_cve_find_or_add_variable (sparp_t *sparp, ctor_var_enumerator_t *haystack_c
 int
 sparp_gp_trav_ctor_var_to_limofs_aref (sparp_t *sparp, SPART *curr, sparp_trav_state_t *sts_this, void *common_env)
 { /* This rewrites variables that are nested into backquoted expressions in ctor template when the query has limit or offset clause */
-  ctor_var_enumerator_t *cve = common_env;
+  ctor_var_enumerator_t *cve = (ctor_var_enumerator_t *)common_env;
   int curr_type = SPART_TYPE (curr);
   int var_ctr;
   if (SPAR_BLANK_NODE_LABEL == curr_type)
@@ -268,6 +272,34 @@ sparp_gp_trav_ctor_var_to_limofs_aref (sparp_t *sparp, SPART *curr, sparp_trav_s
   return SPAR_GPT_NODOWN;
 }
 
+int
+spar_ctor_uses_default_graph (SPART *ctor_gp)
+{
+  int triple_ctr;
+  for (triple_ctr = BOX_ELEMENTS_INT (ctor_gp->_.gp.members); triple_ctr--; /* no step */)
+    {
+      SPART *memb = ctor_gp->_.gp.members[triple_ctr];
+      switch (SPART_TYPE (memb))
+        {
+        case SPAR_GP:
+          if (spar_ctor_uses_default_graph (memb))
+            return 1;
+          break;
+        case SPAR_TRIPLE:
+          {
+            SPART *g = memb->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX];
+            if (SPART_IS_DEFAULT_GRAPH_BLANK (g))
+              return 1;
+            break;
+          }
+        default:
+          break;
+        }
+    }
+  return 0;
+}
+
+
 #define CTOR_OPCODE_VARIABLE 1
 #define CTOR_OPCODE_BNODE 2
 #define CTOR_OPCODE_CONST_OR_EXPN 3
@@ -297,7 +329,10 @@ spar_compose_retvals_of_ctor (sparp_t *sparp, SPART *ctor_gp, const char *funnam
           SPART *triple = ctor_gp->_.gp.members[triple_ctr];
           SPART *g = triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX];
           int g_is_default = !cve->cve_make_quads || SPART_IS_DEFAULT_GRAPH_BLANK (g);
-          caddr_t *args = (g_is_default ?
+          caddr_t *args;
+          if (g_is_default && cve->cve_graphs_should_be_set && (NULL == cve->cve_default_graph))
+            spar_internal_error (sparp, "spar_" "compose_retvals_of_ctor(): no default graph but it is needed");
+          args = (g_is_default ?
             (caddr_t *)list (6,
               (ptrlong)CTOR_OPCODE_CONST_OR_EXPN, NULL,
               (ptrlong)CTOR_OPCODE_CONST_OR_EXPN, NULL,
@@ -358,7 +393,7 @@ bnode_found_or_added_for_big_ssl:
         {
           caddr_t *new_consts = (caddr_t *)dk_alloc_box ((ssl_count + 1) * sizeof (caddr_t), DV_ARRAY_OF_POINTER);
           memcpy (new_consts, ssl_consts_ptr[0], ssl_count * sizeof (caddr_t));
-          dk_free_box (ssl_consts_ptr[0]);
+          dk_free_box ((caddr_t)(ssl_consts_ptr[0]));
           ssl_consts_ptr[0] = new_consts;
         }
       ssl_consts_ptr[0][ssl_count] = (caddr_t)list_to_array (list_of_triples);
@@ -374,6 +409,8 @@ bnode_found_or_added_for_big_ssl:
       SPART **tvector_args;
       SPART *tvector_call;
       int triple_is_const = 1;
+      if (g_is_default && cve->cve_graphs_should_be_set && (NULL == cve->cve_default_graph))
+        spar_internal_error (sparp, "spar_" "compose_retvals_of_ctor(): no default graph but it is needed");
       tvector_args = (g_is_default ?
         (SPART **)t_list (6, NULL, NULL, NULL, NULL, NULL, NULL) :
         (SPART **)t_list (8, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) );
@@ -427,7 +464,7 @@ bnode_found_or_added:
             default:
               {
                 int old_total_vars_count = cve->cve_total_vars_count;
-                sparp_gp_trav (sparp, fld, cve, NULL, NULL, sparp_gp_trav_ctor_var_to_limofs_aref, NULL, NULL, NULL);
+                sparp_gp_trav (sparp, NULL /*unused*/, fld, cve, NULL, NULL, sparp_gp_trav_ctor_var_to_limofs_aref, NULL, NULL, NULL);
                 opcode_arg_ptr[0] = t_box_num_nonull (CTOR_OPCODE_CONST_OR_EXPN);
                 val_arg_ptr[0] = fld;
                 if (cve->cve_total_vars_count != old_total_vars_count)
@@ -467,7 +504,7 @@ args_ready:
       (SPART **)t_list (4, arg1, var_vector_arg, arg3, t_box_num_nonull (use_limits)) );
   if (cve->cve_limofs_var_alias)
     {
-      SPART *alias = spartlist (sparp, 4, SPAR_ALIAS, var_vector_expn, cve->cve_limofs_var_alias, SSG_VALMODE_AUTO);
+      SPART *alias = spartlist (sparp, 6, SPAR_ALIAS, var_vector_expn, cve->cve_limofs_var_alias, SSG_VALMODE_AUTO, (ptrlong)0, (ptrlong)0);
       retvals[0] = (SPART **)t_list (2, ctor_call, alias);
     }
   else
@@ -505,7 +542,6 @@ spar_compose_retvals_of_construct (sparp_t *sparp, SPART *top, SPART *ctor_gp,
   cve.cve_default_graph = NULL;
   spar_compose_retvals_of_ctor (sparp, ctor_gp, "sql:SPARQL_CONSTRUCT", NULL /* no big ssl const */, NULL, NULL,
     &(top->_.req_top.retvals), &cve, formatter, agg_formatter, agg_mdata, use_limits );
-
 }
 
 SPART *
@@ -566,7 +602,7 @@ spar_compose_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top, SPART *gra
     }
   graph_to_patch = spar_simplify_graph_to_patch (sparp, graph_to_patch);
   memset (&cve, 0, sizeof (ctor_var_enumerator_t));
-  log_mode = sparp->sparp_env->spare_sparul_log_mode;
+  log_mode = sparp->sparp_sg->sg_sparul_log_mode;
   if (NULL == log_mode)
     log_mode = t_NEW_DB_NULL;
   if (need_limofs_trick)
@@ -581,18 +617,21 @@ spar_compose_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top, SPART *gra
     cve.cve_bnodes_are_prohibited = 1;
   cve.cve_make_quads = ((NULL != multigraph) ? 1 : 0);
   cve.cve_default_graph = graph_to_patch;
+  cve.cve_graphs_should_be_set = 1;
   spar_compose_retvals_of_ctor (sparp, ctor_gp, "sql:SPARQL_CONSTRUCT", sc_for_big_ssl_const, NULL, NULL,
     &(top->_.req_top.retvals), &cve, NULL, NULL, NULL, 0 );
   rv = top->_.req_top.retvals;
-  if (NULL != sparp->sparp_env->spare_output_route_name)
+  if (NULL == graph_to_patch)
+    graph_to_patch = (SPART *)uname_virtrdf_ns_uri_DefaultSparul11Target;
+  if (NULL != sparp->sparp_sg->sg_output_route_name)
     {
-      top_fname = t_box_sprintf (200, "sql:SPARQL_ROUTE_DICT_CONTENT_%.100s", sparp->sparp_env->spare_output_route_name);
+      top_fname = t_box_sprintf (200, "sql:SPARQL_ROUTE_DICT_CONTENT_%.100s", sparp->sparp_sg->sg_output_route_name);
       rv[0] = spar_make_funcall (sparp, 0, top_fname,
         (SPART **)t_list (11, graph_to_patch,
           t_box_dv_short_string ((NULL != multigraph) ? (top_subtype_is_insert ? "INSERT_QUAD" : "DELETE_QUAD") : (top_subtype_is_insert ? "INSERT" : "DELETE")),
           ((NULL == sparp->sparp_env->spare_storage_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_storage_name),
-          ((NULL == sparp->sparp_env->spare_output_storage_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_output_storage_name),
-          ((NULL == sparp->sparp_env->spare_output_format_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_output_format_name),
+          ((NULL == sparp->sparp_sg->sg_output_storage_name) ? t_NEW_DB_NULL : sparp->sparp_sg->sg_output_storage_name),
+          ((sparp->sparp_env->spare_disable_output_formatting || NULL == sparp->sparp_sg->sg_output_format_name) ? t_NEW_DB_NULL : sparp->sparp_sg->sg_output_format_name),
           ((INSERT_L == top->_.req_top.subtype) ? t_NEW_DB_NULL : (caddr_t)(rv[0])),
           ((INSERT_L == top->_.req_top.subtype) ? (caddr_t)(rv[0]) : t_NEW_DB_NULL),
           t_NEW_DB_NULL,
@@ -658,7 +697,7 @@ spar_compose_retvals_of_modify (sparp_t *sparp, SPART *top, SPART *graph_to_patc
     }
   need_limofs_trick = CTOR_NEEDS_LIMOFS_TRICK(top);
   memset (&cve, 0, sizeof (ctor_var_enumerator_t));
-  log_mode = sparp->sparp_env->spare_sparul_log_mode;
+  log_mode = sparp->sparp_sg->sg_sparul_log_mode;
   if (NULL == log_mode)
     log_mode = t_NEW_DB_NULL;
   if (need_limofs_trick)
@@ -670,6 +709,7 @@ spar_compose_retvals_of_modify (sparp_t *sparp, SPART *top, SPART *graph_to_patc
   cve.cve_bnodes_are_prohibited = 1;
   cve.cve_make_quads = g_may_vary;
   cve.cve_default_graph = graph_to_patch;
+  cve.cve_graphs_should_be_set = 1;
   spar_compose_retvals_of_ctor (sparp, del_ctor_gp, "sql:SPARQL_CONSTRUCT", NULL /* no big ssl const */, NULL, NULL,
     &(top->_.req_top.retvals), &cve, NULL, NULL, NULL, 0 );
   cve.cve_limofs_var_alias = NULL;
@@ -678,14 +718,14 @@ spar_compose_retvals_of_modify (sparp_t *sparp, SPART *top, SPART *graph_to_patc
     &ins, &cve, NULL, NULL, NULL, 0 );
   rv = top->_.req_top.retvals;
 
-  if (NULL != sparp->sparp_env->spare_output_route_name)
+  if (NULL != sparp->sparp_sg->sg_output_route_name)
     rv[0] = spar_make_funcall (sparp, 0,
-      t_box_sprintf (200, "sql:SPARQL_ROUTE_DICT_CONTENT_%.100s", sparp->sparp_env->spare_output_route_name),
+      t_box_sprintf (200, "sql:SPARQL_ROUTE_DICT_CONTENT_%.100s", sparp->sparp_sg->sg_output_route_name),
       (SPART **)t_list (11, graph_to_patch,
         t_box_dv_short_string (g_may_vary ? "MODIFY" : "MODIFY_QUAD"),
         ((NULL == sparp->sparp_env->spare_storage_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_storage_name),
-        ((NULL == sparp->sparp_env->spare_output_storage_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_output_storage_name),
-        ((NULL == sparp->sparp_env->spare_output_format_name) ? t_NEW_DB_NULL : sparp->sparp_env->spare_output_format_name),
+        ((NULL == sparp->sparp_sg->sg_output_storage_name) ? t_NEW_DB_NULL : sparp->sparp_sg->sg_output_storage_name),
+        ((sparp->sparp_env->spare_disable_output_formatting || NULL == sparp->sparp_sg->sg_output_format_name) ? t_NEW_DB_NULL : sparp->sparp_sg->sg_output_format_name),
         rv[0], ins[0],
         t_NEW_DB_NULL,
         spar_exec_uid_and_gs_cbk (sparp), log_mode, spar_compose_report_flag (sparp) ) );
@@ -695,6 +735,83 @@ spar_compose_retvals_of_modify (sparp_t *sparp, SPART *top, SPART *graph_to_patc
         spar_exec_uid_and_gs_cbk (sparp), log_mode, spar_compose_report_flag (sparp) ) );
 }
 
+void
+spar_compose_ctor_triples_from_where_gp (sparp_t *sparp, int subtype, SPART *gp, SPART *g, dk_set_t *ret_tmpls)
+{
+  int memb_ctr;
+  if (SPAR_GP != SPART_TYPE (gp))
+    spar_error (sparp, "Cannot convert a pattern into constructor template, DELETE WHERE works only for simple quad templates");
+  if ((0 != gp->_.gp.subtype) && (WHERE_L != gp->_.gp.subtype) && (OPTIONAL_L != gp->_.gp.subtype))
+    spar_error (sparp, "DELETE WHERE works only for simple quad templates, only basic group patterns and OPTIONAL are allowed");
+  /*if (0 != BOX_ELEMENTS_0 (gp->_.gp.filters))
+    spar_error (sparp, "DELETE WHERE does not support FILTER");*/
+  DO_BOX_FAST (SPART *, memb, memb_ctr, gp->_.gp.members)
+    {
+      SPART *tmpl;
+      if (SPAR_TRIPLE != SPART_TYPE (memb))
+        {
+          spar_compose_ctor_triples_from_where_gp (sparp, subtype, memb, g, ret_tmpls);
+          continue;
+        }
+      if (DELETE_L == subtype)
+        {
+          if (0 < BOX_ELEMENTS_0 (memb->_.triple.options))
+            spar_error (sparp, "DELETE WHERE does not support OPTION () clauses of triples and related features, such as transitivity");
+          if (SPART_IS_DEFAULT_GRAPH_BLANK (memb->_.triple.tr_graph) && (NULL == g))
+            spar_error (sparp, "DELETE WHERE requires default graph but it is not provided");
+        }
+      tmpl = sparp_tree_full_copy (sparp, memb, gp);
+      t_set_push (ret_tmpls, tmpl);
+    }
+  END_DO_BOX_FAST;
+}
+
+SPART *
+spar_compose_ctor_gp_from_where_gp (sparp_t *sparp, int subtype, SPART *where_gp, SPART *gtp)
+{
+  dk_set_t tmpls = NULL;
+  caddr_t prev_fixed_graph = NULL;
+  int g_grp_count = 0;
+  SPART *tmpl_gp;
+  spar_compose_ctor_triples_from_where_gp (sparp, subtype, where_gp, gtp, &tmpls);
+  DO_SET (SPART *, triple, &tmpls)
+    {
+      SPART *g = triple->_.triple.tr_graph;
+      if (SPART_IS_DEFAULT_GRAPH_BLANK (g))
+        g = gtp;
+      if ((SPAR_LIT != SPART_TYPE (g)) && (SPAR_QNAME != SPART_TYPE (g)))
+        g_grp_count++;
+      else
+        {
+          caddr_t g_val = SPAR_LIT_OR_QNAME_VAL (g);
+          if ((DV_STRING == DV_TYPE_OF (g_val)) || (DV_UNAME == DV_TYPE_OF (g_val)))
+            {
+              if ((NULL == prev_fixed_graph) || strcmp (prev_fixed_graph, g_val))
+                g_grp_count++;
+              prev_fixed_graph = g_val;
+            }
+          else
+            g_grp_count++;
+        }
+    }
+  END_DO_SET()
+  tmpl_gp = (SPART *)(t_box_copy ((caddr_t)where_gp)); /* Dirty hack here, indeed, however we need only a box with type and proper length, nothing else */
+  tmpl_gp->_.gp.members = (SPART **)t_revlist_to_array (tmpls);
+  tmpl_gp->_.gp.options = NULL;
+  if (1 < g_grp_count)
+    tmpl_gp->_.gp.options = (SPART **)t_list (2, (SPART *)((ptrlong)QUAD_L), t_box_num_nonull (g_grp_count));
+  return tmpl_gp;
+}
+
+void
+spar_compose_retvals_of_delete_from_wm (sparp_t *sparp, SPART *tree, SPART *graph_to_patch)
+{
+  SPART *gtp = spar_simplify_graph_to_patch (sparp, graph_to_patch);
+  SPART *pat = tree->_.req_top.pattern;
+  SPART *tmpl_gp = spar_compose_ctor_gp_from_where_gp (sparp, DELETE_L, pat, gtp);
+  spar_compose_retvals_of_insert_or_delete (sparp, tree, graph_to_patch, tmpl_gp);
+}
+
 SPART *
 spar_emulate_ctor_field (sparp_t *sparp, SPART *opcode, SPART *oparg, SPART **vars)
 {
@@ -707,8 +824,8 @@ spar_emulate_ctor_field (sparp_t *sparp, SPART *opcode, SPART *oparg, SPART **va
       {
         if (NULL == bnode_emulation)
 #ifdef DEBUG
-          bnode_emulation = (SPART *)box_copy_tree ((caddr_t)spartlist (sparp, 7 + (sizeof (rdf_val_range_t) / sizeof (caddr_t)), SPAR_BLANK_NODE_LABEL,
-            NULL, NULL, NULL, NULL, NULL, SPART_RVR_LIST_OF_NULLS, NULL ) );
+          bnode_emulation = (SPART *)box_copy_tree ((caddr_t)spartlist (sparp, 8 + (sizeof (rdf_val_range_t) / sizeof (caddr_t)), SPAR_BLANK_NODE_LABEL,
+            NULL, NULL, NULL, NULL, NULL, SPART_RVR_LIST_OF_NULLS, (ptrlong)(0x0), NULL ) );
 #else
           bnode_emulation = box_copy_tree (spartlist (sparp, 1, SPAR_BLANK_NODE_LABEL));
 #endif
@@ -720,6 +837,19 @@ spar_emulate_ctor_field (sparp_t *sparp, SPART *opcode, SPART *oparg, SPART **va
   return NULL; /* never reached */
 }
 
+void
+spar_emulate_ctor_fields (sparp_t *sparp, SPART *graph_expn, SPART **args, SPART **known_vars, SPART **quad_fields)
+{
+  int g_is_set = (8 <= BOX_ELEMENTS (args));
+  if (g_is_set)
+    quad_fields [SPART_TRIPLE_GRAPH_IDX] = spar_emulate_ctor_field (sparp, args[6], args[7], known_vars);
+  else
+    quad_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
+  quad_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
+  quad_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, args[2], args[3], known_vars);
+  quad_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, args[4], args[5], known_vars);
+}
+
 SPART *
 spar_find_single_physical_triple_pattern (sparp_t *sparp, SPART *tree)
 {
@@ -789,6 +919,12 @@ spar_dealias (sparp_t *sparp, SPART *expn, int expected_type)
   return expn;
 }
 
+#define CTOR_ARGS_HAVE_BNODE(args) ( \
+  (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[0]))) \
+  || (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[2]))) \
+  || (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[4]))) \
+  || ((8 <= BOX_ELEMENTS (args)) && (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[6])))) )
+
 int
 spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
 {
@@ -798,21 +934,23 @@ spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
   SPART **retvals = top->_.req_top.retvals;
   int retvals_count = BOX_ELEMENTS (retvals);
   SPART **var_triples, **args;
-  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
-  if (NULL != sparp->sparp_env->spare_output_route_name)
+  int g_is_set;
+  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call /* unused?? *compose_report_expn */;
+  if (NULL != sparp->sparp_sg->sg_output_route_name)
     return 0; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   triple = spar_find_single_physical_triple_pattern (sparp, top->_.req_top.pattern);
   if (NULL == triple)
     return 0; /* nontrivial pattern, can not be optimized this way */
   arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (arg0)) && (5 == BOX_ELEMENTS (arg0->_.funcall.argtrees)));
-  if (strcmp ("sql:SPARQL_DELETE_DICT_CONTENT", arg0->_.funcall.qname))
+  if (strcmp ("sql:SPARQL_DELETE_DICT_CONTENT", arg0->_.funcall.qname)
+    && strcmp ("sql:SPARQL_DELETE_QUAD_DICT_CONTENT", arg0->_.funcall.qname) )
     return 0;
   graph_expn		= arg0->_.funcall.argtrees[0];
   ctor			= arg0->_.funcall.argtrees[1];
   uid_expn		= arg0->_.funcall.argtrees[2];
   log_mode_expn		= arg0->_.funcall.argtrees[3];
-  compose_report_expn	= arg0->_.funcall.argtrees[4];
+  /* unused?? compose_report_expn	= arg0->_.funcall.argtrees[4]; */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ctor)) && (4 == BOX_ELEMENTS (ctor->_.funcall.argtrees)));
   dbg_assert (DELETE_L == top->_.req_top.subtype);
   var_triples = ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
@@ -826,13 +964,22 @@ spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
   if (1 != BOX_ELEMENTS (var_triples))
     return 0; /* nontrivial constructor, can not be optimized this way */
   args = var_triples[0]->_.funcall.argtrees;
-  if ((CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[0]))) ||
-    (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[2]))) ||
-    (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[4]))) )
+  g_is_set = (8 <= BOX_ELEMENTS (args));
+  if (CTOR_ARGS_HAVE_BNODE (args))
     return 0; /* bnodes in constructor can not be optimized this way (BTW that is blab when inside DELETE) */
-  if (!spar_tr_fields_are_similar (sparp, graph_expn,
-      triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX] ) )
-    return 0;
+  if (g_is_set)
+    {
+      emu = spar_emulate_ctor_field (sparp, args[6], args[7], known_vars);
+      if (!spar_tr_fields_are_similar (sparp, emu,
+          triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX] ) )
+        return 0;
+    }
+  else
+    {
+      if (!spar_tr_fields_are_similar (sparp, graph_expn,
+          triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX] ) )
+        return 0;
+    }
   emu = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
   if (!spar_tr_fields_are_similar (sparp, emu,
       triple->_.triple.tr_fields[SPART_TRIPLE_SUBJECT_IDX] ) )
@@ -870,18 +1017,21 @@ spar_optimize_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top)
   dk_set_t bad_triples = NULL;
   int all_triple_count, bad_triple_count, tctr;
   const char *fname;
-  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
-  if (NULL != sparp->sparp_env->spare_output_route_name)
+  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call /* unused?? *compose_report_expn */;
+  if (NULL != sparp->sparp_sg->sg_output_route_name)
     return; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (arg0)) && (5 == BOX_ELEMENTS (arg0->_.funcall.argtrees)));
-  if (strcmp ("sql:SPARQL_INSERT_DICT_CONTENT", arg0->_.funcall.qname) && strcmp ("sql:SPARQL_DELETE_DICT_CONTENT", arg0->_.funcall.qname))
+  if (strcmp ("sql:SPARQL_INSERT_DICT_CONTENT", arg0->_.funcall.qname)
+    && strcmp ("sql:SPARQL_INSERT_QUAD_DICT_CONTENT", arg0->_.funcall.qname)
+    && strcmp ("sql:SPARQL_DELETE_DICT_CONTENT", arg0->_.funcall.qname)
+    && strcmp ("sql:SPARQL_DELETE_QUAD_DICT_CONTENT", arg0->_.funcall.qname) )
     return;
   graph_expn		= arg0->_.funcall.argtrees[0];
   ctor			= arg0->_.funcall.argtrees[1];
   uid_expn		= arg0->_.funcall.argtrees[2];
   log_mode_expn		= arg0->_.funcall.argtrees[3];
-  compose_report_expn	= arg0->_.funcall.argtrees[4];
+  /* unused?? compose_report_expn	= arg0->_.funcall.argtrees[4]; */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ctor)) && (4 == BOX_ELEMENTS (ctor->_.funcall.argtrees)));
   var_triples = ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
   all_triple_count = bad_triple_count = BOX_ELEMENTS (var_triples);
@@ -896,13 +1046,8 @@ spar_optimize_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top)
     {
       SPART **args = var_triples[tctr]->_.funcall.argtrees;
       SPART *quad_fields[SPART_TRIPLE_FIELDS_COUNT];
-      quad_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
-      quad_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
-      quad_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, args[2], args[3], known_vars);
-      quad_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, args[4], args[5], known_vars);
-      if ((CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[0]))) ||
-        (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[2]))) ||
-        (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[4]))) )
+      spar_emulate_ctor_fields (sparp, graph_expn, args, known_vars, quad_fields);
+      if (CTOR_ARGS_HAVE_BNODE (args))
         t_set_push (&positions_with_bnodes, ((void *)((ptrlong)tctr)));
       if (CTOR_DISJOIN_WHERE ==
         sparp_ctor_fields_are_disjoin_with_data_gathering (sparp, quad_fields, top, 1) )
@@ -959,8 +1104,8 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
   dk_set_t bad_ins_triples = NULL;
   int all_del_triple_count, bad_del_triple_count, del_const_count, del_tctr;
   int all_ins_triple_count, bad_ins_triple_count, ins_tctr;
-  SPART *arg0, *graph_expn, *del_ctor, *ins_ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
-  if (NULL != sparp->sparp_env->spare_output_route_name)
+  SPART *arg0, *graph_expn, *del_ctor, *ins_ctor, *uid_expn, *log_mode_expn, *good_ctor_call /* unused?? *compose_report_expn */;
+  if (NULL != sparp->sparp_sg->sg_output_route_name)
     return; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (arg0)) && (6 == BOX_ELEMENTS (arg0->_.funcall.argtrees)));
@@ -971,7 +1116,7 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
   ins_ctor		= arg0->_.funcall.argtrees[2];
   uid_expn		= arg0->_.funcall.argtrees[3];
   log_mode_expn		= arg0->_.funcall.argtrees[4];
-  compose_report_expn	= arg0->_.funcall.argtrees[5];
+  /* unused?? compose_report_expn	= arg0->_.funcall.argtrees[5]; */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (del_ctor)) && (4 == BOX_ELEMENTS (del_ctor->_.funcall.argtrees)));
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ins_ctor)) && (4 == BOX_ELEMENTS (ins_ctor->_.funcall.argtrees)));
   del_var_triples = del_ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
@@ -992,10 +1137,7 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
     {
       SPART **args = del_var_triples[del_tctr]->_.funcall.argtrees;
       SPART *quad_fields[SPART_TRIPLE_FIELDS_COUNT];
-      quad_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
-      quad_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
-      quad_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, args[2], args[3], known_vars);
-      quad_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, args[4], args[5], known_vars);
+      spar_emulate_ctor_fields (sparp, graph_expn, args, known_vars, quad_fields);
       if (CTOR_DISJOIN_WHERE ==
         sparp_ctor_fields_are_disjoin_with_data_gathering (sparp, quad_fields, top, 1) )
         {
@@ -1017,13 +1159,8 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
     {
       SPART **args = ins_var_triples[ins_tctr]->_.funcall.argtrees;
       SPART *quad_fields[SPART_TRIPLE_FIELDS_COUNT];
-      quad_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
-      quad_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
-      quad_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, args[2], args[3], known_vars);
-      quad_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, args[4], args[5], known_vars);
-      if ((CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[0]))) ||
-        (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[2]))) ||
-        (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[4]))) )
+      spar_emulate_ctor_fields (sparp, graph_expn, args, known_vars, quad_fields);
+      if (CTOR_ARGS_HAVE_BNODE (args))
         t_set_push (&positions_with_bnodes, ((void *)((ptrlong)ins_tctr)));
       if (CTOR_DISJOIN_WHERE !=
         sparp_ctor_fields_are_disjoin_with_data_gathering (sparp, quad_fields, top, 1) )
@@ -1032,24 +1169,18 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
         {
           SPART **del_args = del_var_triples[del_tctr]->_.funcall.argtrees;
           SPART *del_fields[SPART_TRIPLE_FIELDS_COUNT];
-          del_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
-          del_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, del_args[0], del_args[1], known_vars);
-          del_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, del_args[2], del_args[3], known_vars);
-          del_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, del_args[4], del_args[5], known_vars);
+          spar_emulate_ctor_fields (sparp, graph_expn, del_args, known_vars, del_fields);
           if (CTOR_DISJOIN_WHERE !=
-            sparp_ctor_fields_are_disjoin_with_where_fields (sparp, quad_fields, del_fields) )
+            sparp_ctor_fields_are_disjoin_with_where_fields (sparp, top, quad_fields, del_fields) )
             goto ins_is_bad; /* see below */
         }
       for (del_tctr = del_const_count; del_tctr--; /* no step */)
         {
           SPART **del_args = del_const_triples[del_tctr]->_.funcall.argtrees;
           SPART *del_fields[SPART_TRIPLE_FIELDS_COUNT];
-          del_fields [SPART_TRIPLE_GRAPH_IDX] = graph_expn;
-          del_fields [SPART_TRIPLE_SUBJECT_IDX] = spar_emulate_ctor_field (sparp, del_args[0], del_args[1], known_vars);
-          del_fields [SPART_TRIPLE_PREDICATE_IDX] = spar_emulate_ctor_field (sparp, del_args[2], del_args[3], known_vars);
-          del_fields [SPART_TRIPLE_OBJECT_IDX] = spar_emulate_ctor_field (sparp, del_args[4], del_args[5], known_vars);
+          spar_emulate_ctor_fields (sparp, graph_expn, del_args, known_vars, del_fields);
           if (CTOR_DISJOIN_WHERE !=
-            sparp_ctor_fields_are_disjoin_with_where_fields (sparp, quad_fields, del_fields) )
+            sparp_ctor_fields_are_disjoin_with_where_fields (sparp, top, quad_fields, del_fields) )
             goto ins_is_bad; /* see below */
         }
       t_set_push (&good_positions, ((void *)((ptrlong)ins_tctr)));
