@@ -1,14 +1,14 @@
 /*
  *  sqlbif2.c
  *
- *  $Id: sqlbif2.c,v 1.31.2.14 2010/12/09 09:53:13 source Exp $
+ *  $Id$
  *
  *  SQL Built In Functions. Part 2
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -83,8 +83,7 @@ bif_ddl_read_constraints (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
   return NULL;
 }
 
-static id_hash_t *name_to_pl_name;
-
+id_hash_t *name_to_pl_name;
 
 void
 pl_bif_name_define (const char *name)
@@ -117,7 +116,7 @@ find_pl_bif_name (caddr_t name)
       full = (caddr_t *) id_hash_get (name_to_pl_name, (caddr_t) &name2);
       dk_free_box (name2);
       if (full)
-	return *full;
+        return *full;
     }
   return name;
 }
@@ -716,6 +715,9 @@ bif_client_attr (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
       else
 	return NULL;
 
+      if (!cert)
+	return NULL;
+
       in = BIO_new (BIO_s_mem());
 
       if (!in)
@@ -1136,12 +1138,12 @@ rfc1808_expand_uri (/*query_instance_t *qi,*/ ccaddr_t base_uri, ccaddr_t rel_ur
     }
   if ((base_cs != buffer_cs_upcase) && !((NULL != base_cs) && (NULL != buffer_cs_upcase) && !strcmp (base_cs, buffer_cs_upcase)))
     {
-      base_uri = charset_recode_from_named_to_named ((caddr_t)base_uri, base_cs, buffer_cs_upcase, &base_uri_is_temp, err_ret);
+      base_uri = charset_recode_from_named_to_named ((query_instance_t *)NULL, (caddr_t)base_uri, base_cs, buffer_cs_upcase, &base_uri_is_temp, err_ret);
       if (err_ret[0]) goto res_complete; /* see below */
     }
   if ((rel_cs != buffer_cs_upcase) && !((NULL != rel_cs) && (NULL != buffer_cs_upcase) && !strcmp (rel_cs, buffer_cs_upcase)))
     {
-      rel_uri = charset_recode_from_named_to_named ((caddr_t)rel_uri, rel_cs, buffer_cs_upcase, &rel_uri_is_temp, err_ret);
+      rel_uri = charset_recode_from_named_to_named ((query_instance_t *)NULL, (caddr_t)rel_uri, rel_cs, buffer_cs_upcase, &rel_uri_is_temp, err_ret);
       if (err_ret[0]) goto res_complete; /* see below */
     }
   if ((NULL == base_uri) || ('\0' == base_uri[0]))
@@ -1347,7 +1349,7 @@ buffer_ready:
     ((NULL == buffer_cs_upcase) || (NULL == output_cs_upcase) || strcmp(buffer_cs_upcase, output_cs_upcase)) )
     {
       caddr_t boxed_buffer = box_dv_short_nchars (buffer, buf_tail - buffer);
-      res = charset_recode_from_named_to_named (boxed_buffer, buffer_cs_upcase, output_cs_upcase, &res_is_new, err_ret);
+      res = charset_recode_from_named_to_named ((query_instance_t *)NULL, boxed_buffer, buffer_cs_upcase, output_cs_upcase, &res_is_new, err_ret);
       if (res_is_new)
         dk_free_box (boxed_buffer);
       else
@@ -1371,6 +1373,8 @@ buffer_ready:
     }
 
 res_complete:
+  if (!res_is_new)
+    res = box_copy (res);
   dk_free_box (output_cs_upcase);
   if (base_uri_is_temp)
     dk_free_box ((caddr_t) base_uri);
@@ -1595,13 +1599,110 @@ bif_format_number (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return res;
 }
 
+static caddr_t
+soundex (caddr_t name)
+{
+#define raw_toupper(C) ((C) & (255-32))
+  char tmp[5] = {'0','0','0','0',0} , current_code = '\0', previous_code = '\0';
+  int inx = 0, i = 0;
+  if (box_length (name) > 1)
+    {
+      while (name[inx] && !isalpha (name[inx])) inx ++;
+      if (name[inx]) tmp[i++] = raw_toupper (name[inx++]);
+      for (; inx < box_length (name) - 1; inx ++)
+	{
+	  char c;
+	  c = raw_toupper (name[inx]);
+	  if (!isalpha (c)) continue;
+	  current_code = '\0';
+	  if (strchr ("BFPV", c))
+	    current_code = '1';
+	  else if (strchr ("CSKGJQXZ", c))
+	    current_code = '2';
+	  else if (strchr ("DT", c))
+	    current_code = '3';
+	  else if (strchr ("L", c))
+	    current_code = '4';
+	  else if (strchr ("MN", c))
+	    current_code = '5';
+	  else if (strchr ("R", c))
+	    current_code = '6';
+	  if (current_code != previous_code && current_code != '\0')
+	    tmp [i++] = current_code;
+	  if (i >= 4) break;
+	  /*if (current_code != '\0')*/
+	  previous_code = current_code;
+	}
+      for (;i < 4; i ++)
+	tmp[i] = '0';
+    }
+  return box_dv_short_string (tmp);
+}
+
+static caddr_t
+bif_soundex (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t name = bif_string_arg (qst, args, 0, "soundex");
+  return soundex (name);
+}
+
+static caddr_t
+bif_difference (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t name1 = bif_string_arg (qst, args, 0, "difference");
+  caddr_t name2 = bif_string_arg (qst, args, 1, "difference");
+  caddr_t soundex1, soundex2;
+  int result = 0;
+  char tmp[3] = {0,0,0};
+  if (box_length (name1) <= 1 || box_length (name2) <= 1)
+    return box_num (0);
+  soundex1 = soundex (name1);
+  soundex2 = soundex (name2);
+  if (!strcmp (soundex1, soundex2))
+    {
+      result = 4;
+      goto ret;
+    }
+  if (soundex1[0] == soundex2[0])
+    result = 1;
+  if (strstr (soundex2, soundex1 + 1)) /* 2,3,4 */
+    {
+      result += 3;
+      goto ret;
+    }
+  if (strstr (soundex2, soundex1 + 2)) /* 3,4 */
+    {
+      result += 2;
+      goto ret;
+    }
+  memcpy (&tmp[0], soundex1 + 1, 2);
+  if (strstr (soundex2, tmp)) /* 2,3 */
+    {
+      result += 2;
+      goto ret;
+    }
+  if (strchr (soundex2, soundex1[1]))
+    result ++;
+  if (strchr (soundex2, soundex1[2]))
+    result ++;
+  if (strchr (soundex2, soundex1[3]))
+    result ++;
+ret:
+  dk_free_box (soundex1);
+  dk_free_box (soundex2);
+  return box_num (result);
+}
 
 static caddr_t
 bif_this_server (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   return NEW_DB_NULL;
 }
-
+static caddr_t
+bif_is_geometry (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  return box_num (0);
+}
 
 void
 sqlbif2_init (void)
@@ -1632,7 +1733,10 @@ sqlbif2_init (void)
   bif_define ("patch_restricted_xml_chars", bif_patch_restricted_xml_chars);
   bif_define_typed ("format_number", bif_format_number, &bt_varchar);
   bif_define ("__stop_cpt", bif_stop_cpt);
+  bif_define ("soundex", bif_soundex);
+  bif_define ("difference", bif_difference);
   bif_define ("repl_this_server", bif_this_server);
+  bif_define ("isgeometry", bif_is_geometry);
   /*sqls_bif_init ();*/
   sqls_bif_init ();
   sqlo_inv_bif_int ();
