@@ -1,14 +1,14 @@
 /*
  *  sqlcmps.h
  *
- *  $Id: sqlcmps.h,v 1.14.2.10 2010/02/01 13:08:57 source Exp $
+ *  $Id$
  *
  *  SQL Compiler Data Structures
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -325,6 +325,9 @@ typedef struct trig_cols_s
   } trig_cols_t;
 
 
+int key_matches_index_opt (dbe_key_t * key, caddr_t opt);
+dbe_key_t *  tb_key_by_index_opt (dbe_table_t * tb, caddr_t opt);
+
 state_slot_t * col_ref_col (sql_comp_t * sc, caddr_t ref);
 
 void comp_scalar_exp (sql_comp_t *sc, dk_set_t * code_vec, sql_tree_t * tree,
@@ -526,7 +529,7 @@ void sqlc_meta_data_hook (sql_comp_t * sc, ST * tree);
 void sqlc_proc_table_cols (sql_comp_t * sc, comp_table_t * ct);
 
 void sqlc_top_select_dt (sql_comp_t * sc, ST * tree);
-ST ** sqlc_selection_names (ST * tree);
+ST ** sqlc_selection_names (ST * tree, int only_edit_tree);
 
 #define P_NO_MATCH 0
 #define P_EXACT 1
