@@ -1,14 +1,14 @@
 /*
  *  sqlhash.c
  *
- *  $Id: sqlhash.c,v 1.5.2.9 2010/10/29 14:11:04 source Exp $
+ *  $Id$
  *
  *  Dynamic SQL Compiler, part 2
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -223,8 +223,6 @@ setp_distinct_hash (sql_comp_t * sc, setp_node_t * setp, long n_rows)
 	  case AMMSC_COUNTSUM:
 	  case AMMSC_SUM:
 	      case AMMSC_AVG:
-		  ha->ha_memcache_only = 0;
-		  break;
 	  case AMMSC_MIN:
 	  case AMMSC_MAX:
 		    {
