@@ -1,14 +1,14 @@
 /*
  *  sqlo.c
  *
- *  $Id: sqlo.c,v 1.15.2.14 2010/05/18 09:08:13 source Exp $
+ *  $Id$
  *
  *  sql scope inference
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -36,6 +36,7 @@
 #include "arith.h"
 #include "security.h"
 #include "sqlo.h"
+#include "remote.h"
 
 
 #define CHECK_OBSERVER(tree_ptr) \
@@ -866,6 +867,7 @@ sqlo_add_table_ref (sqlo_t * so, ST ** tree_ret, dk_set_t *res)
 	view = (ST*) sch_view_def (so->so_sc->sc_cc->cc_schema, tb->tb_name);
 	if (!view || inside_view)
 	  {
+	    remote_table_t * rt = find_remote_table (tb->tb_name, 0);
 	    t_NEW_VARZ (op_table_t, ot);
 	    ot->ot_opts = ST_OPT (tree, caddr_t *, _.table.opts);
 	    ot->ot_prefix = tree->_.table.prefix;
@@ -873,6 +875,7 @@ sqlo_add_table_ref (sqlo_t * so, ST ** tree_ret, dk_set_t *res)
 	    ot->ot_new_prefix = t_box_string (tmp);
 	    tree->_.table.prefix = ot->ot_new_prefix;
 	    ot->ot_table = tb;
+	    ot->ot_rds = rt ? rt->rt_rds : NULL;
 	    ot->ot_u_id = (oid_t) unbox (tree->_.table.u_id);
 	    ot->ot_g_id = (oid_t) unbox (tree->_.table.g_id);
 	    if (ST_P (view, PROC_TABLE))
@@ -1353,6 +1356,19 @@ sqlo_expand_dt (sqlo_t *so, ST *tree, ST ** from_ret, op_table_t *ot, int is_in_
     }
 }
 
+/* check after sqlo_dt_inlineable if right side selection list has expressions containing no columns of the table */
+static int
+sqlo_selection_all_cols (sqlo_t * so, caddr_t * tree, op_table_t * ot)
+{
+  int inx;
+  DO_BOX (ST *, s, inx, (ST**)tree)
+    {
+      if (!sqlo_has_col_ref (s))
+	return 0;
+    }
+  END_DO_BOX;
+  return 1;
+}
 
 int
 sqlo_inline_jt (sqlo_t * so, ST * tree, ST * exp, op_table_t * ot)
@@ -1371,7 +1387,8 @@ sqlo_inline_jt (sqlo_t * so, ST * tree, ST * exp, op_table_t * ot)
       else if (ST_P (exp->_.join.left, TABLE_REF) && ST_P (exp->_.join.left->_.table_ref.table, JOINED_TABLE))
 	any += sqlo_inline_jt (so, tree, exp->_.join.left->_.table_ref.table, ot);
       if (OJ_LEFT == exp->_.join.type
-	  && sqlo_dt_inlineable (so, tree, exp->_.join.right, ot, 1))
+	  && sqlo_dt_inlineable (so, tree, exp->_.join.right, ot, 1)
+	  && sqlo_selection_all_cols (so, exp->_.join.right->_.table_ref.table->_.select_stmt.selection, ot))
 	{
 	  /* left oj with single table dt to the right. */
 	  ST * texp = exp->_.join.right->_.table_ref.table->_.select_stmt.table_exp;
@@ -1583,7 +1600,10 @@ sqlo_check_ft_offband (sqlo_t * so, op_table_t * ot, ST ** args, char type)
 	  inx + 1, sqlo_spec_predicate_name(type), arg );
     }
   if (off)
-    ot->ot_text_offband = (op_virt_col_t **) list_to_array (off);
+    {
+      ot->ot_text_offband = (op_virt_col_t **) t_list_to_array (off);
+      dk_set_free (off);
+    }
 }
 
 
@@ -2013,23 +2033,29 @@ sqlo_opt_value (caddr_t * opts, int opt)
 }
 
 
-void
+dk_set_t
+t_set_diff_ordered (dk_set_t s, dk_set_t minus)
+{
+  return dk_set_nreverse (t_set_diff (s, minus));
+}
+
+int
 sqlo_expand_distinct_joins (sqlo_t * so, ST *tree, op_table_t *sel_ot, dk_set_t *res)
 {
-  int inx;
+  int inx, has_expand = 0;
   if (!tree->_.select_stmt.table_exp || !SEL_IS_DISTINCT (tree))
-    return;
+    return 0;
   _DO_BOX (inx, tree->_.select_stmt.table_exp->_.table_exp.from)
     {
       ST **tbp = & (tree->_.select_stmt.table_exp->_.table_exp.from[inx]);
-      if (ST_P (*tbp, TABLE_REF))
+      while (ST_P (*tbp, TABLE_REF))
 	tbp = & (*tbp)->_.table_ref.table;
 
       if (ST_P (*tbp, JOINED_TABLE) && ((*tbp)->_.join.type == OJ_LEFT || (*tbp)->_.join.type == OJ_FULL))
 	{
-	  ST *rtb = (*tbp)->_.join.right;
+	  ST *rtb = (*tbp)->_.join.right, *ltp;
 	  op_table_t *ot = NULL;
-	  if (ST_P (rtb, TABLE_REF))
+	  while (ST_P (rtb, TABLE_REF))
 	    rtb = rtb->_.table_ref.table;
 	  if (ST_P (rtb, TABLE_DOTTED))
 	    ot = sqlo_cname_ot (so, rtb->_.table.prefix);
@@ -2045,7 +2071,7 @@ sqlo_expand_distinct_joins (sqlo_t * so, ST *tree, op_table_t *sel_ot, dk_set_t
 		}
 	      if (ot->ot_from_ots)
 		{
-		  so->so_tables = t_set_diff (so->so_tables, ot->ot_from_ots);
+		  so->so_tables = t_set_diff_ordered (so->so_tables, ot->ot_from_ots);
 		  so->so_scope->sco_tables = t_set_diff (so->so_tables, ot->ot_from_ots);
 		  sel_ot->ot_from_ots = t_set_diff (sel_ot->ot_from_ots, ot->ot_from_ots);
 		}
@@ -2053,11 +2079,16 @@ sqlo_expand_distinct_joins (sqlo_t * so, ST *tree, op_table_t *sel_ot, dk_set_t
 	      t_set_delete (&so->so_scope->sco_tables, ot);
 	      t_set_delete (res, (*tbp)->_.join.cond);
 	      t_set_delete (&sel_ot->ot_from_ots, ot);
-	      *tbp = (*tbp)->_.join.left;
+	      ltp = (*tbp)->_.join.left;
+	      if (ST_P (tree->_.select_stmt.table_exp->_.table_exp.from[inx], TABLE_REF) && ST_P (ltp, TABLE_REF))
+		ltp = ltp->_.table_ref.table;
+	      *tbp = ltp;
+	      has_expand ++;
 	    }
 	}
     }
   END_DO_BOX;
+  return has_expand;
 }
 
 
@@ -2749,8 +2780,16 @@ sqlo_select_scope (sqlo_t * so, ST ** ptree)
     }
   /* end dt expansion */
 #endif
-  if (texp && SEL_IS_DISTINCT (tree))
-    sqlo_expand_distinct_joins (so, tree, ot, &res);
+  if (texp && SEL_IS_DISTINCT (tree) && sqlo_expand_distinct_joins (so, tree, ot, &res))
+    {
+      char old_rescope = so->so_is_rescope;
+      so->so_this_dt = old_dt;
+      so->so_is_rescope = 1;
+      so->so_scope = so->so_scope->sco_super;
+      sqlo_scope (so, ptree);
+      so->so_is_rescope = old_rescope;
+      return;
+    }
   DO_SET (ST *, jc, &res)
     {
       jc->_.join.cond = (ST *) STAR;
@@ -2965,6 +3004,8 @@ sqlo_scalar_subq_scope (sqlo_t * so, ST ** ptree)
   dk_set_t s;
   sql_scope_t * sco = so->so_scope;
   ST * org, * res;
+  if (!sco)
+    SQL_GPF_T1 (so->so_sc->sc_cc, "no scope");
   for (s = sco->sco_scalar_subqs; s; s = s->next->next)
     {
       org = (ST*)s->data;
@@ -3074,6 +3115,7 @@ sqlo_scope (sqlo_t * so, ST ** ptree)
       {
 	int inx;
 	ST *res;
+	char * call_name = tree->_.call.name;
 	if (DV_ARRAY_OF_POINTER == DV_TYPE_OF (tree->_.call.name) && BOX_ELEMENTS (tree->_.call.name) == 1)
 	  {
 	    sqlo_scope (so, &(((ST **) tree->_.call.name)[0]));
@@ -3082,6 +3124,13 @@ sqlo_scope (sqlo_t * so, ST ** ptree)
 	  {
 	    CHECK_METHOD_CALL (ptree);
 	  }
+	/* mark qr to do lock if it is for SPARQL insert/delete triples */
+	if (DV_STRINGP (call_name) &&
+	    (!casemode_strcmp (call_name, "DB.DBA.SPARQL_INSERT_DICT_CONTENT") ||
+	     !casemode_strcmp (call_name, "DB.DBA.SPARQL_DELETE_DICT_CONTENT") ||
+	     !casemode_strcmp (call_name, "DB.DBA.SPARUL_LOAD") ||
+	     !casemode_strcmp (call_name, "DB.DBA.SPARUL_CLEAR")))
+	  so->so_sc->sc_cc->cc_query->qr_lock_mode = PL_EXCLUSIVE;
 	_DO_BOX (inx, tree->_.call.params)
 	  {
 	    sqlo_scope (so, &(tree->_.call.params[inx]));
