@@ -1,14 +1,14 @@
 /*
  *  sqlo.h
  *
- *  $Id: sqlo.h,v 1.22.2.12 2011/02/14 11:48:50 source Exp $
+ *  $Id$
  *
  *  sql opt graph
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -538,6 +538,7 @@ typedef struct index_choice_s
   char		ic_text_order;
   char		ic_geo_order;
   char		ic_o_string_range_lit; /* 1 if o is known to be a string 2 if literal strings */
+  dk_set_t	ic_inx_sample_cols;
 } index_choice_t;
 
 #define IC_OPT_ITERS 0 /* can change in or rdf inf iters into after test */
@@ -796,6 +797,7 @@ int dfe_is_eq_pred (df_elt_t * pred);
 float sqlo_index_path_cost (dk_set_t path, float * cost_ret, float * card_ret, char * sure_ret);
 data_source_t * sqlg_make_ts (sqlo_t * so, df_elt_t * tb_dfe);
 int dfe_is_o_ro2sq_range (df_elt_t * pred, df_elt_t * tb_dfe, df_elt_t ** o_col_dfe_ret, df_elt_t ** exp_dfe_ret, int * op_ret);
+void sqlg_in_iter_nodes (sqlo_t * so, data_source_t * ts, data_source_t ** head);
 int sqlo_has_col_ref (ST * tree);
 
 float dfe_exp_card (sqlo_t * so, df_elt_t * dfe);
