@@ -1,12 +1,12 @@
 /*
  *  strlike.h
  *
- *  $Id: strlike.h,v 1.2.2.2 2010/09/09 19:19:35 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -32,7 +32,9 @@
 #define LIKE_ARG_WCHAR	2
 #define LIKE_ARG_UTF	3
 
-int cmp_like (const char *string, const char *pattern, collation_t *collation, char escape_char, int strtype, int patterntype );
-unsigned char *nc_strstr (const unsigned char *string1, const unsigned char *string2);
+extern int cmp_like (const char *string, const char *pattern, collation_t *collation, char escape_char, int strtype, int patterntype );
+extern unsigned char *nc_strstr (const unsigned char *string1, const unsigned char *string2);
+extern wchar_t *nc_strstr__wide (const wchar_t *string1, const wchar_t *string2);
+
 
 #endif /* _WI_STRLIKE_H */
