@@ -1,12 +1,12 @@
 /*
  *  xmlenc.h
  *
- *  $Id: xmlenc.h,v 1.6.2.4 2010/10/13 12:12:21 source Exp $
+ *  $Id$
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *
- *  Copyright (C) 1998-2006 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -589,7 +589,7 @@ caddr_t wsse_get_content_val (caddr_t * curr);
 
 xenc_key_t * xenc_key_aes_create (const char * name, int keylen, const char * pwd);
 void xenc_key_remove (xenc_key_t * key, int lock);
-int __xenc_key_dsa_init (char *name, int lock);
+int __xenc_key_dsa_init (char *name, int lock, int num);
 int __xenc_key_dh_init (char *name, int lock);
 
 void xenc_key_3des_init (xenc_key_t * pkey, unsigned char * k1, unsigned char * k2, unsigned char * k3);
