@@ -8,7 +8,7 @@
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
  *  
- *  Copyright (C) 1998-2012 OpenLink Software
+ *  Copyright (C) 1998-2013 OpenLink Software
  *  
  *  This project is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@
  *  
 */
 
+#ifndef HAVE_STPCPY
 char *
 stpcpy (char *dst, const char *src)
 {
@@ -33,3 +34,4 @@ stpcpy (char *dst, const char *src)
     ;
   return --dst;
 }
+#endif
